/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       apropos.cpp                                               *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       16-Mar-2013 12.52.55 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* View apropos information                                                  *
* User can select one to load the associated manual page                    *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <iostream>

#include "apropos.h"
#include "manpage.h"
#include "gman.h"
#include "misc.h"

using namespace std;


GMan *Apropos::gman;

Apropos::Apropos(GMan *g)
{
  cout << "Apropos:\n";

  gman = g;
  window = NULL;
  pixbuf = NULL;
}


Apropos::~Apropos(void)
{
  cout << "~Apropos:\n";

  if (window)
    {
      gtk_widget_destroy ( GTK_WIDGET(window) );
      window = NULL;
    }

  if (pixbuf) g_object_unref(G_OBJECT(pixbuf));
}

void Apropos::close_dialog (GtkWidget *widget, Apropos *ap)
{
  ap->gman->delete_apropos(ap);
}


void Apropos::display_manpage(char *buffer, Apropos *ap)
{
using namespace misc;

char *progname = NULL;     // The program name IN the manpage
char *filename = NULL;     // The name of the manpage file
char *section_num = NULL;  // The section # of the manpage file
char *s, *t;
char temp;

s = t  = buffer;
while ( (*t) && (!isspace(*t)) ) t++;
temp = *t;
*t = 0;
progname = make_string(s);
*t = temp;

if (!progname)
  {
  cout << "FAILED to get manpage name\n";
  Complain *c = new Complain;
  string buf;

  // INFORM THE USER!
  // ================
  buf = "Manpage name not found!";

  c->complain(c->CN_CONTINUE,
  (char *)"APROPOS",
  (char *)buf.c_str(),
  NULL,
  NULL,
  gman);      

  return;
  }


  
if (temp != '\n') // not at EOL
  {
  t++;
  while ( (*t) && (*t != '\n') && isspace(*t) ) t++;

  if (*t == '[') // This may not be present
    {
    // Recover the actual filename
    t++;
    s = t;
    while ( (*t) && (*t != '\n') &&
		      (!isspace(*t)) && (*t != ']') ) t++;
    temp = *t;
    *t = 0;
    filename = make_string(s);
    *t = temp;
    t++;
	  }

    
  if ( (*t) && (*t != '\n') ) // not at EOL
	  {
	  while ( (*t) && (*t != '\n') && isspace(*t) ) t++;

	  if (*t == '(')
	    {
	    // Recover section number
	    t++;
	    s = t;
	    while ( (*t) && (*t != '\n') &&
		      (!isspace(*t)) && (*t != ')') ) t++;
	    temp = *t;
	    *t = 0;
	    section_num = make_string(s);
	    *t = temp;
	    }

	  }
  }
//cout << "Apropos::display_manpage - progname= " << progname << endl;
//cout << "Apropos::display_manpage - section_num= " << section_num << endl;
//if (filename) cout << "Apropos::display_manpage - filename= " << filename << endl;
  
if (section_num)
  {
  if (!(ap->find_in_section((char *)( (filename) ? filename : progname ),
		                        section_num)))
    {

    Complain *c = new Complain;
    string buf;

    // INFORM THE USER!
    // ================
    buf = "Manpage \"";
    buf += progname;
    buf += "\" (Section ";
    buf += section_num;
    buf += ") not found.";

    c->complain(c->CN_CONTINUE,
    (char *)"APROPOS",
    (char *)buf.c_str(),
    NULL,
    NULL,
    gman);      
    }

  free(section_num);   
  }
else
  {
  Complain *c = new Complain;
  string buf;

  // INFORM THE USER!
  // ================
  buf = "Manpage \"";
  buf += progname;
  buf += "\"  not found.";
  buf += "\nNo section number found.";

  c->complain(c->CN_CONTINUE,
  (char *)"APROPOS",
  (char *)buf.c_str(),
  NULL,
  NULL,
  gman);
  }

if (progname) free(progname);
if (filename) free(filename);

}

void Apropos::callback_list_select(GtkWidget *widget, Apropos *ap)
{
GtkTreeModel* model;	
GtkTreeIter iter;
GList *list = NULL, *node;

gchar *buffer;

//cout <<  "Apropos::callback_list_select - ENTERED" << endl;

list = gtk_tree_selection_get_selected_rows(
											gtk_tree_view_get_selection(GTK_TREE_VIEW(ap->view)),
											&model);

//num_selected = g_list_length(list);
// NOTE - this could be made multiple select, but the user may get confused
for (node=list; node; node=node->next)
	{
  buffer = NULL;
	if (gtk_tree_model_get_iter(model, &iter, (GtkTreePath *)node->data))
		{
		gtk_tree_model_get(model, &iter, 
                          0, &buffer,
                          -1);
//    cout <<  "Apropos::callback_list_select - GOT: " << buffer << endl;
    if (buffer)
      {
      ap->display_manpage(buffer, ap);
      g_free(buffer);
      }
    break;
		}
	}
}


// Perform directory search and display all manpages found
gboolean Apropos::search(char *mpage)
{
// cout << "Apropos::search - " << mpage << endl;
  
gboolean found = FALSE;
list<string>::iterator i;


for (i=gman->section_num_list.begin(); i!=gman->section_num_list.end(); ++i)
  {
    
//  cout << "Apropos::search - checking " << (char *)i->c_str() << endl;
    
  if ( (find_in_section(mpage, (char *)i->c_str())) )
    {
//     cout << "Apropos::search - FOUND " << mpage << endl;
    found = TRUE;
    }
  }
return(found);
}


gboolean Apropos::find_in_section(char *mpage, char *s_num )
{
using namespace misc;

DIR    *dp;                   // Directory pointer
struct dirent *direntry;      // This directory entry
string mdir;
char mname[1024];
char *path;
char *c;
gboolean found = FALSE;
  
// cout << "Apropos::find_in_section - ENTERED" << endl;
  
GList *manlist = gman->manpath;

  
while (manlist && found==FALSE)
  {
  path = (char *)manlist->data;

  mdir =  path;
  mdir += "/man";
  mdir += s_num;
      
    
  dp = opendir(mdir.c_str());
  if (dp)
	  {
	  while ( (direntry = readdir(dp)) )
	    {
      if ( (STREQ(direntry->d_name, (char *)"." )) ||
          (STREQ(direntry->d_name, (char *)"..")) ) continue;

      strcpy(mname, direntry->d_name);
      if (SUFIEQ(mname, (char *)".gz"))
		    {
		    mname[strlen(mname) - 3] = 0;
		    }
      else if (SUFIEQ(mname, (char *)".bz2"))
		    {
		    mname[strlen(mname) - 4] = 0;
		    }

      c = &mname[strlen(mname) - 1]; // last char
      while (c > mname)
		    {
        if (*c == '.')
		      {
		      *c = 0;
		      break;
		      }
		    c--;
		    }

      if ( STRIEQ(mname, mpage) )  // whatever the case
		    {
		    // FOUND IT
//        cout << "Apropos::find_in_section - FOUND IT" << endl;

        mdir += "/";
        mdir += direntry->d_name;

//        cout << "Apropos::find_in_section - " << mdir << endl;
             
		    ManPage *manpage = new ManPage(gman);
		    gman->manpages = g_list_append(gman->manpages, manpage);
		    manpage->load_manpage(path, (char *)mdir.c_str(), mpage);
		    found = TRUE;
		    }
	    }

	  closedir(dp);
	  }

  manlist = manlist->next;
  }
return(found);
}


void Apropos::apropos(char *str)
{
using namespace misc;

string ap;

string command = "apropos ";
command +=  str;  
  
// Create list here
ap = get_file(command);
create_list((char *)ap.c_str());

create_display(str);
}


void Apropos::create_list(char *buffer)
{
char *t, *s, *p;
bool ok_flag;
char *end = &buffer[strlen(buffer)];
string section;
  
// cout << "Apropos::create_list - ENTERED" << endl;
  
ap_list.clear();

if (!buffer) return;

  
t = s = buffer;

while (*t && t < end)
  {
  ok_flag = FALSE;
  p = s;
  while (*t && (t<end) && (*t!='\n') ) t++;
	while (isgraph(*p)) p++;
	while (*p == ' ') p++;

    // NOTE: "(section)" required for Debian's   nnnnn (section)
    //       "[]"     (section) required for Fedora's   nnnnn [] (but NOT section==rpm!)

  if ( (*p == '[') && (*(p+1) == ']') )
    {
    ok_flag = TRUE;
    }
  else if (*p == '(')
    {
    p++;
    section.clear();
    while ((p < t) && (*p != ')'))
      {
      section += *p++;
      }
		if (section != "rpm") ok_flag = TRUE;
		}


  if (ok_flag && (t>s))
    {
	  *t = 0;
    ap_list.push_back(s);
    }
  s = ++t;
  }

if (ap_list.empty())
  {
  // NOTHING IN LIST!
  ap_list.push_back("\n   << NOTHING FOUND >>   ");
  }
}



void  Apropos::create_display(char *str)
{
  GtkWidget *scrolled_window;
  GtkWidget *wid;
  GtkWidget *box, *vbox;
  string title = "APROPOS (choose): ";
  title += str;
	
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_show (window);  // necessary to prevent manpage of top entry appearing
	
  g_signal_connect (G_OBJECT (window), "destroy",
		    G_CALLBACK (close_dialog),
		    this);

  gtk_window_set_title (GTK_WINDOW (window), title.c_str());
  
	gtk_window_set_default_size(GTK_WINDOW (window),
                              gman->prefs.get_apropos_width(),
			                        gman->prefs.get_apropos_height());
                            
  gtk_container_set_border_width(GTK_CONTAINER(window),5);
	
  pixbuf = gtk_widget_render_icon_pixbuf(GTK_WIDGET(window),
				  GTK_STOCK_INDEX,
				  GTK_ICON_SIZE_DIALOG);

  if (pixbuf) gtk_window_set_icon( GTK_WINDOW(window), pixbuf);

	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
	gtk_box_set_homogeneous(GTK_BOX(vbox), FALSE);

  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (window), vbox);


  scrolled_window = gtk_scrolled_window_new(NULL, NULL);

    /* the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
     * GTK_POLICY_AUTOMATIC will automatically decide whether you need
     * scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
     * there.  The first one is the horizontal scrollbar, the second, 
     * the vertical. */
  gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW(scrolled_window),
                                GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );

  gtk_widget_show (scrolled_window);
  gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);

	box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
  gtk_box_set_homogeneous(GTK_BOX(box), FALSE);
	
  //gtk_widget_show (box);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window),
					box);
  
  wid = create_view();
  gtk_widget_show (wid);
  gtk_container_add(GTK_CONTAINER(box), wid);
  add_list_to_view();
  
	box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);

  //gtk_widget_show (box);

  gtk_box_pack_end (GTK_BOX(vbox), box, FALSE, FALSE, 0);

  // --- Create the "Close" button
  wid = gtk_button_new_with_label ("Close");

  // --- Need to close the window if they press "Close"
  g_signal_connect (G_OBJECT (wid), "clicked",
		    G_CALLBACK (close_dialog), this);

  // --- Allow it to be the default button
  gtk_widget_set_can_default(GTK_WIDGET(wid), TRUE); //GTK_WIDGET_SET_FLAGS (wid, GTK_CAN_DEFAULT);

  // --- Add the button to the dialog
  gtk_box_pack_end(GTK_BOX(box), wid, FALSE, FALSE, 0);

  // --- Make the button the default button
  gtk_widget_grab_default (wid);

  // --- Make everything visible 
	//gtk_widget_show (wid);
  gtk_widget_show_all(window);
}




void Apropos::add_list_to_view(void)
{
GtkTreeIter iter;  
GtkTreeModel *model;
list<string>::iterator i;
  
model = gtk_tree_view_get_model( GTK_TREE_VIEW(view) );

for (i=ap_list.begin(); i!=ap_list.end(); ++i)
  {
  gtk_list_store_append (GTK_LIST_STORE(model), &iter);
  gtk_list_store_set (GTK_LIST_STORE(model), &iter, 0, (gchar *)i->c_str(), -1);
  }
  
g_signal_connect(G_OBJECT ( gtk_tree_view_get_selection(GTK_TREE_VIEW(view)) ),
								 "changed",
								 G_CALLBACK(callback_list_select), this);
}


GtkWidget *Apropos::create_view(void)
{
GtkCellRenderer *renderer;
GtkTreeViewColumn *column;
GtkTreeModel *model;

model = GTK_TREE_MODEL(gtk_list_store_new(1, G_TYPE_STRING));
  
// create tree view
view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(model));

gtk_tree_view_set_rules_hint(GTK_TREE_VIEW (view), TRUE);
gtk_tree_view_set_search_column(GTK_TREE_VIEW(view), 0);

g_object_unref(model);

// column 1
renderer = gtk_cell_renderer_text_new();
column = gtk_tree_view_column_new_with_attributes("Manpage",
                 renderer,
                 "text",
                 0,
                 NULL);
gtk_tree_view_column_set_resizable(column, TRUE);
gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
//gtk_tree_view_column_set_visible(column, FALSE);

// Make headers invisible
gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);

gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(view)),
														GTK_SELECTION_SINGLE);

return view;
}

