/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       complain.cpp                                              *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       05-Apr-2013 15.24.39 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Warn user and get user input                                              *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <iostream>

#include "complain.h"

using namespace std;



Complain::Complain(void)
{
  cout << "Complain:\n";
}
Complain::~Complain(void)
{
  cout << "~Complain:\n";

  if (pixbuf) g_object_unref(G_OBJECT(pixbuf));
}

void Complain::end_complain (GtkWidget *widget, Complain *c)
{
  //  cout << "end_complain: ENTERED\n";
  gtk_widget_destroy (c->dialog_window);
}

void Complain::clear_complain(GtkWidget *widget, Complain *c)
{
  //  cout << "clear_complain: ENTERED\n";
  gtk_grab_remove (widget);
  delete(c);
}

/********************************************************
 * Complain(n, title, msg)	      	       		*
 * Displays a requester.	                        *
 * (n = 1) gives OK or (n = 2) RETRY and ABORT.		*
 * or (n=-1) gives only CONTINUE gadget.		*
 * (n = 0) gives only ABORT gadget.			*
 * Returns TRUE (OK gadget) or FALSE.			*
 ********************************************************/

// Not sure how to make this return a value
// I think that to make this a general purpose subroutine
// it has to have the left & right functions passed as well.
// What happens about their args?
// This prevents serial programming!  The calling code carries on
// and doesn't stop as far as I can see.
// This code should not only take the focus; it also has to grab
// the execution completely from the calling routine without
// returning until it exits with a mouse button or whatever.

void Complain::complain (int n, char *title, char *msg, 
			 void (*YesFunc)(GtkWidget *widget, Complain *c), 
			 void (*NoFunc)(GtkWidget *widget, Complain *c),
			 gpointer d )
{
    GtkWidget *label;
    GtkWidget *button;

    const char *lb, *rb;

    data = d;

    switch (n)
      {

      case CN_ABORT:
	lb = NULL;
	rb = "ABORT";
	break;

      case CN_CONTINUE:
	lb = "CONTINUE";
	rb = NULL;
	break;

      case CN_OK:
	lb = "OK";
	rb = NULL;
	break;
	
      case CN_CANCEL:
	lb = NULL;
	rb = "CANCEL";
	break;
	
      case CN_OK_CANCEL:
	lb = "OK";
	rb = "CANCEL";
	break;

      case CN_OK_ABORT:
	lb = "OK";
	rb = "ABORT";
	break;

      case CN_RETRY_ABORT:
	lb = "RETRY";
	rb = "ABORT";
	break;

      case CN_RETRY_CANCEL:
	lb = "RETRY";
	rb = "CANCEL";
	break;

      case CN_YES_NO:
      default:
	lb = "YES";
	rb = "NO";
	break;
      }

    /* --- Create a dialog window --- */
    dialog_window = gtk_dialog_new ();

    g_signal_connect (G_OBJECT (dialog_window), "destroy",
              G_CALLBACK(clear_complain),
              this);

    /* --- Set the title and add a border --- */
    gtk_window_set_title (GTK_WINDOW (dialog_window), title);
    gtk_container_set_border_width(GTK_CONTAINER (dialog_window), 0);
    gtk_window_set_modal(GTK_WINDOW(dialog_window), TRUE);


    pixbuf = gtk_widget_render_icon_pixbuf(GTK_WIDGET(dialog_window),
				    GTK_STOCK_DIALOG_QUESTION,
				    GTK_ICON_SIZE_DIALOG);
    if (pixbuf) gtk_window_set_icon( GTK_WINDOW(dialog_window), pixbuf);

    /* --- Create an LEFT button (with the focus if no right button) --- */
    if (lb)
      {
	button = gtk_button_new_with_label ( lb );

	if (YesFunc)
	  {
	    g_signal_connect (G_OBJECT (button), "clicked",
				G_CALLBACK(YesFunc),
				this);
	  }
	else
	  {
	    g_signal_connect (G_OBJECT (button), "clicked",
			    G_CALLBACK(end_complain),
			    this);
	  }

	/* --- Default the "Ok" button --- */
//				GtkWidget *         gtk_dialog_get_action_area          (GtkDialog *dialog);
		gtk_widget_set_can_default(GTK_WIDGET(button), TRUE);
//gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->action_area), 
//			    button, TRUE, TRUE, 0);

gtk_box_pack_start(GTK_BOX (gtk_dialog_get_action_area (GTK_DIALOG(dialog_window))), 
			    button, TRUE, TRUE, 0);
if(!rb)
	  {
	  gtk_widget_set_can_default(GTK_WIDGET(button), TRUE);
	  gtk_widget_grab_default (button);
	  }

	gtk_widget_show (button);
      }

    /* --- Create a RIGHT button with the focus --- */
    if (rb)
      {
	button = gtk_button_new_with_label ( rb );

	if (NoFunc)
	  {
	    g_signal_connect (G_OBJECT (button), "clicked",
				G_CALLBACK(NoFunc),
				this);
	  }
	else
	  {
	    g_signal_connect (G_OBJECT (button), "clicked",
				G_CALLBACK(end_complain),
				this);
	  }

	gtk_widget_set_can_default(GTK_WIDGET(button), TRUE); // GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
//	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->action_area), 
	gtk_box_pack_start(GTK_BOX (gtk_dialog_get_action_area (GTK_DIALOG(dialog_window))), 
              button, TRUE, TRUE, 0);
	gtk_widget_grab_default (button);
	gtk_widget_show (button);
      }


    /* --- Create a descriptive label --- */
    label = gtk_label_new (msg);

    /* --- Put some room around the label text --- */
    gtk_misc_set_padding (GTK_MISC (label), 10, 10);

    /* --- Add label to designated area on dialog --- */
	//GtkWidget *         gtk_dialog_get_content_area         (GtkDialog *dialog);
    gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG (dialog_window))), 
              label, TRUE, TRUE, 0);

    /* --- Show the label --- */
    gtk_widget_show (label);


    /* --- Show the dialog --- */
    gtk_widget_show (dialog_window);

    // Make sure the dialog keeps the focus
    gtk_grab_add(dialog_window);
    gtk_widget_grab_focus(dialog_window);


 }

