/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       edit.cpp                                                  *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       29-Mar-2013 13.04.54 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* The edit module                                                           *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <fstream>
#include <errno.h>
#include <pwd.h>

#include "edit.h"
#include "misc.h"
#include "tfile.h"

using namespace misc;


Edit::~Edit(void)
{
cout << "~Edit:\n";
window = NULL;
if (search)
  {
  search->close_search(NULL, search);
  search = NULL;
  }

undo_list.clear();
redo_list.clear();
	
if (timer_id) g_source_remove(timer_id);
if (manpage) delete_manpage(manpage);
if (pixbuf) g_object_unref(G_OBJECT(pixbuf));
}

// Can we use just the file's pathname? - insteadd of mp...
// We also need sourcebuffer, the actual contents of the file

Edit::Edit(GMan *g, char *pathname, char *sourcebuf)
{	
cout << "Edit:\n";

gman = g;
tbuf = NULL;
tbuf_changed = terminating = undoing= redoing  = replacing = FALSE;
accel_group = NULL;
search = NULL;
clipboard = NULL;
 
this->pathname = pathname;

window = NULL;
manpage = NULL;
num_changes = 0;
old_line_number = -1;
timer_id = 0;
sourcebuffer = sourcebuf;
seconds = gman->prefs.get_autosave_seconds();
create_display();
}

// Called when the window is destroyed
void Edit::callback_destroyed(GtkWidget *widget, Edit *e)
{
e->window = NULL;
if (g_list_find(e->gman->edits, e))
  {    
  e->gman->edits = g_list_remove(e->gman->edits, e);
  }

delete e;
}

// This appears to work OK
void Edit::delete_manpage(ManPage *mp)
{
cout << "Edit::delete_manpage";

if (mp && mp->man_window)
  {
  cout << " - deleting" << endl;
  GtkWidget *w = mp->man_window;
  mp->man_window = NULL;
  gtk_widget_destroy(GTK_WIDGET(w));
  delete mp;
  mp->edit = NULL;
  }
else cout << " - NOT deleting" << endl;
manpage = NULL;

}


void Edit::do_quit(GtkWidget *widget, Edit *e)
{  
if (e->window)
  {
  gtk_widget_destroy( GTK_WIDGET(e->window) );
  }
}


void Edit::save_close_yes(GtkWidget *widget, Complain *c)
{
//cout << "Edit::save_close_yes" << endl;

((Edit *)c->data)->write_source(((Edit *)c->data), TRUE, TRUE);  
((Edit *)c->data)->do_quit(widget, ((Edit *)c->data));
c->end_complain(widget, c);  
}

void Edit::save_close_no(GtkWidget *widget, Complain *c)
{
//cout << " Edit::save_close_no" << endl;
  
((Edit *)c->data)->do_quit(widget, ((Edit *)c->data)); 
c->end_complain(widget, c); 
}

// This gets called if the window is destroyed
void Edit::close_dialog (GtkWidget *widget, Edit *e)
{
cout << "Edit::close_dialog\n";

if (e->tbuf_changed)  
  {
  GtkTextIter start, end;

  // We have to extract tbuf here so that it can be saved if needed    
  gtk_text_buffer_get_bounds(e->tbuf, &start, &end);
  e->sourcebuffer = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(e->tbuf), &start, &end, FALSE);

//  if (e->pathname.empty()) //(!e->filename.length())
//    {
//    // NO FILENAME
//    TFile tf;
//    string home = getenv("HOME");    
//    home += "/gmanpage_save.";
//    e->pathname = *(tf.tname(&home, 3));
//    }

  Complain *c = new Complain;
  string buffer = "File \"";
  buffer += e->pathname;
  buffer += "\" has not been saved.\n";
  buffer += "Do you want it saved?";
      // NOTE: Complain deletes itself, but Yes and No funcs must terminate with
      //               c->end_complain(widget, c);   (widget can be NULL)
      //        Type, Title, String, YesFunc, NoFunc, data
      // ===================================================================

  c->complain(c->CN_YES_NO,
  (char *)"WARNING",
  (char *)buffer.c_str(),
  save_close_yes,
  save_close_no,
  (gpointer)e);    
  }
else
  {
  do_quit(widget, e);
  }
  
//cout << "Edit::close_dialog - DONE e->gman->delete_edit(e);\n";

}


gboolean Edit::autosave(gpointer data) //Edit *e)
{
cout << "Edit::autosave" << endl;

Edit *e = (Edit *)data;
  
if (e->pathname.empty()) // we need a filename
	{
	// SaveAs is probably best for now
	callback_save_as(NULL, e);
	}
else                // we already have a file so overwrite it
	{
	e->write_source(e, TRUE, FALSE);  
	}

if (e->num_changes)  e->num_changes = 0; // do this anyway...

return TRUE;
}

void Edit::restart_timed_autosaves(Edit *e)
{
if (e->timer_id)	g_source_remove(timer_id);
if (e->seconds) e->timer_id = g_timeout_add_seconds(e->seconds, autosave, e);
}

void Edit::autosave_changes(Edit *e)
{
e->autosave(e);
e->restart_timed_autosaves(e);
	
/*
if (e->timer_id)
  {
  g_source_remove(timer_id);
  if (e->seconds) e->timer_id = g_timeout_add_seconds(e->seconds, autosave, e);
  }
*/
	
}











// If a file is loaded, tbuf changes so this starts off already marked!
void Edit::callback_tbuf_changed(GtkWidget *widget, Edit *e)
{
// invalid value - NBG I think!
// When another file gets loaded it should be re-initialized.
// Therefore put this in class Edit
// FIX THIS!

GtkTextIter target_iter;  
gint new_line_number;
gint buffer_x, buffer_y;
bool same_line_number;

cout << "Edit::callback_tbuf_changed - ENTERED" << endl;
if (!e->tbuf_changed)
  {
  string title = "*";
  title += gtk_window_get_title(GTK_WINDOW(e->window));
  gtk_window_set_title(GTK_WINDOW(e->window), title.c_str());
  e->tbuf_changed = TRUE;
  }


// Now if the view window is open, save and re-load it
// There is a problem with multiple actions
// We could perhaps check here if the top line is the same as last timer
// and pass that on to re-load

gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(e->text),
                                                    GTK_TEXT_WINDOW_TEXT,
                                                    0,          //gint window_x,
                                                    0,          //gint window_y,
                                                    &buffer_x,
                                                    &buffer_y);  
gtk_text_view_get_line_at_y(GTK_TEXT_VIEW(e->text),
                            &target_iter,
                            buffer_y,        // gint y,
                            NULL);

// What line is this?
new_line_number = gtk_text_iter_get_line(&target_iter);

same_line_number = (new_line_number==e->old_line_number);
e->old_line_number = new_line_number;

if (e->manpage) // If there is a preview window
  {
  e->write_source(e, TRUE, FALSE);
  e->manpage->reload(NULL, e->manpage, same_line_number);
  }
else
  {
  int changes_max = e->gman->prefs.get_autosave_changes();
  e->num_changes++;

  if (changes_max && e->num_changes >= changes_max) // if changes_max is zero, don't do anything
    {
    // Do autosave
    e->autosave_changes(e);
    }
  }
	
cout << "Edit::callback_tbuf_changed - LEAVING" << endl;
}



// =========
// FILE MENU
// =========


void Edit::save_quit_yes(GtkWidget *widget, Complain *c)
{
cout << "Edit:: save_quit_yes" << endl;

((Edit *)c->data)->write_source(((Edit *)c->data), TRUE, FALSE);
((Edit *)c->data)->do_quit(widget,(Edit *)c->data);
c->end_complain(widget, c);
}
void Edit::save_quit_no(GtkWidget *widget, Complain *c)
{
cout << "Edit:: save_quit_no" << endl;

((Edit *)c->data)->tbuf_changed = FALSE; // prevent another Complain!
((Edit *)c->data)->do_quit(widget,(Edit *)c->data);
c->end_complain(widget, c);
}

// This results in the destroy signal which calls close_dialog
void Edit::quit_edit(GtkWidget *widget, Edit *e)
{
cout << "Edit::quit_edit\n";
  
if (e->tbuf_changed)  
  {
  if (e->pathname.length())          //(e->filename.length())
    {
    // Query save
    Complain *c = new Complain;
    string buffer = "File \"";
    buffer += e->pathname;
    buffer += "\" has not been saved.\n";
    buffer += "Do you want to save it?";

      // NOTE: Complain deletes itself, but Yes and No funcs must terminate with
      //               c->end_complain(widget, c);   (widget can be NULL)
      //        Type, Title, String, YesFunc, NoFunc, data
      // ===================================================================

    c->complain(c->CN_YES_NO,
    (char *)"Edit",
    (char *)buffer.c_str(),
    e->save_quit_yes,
    e->save_quit_no,
    (gpointer)e);
    }
  else
    {

    // NO FILENAME
//    TFile tf;
//    string home = getenv("HOME");    
//    home += "/gmanpage_save.";
//    e->pathname = *(tf.tname(&home, 3));
      
// Can we do a save_as?
    e->terminating = TRUE;           // PASS this on to save_as
    e->callback_save_as(widget, e);
    }
  }
else do_quit(widget, e);

}


void Edit::write_source(Edit *e, bool overwrite, bool got_source)
{
GtkTextIter start, end;
gchar *text = NULL; // must be freed with g_free later if not NULL
fstream ofp;

//  cout << "Edit::write_source " << e->pathname << "\n";

gtk_widget_set_sensitive( e->view_menuitem, TRUE );

if (overwrite)
  {
  ofp.open(e->pathname.c_str(), ios::out | ios::binary | ios::trunc );
  }
else ofp.open(e->pathname.c_str(), ios::out | ios::binary );

if ( ofp.is_open() )
  {
  if (!got_source)
    {
    // If not got source, text MUST BE FREED
    gtk_text_buffer_get_bounds(e->tbuf, &start, &end);
    text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(e->tbuf), &start, &end, FALSE);
    }
  else text = (gchar *)sourcebuffer.c_str();
    
  if (text)
	  {
	  if ( SUFEQ((char *)e->pathname.c_str(), (char *)".gz" ) ||
	       SUFEQ((char *)e->pathname.c_str(), (char *)".bz2" ) )
	    {
      // Compression required so copy buffer to a temporary file
      TFile tf;
      fstream tfp;
      string pathname = "/tmp/gmanpage";
      string t = *(tf.tfile(&pathname, &tfp, 3));

      if (t.length()) // if a file was opened, its name is t otherwise NULL
        {
        // Write buffer to the file named t
        string suffix = ".comp";
        string command;
        string compressed_filename = *(tf.tname(&t, 3, &suffix));
        char *c = text;

        while (*c)
          {
          tfp.put(*c);  // should check for errors...
          c++;
          }
        tfp.close();

		    // OK, should have a file named t
		    //     and a temporary filename in compressed_filename.
		    //     Now do the compression.

		    if ( SUFEQ((char *)e->pathname.c_str(), (char *)".gz" ) )
		      {
		      // gzip compressed file
      
		      // e.g. command could be:
		      //     ls -laR $HOME | gzip > homedir.gz
		      // This pipes a listing of the user's home directory to gzip
		      // which sends its output to homedir.gz in the current directory.
          command = "cat ";
          command += t;
          command += " | gzip > ";
          command += compressed_filename;
            
		      }
		    else if ( SUFEQ((char *)e->pathname.c_str(), (char *)".bz2" ) )
		      {
		      // bzip2 compressed file
      
		      // e.g. command could be:
		      //     ls -laR $HOME | bzip2 > homedir.bz2
		      // This pipes a listing of the user's home directory to bzip2
		      // which sends its output to homedir.bz2 in the current directory.
		      // -f with bzip2 over-writes a pre-existing file of the same name.

           
          command = "cat ";
          command += t;
          command += " | bzip2 > ";
          command += compressed_filename;
		      }

		    // Execute the command
		    if (system((char *)command.c_str()) == -1)
		      {
		      // ERROR
          cout << "Edit::write_source - ERROR: system() returned -1" << endl;
		      }

		    // Now copy compressed_filename.c_str() to ofp
		    // We can re-use tfp.

		    tfp.open( compressed_filename.c_str(), ios::in  | ios::binary );
		    if (! tf.do_copy(&ofp, &tfp))
		      {
		      // copy failed, so inform user
		      Complain *c = new Complain;

		      string buf = "Failed to copy \"";
		      buf += compressed_filename.c_str();
		      buf += "\" to \n\"";
		      buf += e->pathname.c_str();
		      buf += "\"\n";
		      buf +=strerror(errno);

		      cout << "Edit::write_source\n  " << buf << endl;

		      c->complain(c->CN_CONTINUE,
				  (char *)"Edit::write_source",
				  (char *)buf.c_str(),
				  NULL,
				  NULL,
				  (gpointer)e);

		      }
        else if (!got_source)  // i.e. window has been destroyed
          {
          // Correct the window title
          string buffer = "EDIT: ";
          buffer += e->pathname;
          gtk_window_set_title(GTK_WINDOW(e->window), buffer.c_str());
          e->tbuf_changed = FALSE;
          }
		    // clean up

		    // close the compressed file
		    tfp.close();

		    // delete the compressed file if present
		    unlink(compressed_filename.c_str());

		    // delete the uncompressed file if present
		    unlink(t.c_str());
		    }
	    }
	  else  // NOT COMPRESSED
	    {
      // assume uncompressed file, straight save

      char *c = text;
      while (*c)
		    {
		    ofp.put(*c);  // Should check for failure
		    c++;
		    }
    
      if (!got_source)  // i.e. window has been destroyed
        {
        string buf = "EDIT: ";
        buf += e->pathname;
        gtk_window_set_title(GTK_WINDOW(e->window), buf.c_str());
        e->tbuf_changed = FALSE;
        }
	    }
	  if (!got_source) g_free(text);
	  }
  else
	  {
	  Complain *c = new Complain;

	  string buf = "\"";
	  buf += e->pathname.c_str();
	  buf += "\" - NO BUFFER, NO TEXT!";

	  cout << "Edit::write_source  " << buf << endl;

	  c->complain(c->CN_CONTINUE,
		  (char *)"Edit::write_source",
		  (char *)buf.c_str(),
		  NULL,
		  NULL,
		  (gpointer)e);
	  }
  }
else
  {
  Complain *c = new Complain;

  string buf = "Can't open \"";
  buf += e->pathname.c_str();
  buf += "\" for write\n";
  buf += strerror(errno);

  c->complain(c->CN_CONTINUE,
  (char *)"Edit::write_source",
  (char *)buf.c_str(),
  NULL,
  NULL,
  (gpointer)e);
  }

ofp.close();
}

void Edit::save_as_yes(GtkWidget *widget, Complain *c)
{
//  cout << "Edit::save_as_yes\n";
((Edit *)c->data)->pathname = ((Edit *)c->data)->temp_pathname;
((Edit *)c->data)->write_source((Edit *)c->data, TRUE, FALSE);

c->end_complain(widget, c);
}


void Edit::save_as_file(GtkWidget *widget, string pathname, Edit *e)
{
string buffer;

cout << "Edit::save_as_file - pathname = " << pathname << endl;

e->temp_pathname = pathname;


if (isfile((char *)pathname.c_str()))
  {
  // Query overwrite

  Complain *c = new Complain;
  buffer = "File \"";
  buffer += pathname;
  buffer += "\" aleady exists.\n";
  buffer += "Do you want to over-write it?";

  // NOTE: Complain deletes itself, but Yes and No funcs must terminate with
  //               c->end_complain(widget, c);   (widget can be NULL)
  //        Type, Title, String, YesFunc, NoFunc, data
  // ===================================================================

  c->complain(c->CN_YES_NO,
		        (char *)"Edit",
		        (char *)buffer.c_str(),
		        e->save_as_yes,
		        NULL,
		      (gpointer)e);
  }
else if ( isname( (char *)pathname.c_str()) )  // ANYTHING ELSE EXISTS, ERROR 
  {
  Complain *c = new Complain;
  buffer = "System name \"";;
  buffer += pathname;
  buffer += "\" exists and is not a regular file.\n";
  buffer += "Can't over-write it.";

  cout << "Edit::save_as_ok - " << buffer << endl;

  c->complain(c->CN_CANCEL,
		        (char *)"Edit",
		        (char *)buffer.c_str(),
		        NULL,
		        NULL,
		        (gpointer)e);
  }

e->pathname = pathname;
e->write_source(e, FALSE, FALSE);   
buffer = "EDIT: ";
buffer += e->pathname;
if (!e->terminating) gtk_window_set_title (GTK_WINDOW (e->window), buffer.c_str());
else e->quit_edit(widget, e);
}



void Edit::callback_save_as(GtkWidget *widget, Edit *e)
{
GtkWidget *dialog = gtk_file_chooser_dialog_new ("Save File AS",
                                      GTK_WINDOW(e->window),
                                      GTK_FILE_CHOOSER_ACTION_SAVE,
                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
                                      NULL);


//
//gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
//
//
//
//
//if (new_document)
//  gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), "Untitled document");
//else
//  gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog), "Existing Name");
//

// This isn't perfect - select a parent folder before create folder is displayed
// Can't select recently used folder for it to appear
gtk_file_chooser_get_create_folders (GTK_FILE_CHOOSER (dialog));
gtk_file_chooser_set_create_folders (GTK_FILE_CHOOSER (dialog), TRUE);


if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
  {
	char *pathname = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));

	cout << "Edit::callback_save_as - pathname= " << pathname << endl;
	e->save_as_file(widget, pathname, e); 

	g_free (pathname);
  }
	
else cout << "Edit::callback_save_as - NO RESULT" << endl;

gtk_widget_destroy (dialog);
}



	
void Edit::callback_save(GtkWidget *widget, Edit *e)
{
//  cout << "Edit::callback_save " << e->pathname << endl;

e->write_source(e, TRUE, FALSE);

}



void Edit::callback_view(GtkWidget *widget, Edit *e)
{
GtkTextIter start, end;

cout << "Edit::callback_view" << endl;

if (e->manpage)
  {
  cout << "Edit::callback_view - view already loaded" << endl; 
    return;
  }

  
ManPage *manpage = new ManPage(e->gman);
char *pname = (char *)e->pathname.c_str();              //(char *)(*tfname ).c_str();

char *p = &pname[strlen(pname)];

while (p > pname)  // Get just the name into p
  {
	if ( *(p-1) == '/') break;
	p--;
  }

e->manpage = manpage;

manpage->load_manpage((char *)"", pname, p, e);

}


// =============
// LOADING FILES
// =============

void Edit::load_over_yes(GtkWidget *widget, Complain *c)
{
Edit *e = (Edit *)c->data;
string buffer;
  
//cout << "Edit::load_over_yes\n";

// Put the file into the exising buffer: this clears the buffer first

gtk_text_buffer_set_text(GTK_TEXT_BUFFER( e->tbuf ),
			   e->sourcebuffer.c_str(),
			   -1);
// Fix the window title
 
buffer = "EDIT: ";
buffer += e->pathname;
gtk_window_set_title (GTK_WINDOW (e->window), buffer.c_str());
e->tbuf_changed = FALSE;
e->num_changes = 0;
e->old_line_number = -1;
c->end_complain(widget, c);
}

void Edit::load_into_new_window(GtkWidget *widget, Complain *c)
{
//  cout << "Edit::load_into_new_window\n";

Edit *e =  (Edit *)c->data;

Edit *edit = new Edit(e->gman,
			(char *)e->pathname.c_str(),
			(char *)e->sourcebuffer.c_str() );
e->gman->edits = g_list_append(e->gman->edits, edit);
  
c->end_complain(widget, c);
}


void Edit::load_selected_file(GtkWidget *widget, string pathname, Edit *e)
{
string command;

if (isfile((char *)pathname.c_str()))
  {
  e->pathname = pathname;

  if ( SUFEQ((char *)pathname.c_str(), (char *)".gz" ) )
	  {
    command = "gunzip -c ";
    command += pathname;
	  }
  else if ( SUFEQ((char *)pathname.c_str(), (char *)".bz2" ) )
	  {
    command = "gbunzip2 -c ";
    command += pathname;
	  }
  else
	  {
    command = "cat ";
    command += pathname;
	  }

  e->sourcebuffer = get_file(command);

  if (gtk_text_buffer_get_char_count(e->tbuf))
	  {
	  Complain *c = new Complain;
	  string buffer = "Do you want to over-write the existing file?";

	  // Over-write or new window

	  c->complain(c->CN_YES_NO,
		      (char *)"Edit",
		      (char *)buffer.c_str(),
		      e->load_over_yes,
		      e->load_into_new_window,
		      (gpointer)e);
	  }
  else
	  {
	  // Otherwise do it
    string buffer;

	  gtk_text_buffer_set_text(GTK_TEXT_BUFFER( e->tbuf ),
				   e->sourcebuffer.c_str(),
				   -1);
 
    buffer = "EDIT: ";
    buffer += e->pathname;
	  gtk_window_set_title (GTK_WINDOW (e->window), buffer.c_str());
    e->tbuf_changed = FALSE;
    e->num_changes = 0;
    e->old_line_number = -1;
	  }
  }
else if ( isname((char *)pathname.c_str()) )  // ANYTHING ELSE EXISTS, ERROR 
  {
  Complain *c = new Complain;
  string buffer = "System name \"";;
  buffer += pathname;
  buffer += "\" exists and is not a regular file.\n";
  buffer += "Can't load it.\n";

  cerr << "Edit::load_selected_file \"" << pathname <<
	"\" exists and is not a regular file.\nCan't load it.\n";

  c->complain(c->CN_CANCEL,
		  (char *)"Edit",
		  (char *)buffer.c_str(),
		  NULL,
		  NULL,
		  (gpointer)e);
  }
else
  {
  Complain *c = new Complain;
  string buffer = "System name \"";;
  buffer += pathname;
  buffer += "\" doesn't exist.\n";
  buffer += "Can't load it.\n";

  cerr << "Edit::save_as_ok - name \"" << pathname <<
	"\" doesn't exist, can't load it\n";

  c->complain(c->CN_CANCEL,
		  (char *)"Edit",
		  (char *)buffer.c_str(),
		  NULL,
		  NULL,
		  (gpointer)e);
  }
}



void Edit::callback_load_file(GtkWidget *widget, Edit *e)
{  
cout << "Edit::callback_load_file" << endl;

GtkWidget *dialog = gtk_file_chooser_dialog_new ("LOAD",
                                      GTK_WINDOW(e->window),
                                      GTK_FILE_CHOOSER_ACTION_OPEN,
                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                      NULL);


if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
	{
  char *pathname;
	string name;
	char *c;
		
	pathname = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
	c = &pathname[strlen(pathname) - 1];

	while ( (c >= pathname) && (*c != '/') )
		{
		name = *c-- + name;
		}
	cout << "Edit::callback_load_file - Pathname= " << pathname << endl;
	cout << "                               name= " << name << endl;


  e->load_selected_file(widget, pathname, e);
	g_free (pathname);
  }
	
else cout << "Edit::callback_load_file - NO RESULT" << endl;


gtk_widget_destroy (dialog);
}




void Edit::callback_clear(GtkWidget *widget, Edit *e)
{
//cout << "Edit::callback_clear\n";

gtk_text_buffer_set_text(GTK_TEXT_BUFFER( e->tbuf ),
			   "",
			   -1);
if (e->tbuf_changed)
  {
  string title = "EDIT: ";
  gtk_window_set_title(GTK_WINDOW(e->window), title.c_str());
  e->tbuf_changed = FALSE;
  }

}





// =================================================
//                     EDIT MENU
// =================================================

void Edit::inspect_undo_list(void)
{

list< Undo >::iterator i;
int p;
Undo u;

cout << "Edit::inspect_undo_list - size:-" << undo_list.size() << endl;
for (p=1, i=undo_list.begin(); i!=undo_list.end(); ++i, ++p)
	{
	u = *i;
	cout << p << ": UNDO action= ";
	if (u.action & UD_INSERT) cout << "UD_INSERT ";
	if (u.action & UD_DELETE) cout << "UD_DELETE ";
	if (u.action & UD_REPLACE) cout << "UD_REPLACE";
	cout << endl;
	cout << "  : index= " << u.index << endl;
	cout << "  : text = " << u.txt << endl;
	}
}


void Edit::inspect_redo_list(void)
{

list< Undo >::iterator i;
int p;
Undo u;

cout << "Edit::inspect_redo_list - size:-" << undo_list.size() << endl;
for (p=1, i=redo_list.begin(); i!=redo_list.end(); ++i, ++p)
	{
	u = *i;
	cout << p << ": REDO action= ";
	if (u.action & UD_INSERT) cout << "UD_INSERT ";
	if (u.action & UD_DELETE) cout << "UD_DELETE ";
	if (u.action & UD_REPLACE) cout << "UD_REPLACE";
	cout << endl;
	cout << "  : index= " << u.index << endl;
	cout << "  : text = " << u.txt << endl;
	}
}



// PERFECT! The start offset is always the lower value
void Edit::callback_delete_range(GtkTextBuffer *tbuf, GtkTextIter *start, GtkTextIter *end, Edit *e)
{
cout << ">>>>Edit::callback_delete_range 1 - replacing= " << e->replacing << endl;

e->inspect_undo_list();
e->inspect_redo_list();
	
if ((!e->undoing) && (!e->redoing))
  {
	
  gchar *txt = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(tbuf), start, end, FALSE);
//  cout << "txt= " << txt << endl;
//  cout << "start offset= " << gtk_text_iter_get_offset(start) << endl;
//  cout << "end   offset= " << gtk_text_iter_get_offset(end) << endl;

  Undo u;
  u.action = (e->replacing) ? (UD_INSERT | UD_REPLACE) :  UD_INSERT;   //UD_INSERT; 
  u.index = gtk_text_iter_get_offset(start);
  u.txt = txt;

  e->undo_list.push_back(u);
  gtk_widget_set_sensitive(GTK_WIDGET(e->undo_menuitem), TRUE);
  }
	
}




// ------------------------------------------------------------------------------
// This needs to record what change took place somehow and put it in the undo list
// ------------------------------------------------------------------------------
void Edit::callback_insert_text(GtkTextBuffer *tbuf, GtkTextIter *iter, gchar *txt, gint len, Edit *e)
{

cout << "Edit::callback_insert_text 1- replacing= " << e->replacing  << endl;

if ((!e->undoing) && (!e->redoing))
  {
  cout << "Edit::callback_insert_text 2- replacing= " << e->replacing  << endl;
//  cout << "txt= " << (char *)txt << " : offset= " << gtk_text_iter_get_offset(iter) << endl;
//  cout << "start offset= " << gtk_text_iter_get_offset(iter) << endl;
//  cout << "end   offset= " << gtk_text_iter_get_offset(iter) + strlen((char *)txt) << endl;

  Undo u;
  u.action = (e->replacing) ? (UD_DELETE | UD_REPLACE) : UD_DELETE;

  u.index = gtk_text_iter_get_offset(iter);
  u.txt = txt;
	e->replacing = FALSE;  // insert always comes AFTER delete and appears first at the end of the undo list

  e->undo_list.push_back(u);
  gtk_widget_set_sensitive(GTK_WIDGET(e->undo_menuitem), TRUE);
  }
	
}



/***************************************
 *                 UNDO                *
 ***************************************/
// It would be easier to understand if this didn't call itself

void Edit::callback_undo(GtkWidget *widget, Edit *e)
{
Undo u;

GtkTextIter start;

cout << "\n>>>>Edit::callback_undo - ENTERED" << endl;

e->inspect_undo_list();
e->inspect_redo_list();

//e->undo_redo(widget, e, TRUE);

e->undoing = TRUE;

if (!e->undo_list.empty())
	{
	u = e->undo_list.back();
	}
else
	{
	//ERROR - shouldn't have got here
	e->undoing = FALSE;

	if (e->redo_list.empty()) gtk_widget_set_sensitive(GTK_WIDGET(e->redo_menuitem), FALSE);
	else gtk_widget_set_sensitive(GTK_WIDGET(e->redo_menuitem), TRUE);
	if (e->undo_list.empty()) gtk_widget_set_sensitive(GTK_WIDGET(e->undo_menuitem), FALSE);
	else  gtk_widget_set_sensitive(GTK_WIDGET(e->undo_menuitem), TRUE);
	return;
	}
	
e->undo_list.pop_back(); // There is at least 1 item in the list
	
if (u.action & UD_REPLACE)
	{
	e->replacing  = TRUE;
	
	if  (e->undo_list.size() < 1)
		{
		//ERROR - should be a second item in the list
		e->replacing  = FALSE;
		e->undoing = FALSE;
		cout << "Edit::callback_undo - ERROR, replace needs a second item in the list, fixing list" << endl;
		return;		
		}
	}
gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(e->tbuf),
                                     &start,
                                     u.index);

if (u.action & UD_INSERT)
	{
	cout << "Edit::undo - INSERTING" << endl;
	gtk_text_buffer_insert(GTK_TEXT_BUFFER(e->tbuf), &start, u.txt.c_str(), -1);
	u.action = UD_DELETE;
	e->redo_list.push_back(u);
	}
		
else if (u.action & UD_DELETE)
	{
	GtkTextIter end;
	cout << "Edit::undo - DELETING" << endl;
	gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(e->tbuf),
                                     &end,
                                     u.index + u.txt.length());

	gtk_text_buffer_delete(GTK_TEXT_BUFFER(e->tbuf), &start, &end);
	u.action = UD_INSERT;
	e->redo_list.push_back(u);	
	}

else
	{
	// ERROR
	}

	
// Make sure this gets done
while (gtk_events_pending ())
gtk_main_iteration ();


if (e->replacing) // if another action is required
	{
	u = e->undo_list.back(); // Already checked that there is another item in the list
	
	e->undo_list.pop_back(); // There is at least 1 item in the list

	gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(e->tbuf),
                                     &start,
                                     u.index);

	if (u.action & UD_INSERT)
		{
		cout << "Edit::undo - INSERTING" << endl;
		gtk_text_buffer_insert(GTK_TEXT_BUFFER(e->tbuf), &start, u.txt.c_str(), -1);
		u.action = UD_DELETE | UD_REPLACE;
		e->redo_list.push_back(u);
		}
		
	else if (u.action & UD_DELETE)
		{
		GtkTextIter end;
		cout << "Edit::undo - DELETING" << endl;
		gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(e->tbuf),
                                     &end,
                                     u.index + u.txt.length());

		gtk_text_buffer_delete(GTK_TEXT_BUFFER(e->tbuf), &start, &end);
		u.action = UD_INSERT | UD_REPLACE;
		e->redo_list.push_back(u);	
		}

	else
		{
		// ERROR
		}
	e->replacing = FALSE;	
	while (gtk_events_pending ())
  gtk_main_iteration ();

	}

	
if (e->redo_list.empty()) gtk_widget_set_sensitive(GTK_WIDGET(e->redo_menuitem), FALSE);
else gtk_widget_set_sensitive(GTK_WIDGET(e->redo_menuitem), TRUE);
if (e->undo_list.empty()) gtk_widget_set_sensitive(GTK_WIDGET(e->undo_menuitem), FALSE);
else  gtk_widget_set_sensitive(GTK_WIDGET(e->undo_menuitem), TRUE);

e->undoing = FALSE;
cout << "Edit::undo - LEAVING" << endl;
e->inspect_undo_list();  	
e->inspect_redo_list();  

}






/***************************************
 *                 REDO                *
 ***************************************/
// It would be easier to understand if this didn't call itself

void Edit::callback_redo(GtkWidget *widget, Edit *e)
{
Undo u;
GtkTextIter start;


cout << "\n>>>>Edit::callback_redo - ENTERED" << endl;

e->inspect_undo_list();
e->inspect_redo_list();

//e->undo_redo(widget, e, TRUE);

e->redoing = TRUE;
	
if (!e->redo_list.empty())
	{
	u = e->redo_list.back();
	}
else
	{
	//ERROR - shouldn't have got here
	if (e->redo_list.empty()) gtk_widget_set_sensitive(GTK_WIDGET(e->redo_menuitem), FALSE);
	else gtk_widget_set_sensitive(GTK_WIDGET(e->redo_menuitem), TRUE);
	if (e->undo_list.empty()) gtk_widget_set_sensitive(GTK_WIDGET(e->undo_menuitem), FALSE);
	else  gtk_widget_set_sensitive(GTK_WIDGET(e->undo_menuitem), TRUE);
	return;
	}
	
e->redo_list.pop_back(); // There is at least 1 item in the list
	
if (u.action & UD_REPLACE)
	{
	e->replacing  = TRUE;
	
	if  (e->redo_list.size() < 1)
		{
		//ERROR - should be a second iitem in the list
		e->replacing  = FALSE;			
		e->redoing = FALSE;
		cout << "Edit::callback_redo - ERROR, replace needs a second item in the list, fixing list" << endl;
		return;		
		}
	}
	
gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(e->tbuf),
                                     &start,
                                     u.index);

if (u.action & UD_INSERT)
	{
	cout << "Edit::undo - INSERTING" << endl;
	gtk_text_buffer_insert(GTK_TEXT_BUFFER(e->tbuf), &start, u.txt.c_str(), -1);
	u.action = UD_DELETE;
	e->undo_list.push_back(u);
	}
		
else if (u.action & UD_DELETE)
	{
	GtkTextIter end;
	cout << "Edit::undo - DELETING" << endl;
	gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(e->tbuf),
                                     &end,
                                     u.index + u.txt.length());

	gtk_text_buffer_delete(GTK_TEXT_BUFFER(e->tbuf), &start, &end);
	u.action = UD_INSERT;
	e->undo_list.push_back(u);	
	}

else
	{
	// ERROR
	}

	
// Make sure this gets done
while (gtk_events_pending ())
gtk_main_iteration ();


if (e->replacing) // if another action is required
	{
	u = e->redo_list.back(); // Already checked that there is another item in the list
	
	e->redo_list.pop_back(); // There is at least 1 item in the list

	gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(e->tbuf),
                                     &start,
                                     u.index);

	if (u.action & UD_INSERT)
		{
		cout << "Edit::undo - INSERTING" << endl;
		gtk_text_buffer_insert(GTK_TEXT_BUFFER(e->tbuf), &start, u.txt.c_str(), -1);
		u.action = UD_DELETE | UD_REPLACE;
		e->undo_list.push_back(u);
		}
		
	else if (u.action & UD_DELETE)
		{
		GtkTextIter end;
		cout << "Edit::undo - DELETING" << endl;
		gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(e->tbuf),
                                     &end,
                                     u.index + u.txt.length());

		gtk_text_buffer_delete(GTK_TEXT_BUFFER(e->tbuf), &start, &end);
		u.action = UD_INSERT | UD_REPLACE;
		e->undo_list.push_back(u);	
		}

	else
		{
		// ERROR
		}
	e->replacing = FALSE;
		
	while (gtk_events_pending ())
  gtk_main_iteration ();

	}

	
if (e->redo_list.empty()) gtk_widget_set_sensitive(GTK_WIDGET(e->redo_menuitem), FALSE);
else gtk_widget_set_sensitive(GTK_WIDGET(e->redo_menuitem), TRUE);
if (e->undo_list.empty()) gtk_widget_set_sensitive(GTK_WIDGET(e->undo_menuitem), FALSE);
else  gtk_widget_set_sensitive(GTK_WIDGET(e->undo_menuitem), TRUE);


e->redoing = FALSE;

cout << "Edit::redo - LEAVING" << endl;
e->inspect_undo_list();  	
e->inspect_redo_list();  

}














void Edit::callback_copy(GtkWidget *widget, Edit *e)
{
  GtkTextIter insert_iter;
  GtkTextMark *insert;

//  cout << "Edit::callback_copy_to_clipboard\n";

  if (! e->clipboard)
    {
      e->clipboard = gtk_widget_get_clipboard(GTK_WIDGET(e->text),
					      GDK_SELECTION_CLIPBOARD );
    }

  // Copy to clipboard
  gtk_text_buffer_copy_clipboard(e->tbuf, e->clipboard);

  // De-select the text
  insert = gtk_text_buffer_get_insert(e->tbuf);
  gtk_text_buffer_get_iter_at_mark(e->tbuf, &insert_iter, insert);
  gtk_text_buffer_select_range(e->tbuf,
			       &insert_iter,
			       &insert_iter);

}

void Edit::callback_cut(GtkWidget *widget, Edit *e)
{

//  cout << "Edit::callback_cut_to_clipboard\n";

  if (! e->clipboard)
    {

      e->clipboard = gtk_widget_get_clipboard(GTK_WIDGET(e->text),
					      GDK_SELECTION_CLIPBOARD );
    }

  gtk_text_buffer_cut_clipboard(e->tbuf,
				e->clipboard,
                                TRUE);
}

void Edit::callback_delete(GtkWidget *widget, Edit *e)
{
  GtkTextIter insert_iter;
  GtkTextIter selection_iter;
  GtkTextMark *insert;
  GtkTextMark *selection;

//  cout <<  "Edit::callback_delete\n";


  insert = gtk_text_buffer_get_insert(e->tbuf);
  selection = gtk_text_buffer_get_selection_bound(e->tbuf);

  gtk_text_buffer_get_iter_at_mark(e->tbuf, &insert_iter, insert);
  gtk_text_buffer_get_iter_at_mark(e->tbuf, &selection_iter, selection);
  gtk_text_buffer_delete(e->tbuf,
			 &insert_iter,
			 &selection_iter);
}


void Edit::callback_paste(GtkWidget *widget, Edit *e)
{
//  cout << "Edit::callback_paste\n";

// TEST if any text is marked...
// If so, delete it - normally paste would delete it but somehow it doesn't...
if (gtk_text_buffer_get_has_selection(GTK_TEXT_BUFFER(e->tbuf)))
  {
  e->callback_delete(widget, e);
  }

if (! e->clipboard)
  {
  e->clipboard = gtk_widget_get_clipboard(GTK_WIDGET(e->text),
					      GDK_SELECTION_CLIPBOARD );
  }

gtk_text_buffer_paste_clipboard(e->tbuf,
				  e->clipboard,
				  NULL,
				  TRUE);

}

void Edit::callback_select_all(GtkWidget *widget, Edit *e)
{
//cout <<  "Edit::callback_select_all" << endl;
  
GtkTextIter start;
GtkTextIter end;
  
gtk_text_buffer_get_bounds(e->tbuf, &start, &end);
 
gtk_text_buffer_move_mark_by_name(e->tbuf, "selection_bound", &start);
gtk_text_buffer_move_mark_by_name(e->tbuf, "insert", &end);
}



// ===========
// SEARCH MENU
// ===========

void Edit::callback_find(GtkWidget *widget, Edit *e)
{
cout <<  "Edit::callback_find\n";
e->search_initialised = FALSE;
if (!e->search)
  {
  e->search = new Search(FALSE, e);
  }
else cout << "Edit::callback_find - search already in use\n";
}

void Edit::callback_replace(GtkWidget *widget, Edit *e)
{
 cout <<  "Edit::callback_replace\n";

 if (!e->search)
	{
	e->search = new Search(TRUE, e);
	}
else cout << "Edit::callback_replace - search already in use\n";
}

void Edit::callback_fr_next(GtkWidget *widget, Edit *e)
{
  cout <<  "Edit::callback_fr_next\n";
  if (e->search)
    {
      e->search->callback_next(widget, e->search);
    }
}



// =======
// TOOLBAR
// =======

void Edit::create_toolbar(GtkWidget *vbox)
{
  GtkWidget *toolbar;
  GtkToolItem *toolitem;

  toolbar = gtk_toolbar_new();
  gtk_widget_show(toolbar);
  gtk_box_pack_start(GTK_BOX(vbox),toolbar, FALSE, TRUE, 0);

  toolitem = gtk_tool_button_new_from_stock(GTK_STOCK_CLOSE);
  gtk_widget_show(GTK_WIDGET(toolitem));
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolitem, 0);
  g_signal_connect (G_OBJECT (toolitem), "clicked",
		    G_CALLBACK ( quit_edit ),  this);
  gtk_widget_set_tooltip_text(GTK_WIDGET(toolitem),
      "Close");

  toolitem = gtk_tool_button_new_from_stock(GTK_STOCK_PASTE);
  gtk_widget_show(GTK_WIDGET(toolitem));
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolitem, 0);
  g_signal_connect (G_OBJECT (toolitem), "clicked",
		    G_CALLBACK ( callback_paste ),  this);
  gtk_widget_set_tooltip_text(GTK_WIDGET(toolitem),
      "Paste");


  toolitem = gtk_tool_button_new_from_stock(GTK_STOCK_DELETE);
  gtk_widget_show(GTK_WIDGET(toolitem));
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolitem, 0);
  g_signal_connect (G_OBJECT (toolitem), "clicked",
		    G_CALLBACK ( callback_delete ),  this);
  gtk_widget_set_tooltip_text(GTK_WIDGET(toolitem),
      "Delete");

  toolitem = gtk_tool_button_new_from_stock(GTK_STOCK_CLEAR);
  gtk_widget_show(GTK_WIDGET(toolitem));
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolitem, 0);
  g_signal_connect (G_OBJECT (toolitem), "clicked",
		    G_CALLBACK ( callback_clear ),  this);
  gtk_widget_set_tooltip_text(GTK_WIDGET(toolitem),
      "Clear");

  toolitem = gtk_tool_button_new_from_stock(GTK_STOCK_CUT);
  gtk_widget_show(GTK_WIDGET(toolitem));
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolitem, 0);
  g_signal_connect (G_OBJECT (toolitem), "clicked",
		    G_CALLBACK ( callback_cut ),  this);
  gtk_widget_set_tooltip_text(GTK_WIDGET(toolitem),
      "Cut");


  toolitem = gtk_tool_button_new_from_stock(GTK_STOCK_COPY);
  gtk_widget_show(GTK_WIDGET(toolitem));
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolitem, 0);
  g_signal_connect (G_OBJECT (toolitem), "clicked",
		    G_CALLBACK ( callback_copy ),  this);
  gtk_widget_set_tooltip_text(GTK_WIDGET(toolitem),
      "Copy");

  toolitem = gtk_tool_button_new_from_stock(GTK_STOCK_SAVE_AS);
  gtk_widget_show(GTK_WIDGET(toolitem));
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolitem, 0);
  g_signal_connect (G_OBJECT (toolitem), "clicked",
		    G_CALLBACK ( callback_save_as ),  this);
	gtk_widget_set_tooltip_text(GTK_WIDGET(toolitem),
      "Save As");


  toolitem = gtk_tool_button_new_from_stock(GTK_STOCK_SAVE);
  gtk_widget_show(GTK_WIDGET(toolitem));
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolitem, 0);
  g_signal_connect (G_OBJECT (toolitem), "clicked",
		    G_CALLBACK ( callback_save ),  this);
  gtk_widget_set_tooltip_text(GTK_WIDGET(toolitem),
      "Save");

  toolitem = gtk_tool_button_new_from_stock(GTK_STOCK_EDIT);
  gtk_widget_show(GTK_WIDGET(toolitem));
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolitem, 0);
  g_signal_connect (G_OBJECT (toolitem), "clicked",
		    G_CALLBACK ( callback_load_file ),  this);
  gtk_tool_button_set_label(GTK_TOOL_BUTTON(toolitem), "Open");
  gtk_widget_set_tooltip_text(GTK_WIDGET(toolitem),
      "Open");



  toolitem = gtk_tool_button_new_from_stock(GTK_STOCK_NEW);
  gtk_widget_show(GTK_WIDGET(toolitem));
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolitem, 0);
  g_signal_connect (G_OBJECT (toolitem), "clicked",
		    G_CALLBACK ( gman->callback_new_edit ),  gman);
  gtk_widget_set_tooltip_text(GTK_WIDGET(toolitem),
      "New");


}


void Edit::create_display(void)
{
  using namespace misc;

// cout << "Edit::create_display - ENTERED\n";
 
  GtkWidget *sw;
  GtkWidget *vbox;
  GtkWidget *menu;
  GtkWidget *menubar;
  GtkWidget *menuitem;
  GtkTextIter start;
  string buffer;
  
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_show (window);
  g_signal_connect (G_OBJECT (window), "destroy",
		    G_CALLBACK(callback_destroyed),
		    this);

  buffer = "EDIT: ";
  buffer += pathname;
  gtk_window_set_title (GTK_WINDOW (window), buffer.c_str());

  gtk_widget_set_size_request (window,
			       gman->prefs.get_edit_width(),
			       gman->prefs.get_edit_height());


  gtk_container_set_border_width(GTK_CONTAINER(window), 2);

  pixbuf = gtk_widget_render_icon_pixbuf(GTK_WIDGET(window),
				  GTK_STOCK_EDIT,
				  GTK_ICON_SIZE_DIALOG);
  if (pixbuf)
    {
      gtk_window_set_icon( GTK_WINDOW(window), pixbuf);
    }

	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);

  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (window), vbox);




  // --- Initialize tooltips. ---
//  tooltips = gtk_tooltips_new();

  // --- Menu Bar --- 
  menubar = gtk_menu_bar_new ();
  gtk_widget_show(menubar);
  gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, TRUE, 0);

  //  cout << "Edit::create_display - created menubar\n";



  //   --- File menu ---
  
  menu = create_bar_submenu(menubar, (char *)"File");
    
  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Open New",
                               accel_group, (char *)"^O", 
  			                      (char *)"Open a new blank edit window", 
  			                      (void *)(gman->callback_new_edit),
			                        (gpointer)gman);

  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Open file in this window",
                               accel_group, (char *)"^T", 
                              (char *)"Open a file to edit (this window)", 
  			                      (void *)(callback_load_file),
			                        (gpointer)this);

  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Load file into new window",
                               accel_group, (char *)"^L", 
                              (char *)"Load a file to edit (new window)", 
                              (void *)(gman->callback_load_for_edit),
			                        (gpointer)gman);
  
  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Save",
                               accel_group, (char *)"^S",
                              (char *)"Save current file", 
  			                      (void *)(callback_save),
			                        (gpointer)this);
  
  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"SaveAs",
                               accel_group, (char *)"^A",
                              (char *)"Save current file with new name", 
  			                      (void *)(callback_save_as),
			                        (gpointer)this);
  
  view_menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Manpage preview",
                               accel_group, (char *)"^M",
                              (char *)"View the current file", 
  			                      (void *)(callback_view),
			                        (gpointer)this);

  if (pathname.empty())
    {
    gtk_widget_set_sensitive(view_menuitem, FALSE);
    }
    
  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Quit",
                               accel_group, (char *)"^Q",
                              (char *)"Terminate editor", 
  			                      (void *)(quit_edit),
			                        (gpointer)this);
  
  menuitem = gtk_separator_menu_item_new();
  gtk_menu_shell_append(GTK_MENU_SHELL (menu), menuitem);
  gtk_widget_show(menuitem);
  
  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Clear",
                               accel_group, (char *)"^W",
                              (char *)"Wipe the slate clean (unsaved)", 
  			                      (void *)(callback_clear),
			                        (gpointer)this);

  
  
//  cout << "Edit::create_display - created File Menu\n";


  //   --- Edit menu ---
  
  menu = create_bar_submenu(menubar, (char *)"Edit");

  undo_menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Undo",
                               accel_group, (char *)"^Z",
                              (char *)"Undo last operation", 
  			                      (void *)(callback_undo),
			                        (gpointer)this);
  gtk_widget_set_sensitive(GTK_WIDGET(undo_menuitem), FALSE);

  redo_menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Redo",
                               accel_group, (char *)"!Z",
                              (char *)"Undo last undo operation", 
  			                      (void *)(callback_redo),
			                        (gpointer)this);
  gtk_widget_set_sensitive(GTK_WIDGET(redo_menuitem), FALSE);

  menuitem = gtk_separator_menu_item_new();
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
  gtk_widget_show(menuitem);

  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Copy",
                               accel_group, (char *)"^C",
                              (char *)"Copy marked text to clipboard", 
  			                      (void *)(callback_copy),
			                        (gpointer)this);

  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Cut",
                               accel_group, (char *)"^X",
                              (char *)"Cut marked text to clipboard", 
  			                      (void *)(callback_cut),
			                        (gpointer)this);

  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Delete",
                               accel_group, (char *)"^D",
                              (char *)"Delete marked text (unsaved)", 
  			                      (void *)(callback_delete),
			                        (gpointer)this);


  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Paste",
                               accel_group, (char *)"^V",
                              (char *)"Paste clipboard text at cursor", 
  			                      (void *)(callback_paste),
			                        (gpointer)this);

  menuitem = gtk_separator_menu_item_new();
  gtk_menu_shell_append(GTK_MENU_SHELL (menu), menuitem);
  gtk_widget_show(menuitem);
  
  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Select All",
                               accel_group, (char *)"",
                              (char *)"Make the whole file selected", 
  			                      (void *)(callback_select_all),
			                        (gpointer)this);

  //   --- Search menu ---

  menu = create_bar_submenu(menubar, (char *)"Search");

  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Find",
                               accel_group, (char *)"^F",
                              (char *)"Find text", 
  			                      (void *)(callback_find),
			                        (gpointer)this);

  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Replace",
                               accel_group, (char *)"^R",
                              (char *)"Replace text", 
  			                      (void *)(callback_replace),
			                        (gpointer)this);
   
  menuitem = create_menuitem (window,
                               menu, 
  			                      (char *)"Find/Replace Next",
                               accel_group, (char *)"^N",
                              (char *)"Find or Replace next", 
  			                      (void *)(callback_fr_next),
			                        (gpointer)this);
   
  
  
// cout << "Edit::create_display - ABOUT TO create_toolbar(vbox)" << endl;

  create_toolbar(vbox);

// cout << "Edit::create_display - done create_toolbar(vbox)" << endl;

  text = gtk_text_view_new();
  tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
  gtk_widget_show (text);

  
  gtk_text_view_set_editable(GTK_TEXT_VIEW(text), TRUE);

  sw = gtk_scrolled_window_new(NULL, NULL);

    /* the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
     * GTK_POLICY_AUTOMATIC will automatically decide whether you need
     * scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
     * there.  The first one is the horizontal scrollbar, the second, 
     * the vertical. */
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC );  //GTK_POLICY_ALWAYS );

  gtk_widget_show (sw);
  gtk_container_add(GTK_CONTAINER (sw), text);
  gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);

  gtk_text_buffer_set_text(GTK_TEXT_BUFFER(tbuf), sourcebuffer.c_str(), -1);

  // Place cursor at start of file...
  gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(tbuf),
                                     &start,
                                     0);
  gtk_text_buffer_select_range(tbuf,
			       &start,
			       &start);
  

  g_signal_connect (G_OBJECT (tbuf), "insert-text",
		    G_CALLBACK ( callback_insert_text ),  this);

  g_signal_connect (G_OBJECT (tbuf), "delete-range",
		    G_CALLBACK ( callback_delete_range ),  this);

// Do this AFTER gtk_text_buffer_set_text!
  g_signal_connect (G_OBJECT (tbuf), "changed",
		    G_CALLBACK ( callback_tbuf_changed ),  this);

  // NOT if seconds is zero
  if (seconds) timer_id = g_timeout_add_seconds(seconds, autosave, this);

  //  cout << "Edit::create_display - LEAVING\n";

}
