/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       edit.h                                                    *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       29-Mar-2013 12.55.48 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Header                                                                    *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#ifndef __EDIT_H__
#define __EDIT_H__

#include <gtk/gtk.h>
#include <iostream>
#include <vector>

#include "complain.h"
#include "gman.h"
#include "manpage.h"
//#include "requester.h"
#include "search.h"

using namespace std;


enum
{
UD_INSERT = 1,
UD_DELETE = 2,
UD_REPLACE= 4
};

/*
#define UD_INSERT_BIT  1
#define UD_DELETE_BIT  1<<1
#define UD_REPLACE_BIT 1<<2
*/

class Undo
{
public:
  int action;
  gint index;  // char_index in the file
  string txt;
};

class Edit
{
public:
  Edit(class GMan *g, char *pathname, char *sourcebuf);
  ~Edit(void);
	
	void restart_timed_autosaves(Edit *e);
  static void save_as_yes(GtkWidget *widget, Complain *c);
  static void save_as_no(GtkWidget *widget, Complain *c);
  void delete_manpage(class ManPage *mp);
  static void do_quit(GtkWidget *widget, Edit *e);

	void inspect_undo_list(void);
	void inspect_redo_list(void);

  class GMan *gman;
  GtkWidget *window;
  class Search *search;
  GtkWidget *text;  // TextView
  GtkTextBuffer *tbuf;
  class ManPage *manpage;
  bool search_initialised;
	bool replacing;

  list< Undo > undo_list;
	list< Undo > redo_list;
	
  guint seconds;        // number of seconds when autosave occurs

private:
  static void callback_destroyed(GtkWidget *widget, Edit *e);

  static void callback_view(GtkWidget *widget, Edit *e);
  static void save_close_yes(GtkWidget *widget, Complain *c);
  static void save_close_no(GtkWidget *widget, Complain *c);
  static void close_dialog(GtkWidget *widget, Edit *e);

  static void callback_insert_text(GtkTextBuffer *tbuf, GtkTextIter *iter, char *txt, gint len, Edit *e);
  static void callback_delete_range(GtkTextBuffer *tbuf, GtkTextIter *start, GtkTextIter *end, Edit *e);

  static void callback_tbuf_changed(GtkWidget *widget, Edit *e);
  void autosave_changes(Edit *e);

  // File Menu
  static void callback_save_as(GtkWidget *widget, Edit *e);
  static void callback_save(GtkWidget *widget, Edit *e);
  static void save_as_file(GtkWidget *widget, string pathname, Edit *e);
  static void load_selected_file(GtkWidget *widget, string pathname, Edit *e);
  static void callback_load_file(GtkWidget *widget, Edit *e);
  static void load_over_yes(GtkWidget *widget, Complain *c);
  static void load_into_new_window(GtkWidget *widget, Complain *c);
  static void callback_clear(GtkWidget *widget, Edit *e);
  static void save_quit_yes(GtkWidget *widget, Complain *c);
  static void save_quit_no(GtkWidget *widget, Complain *c);
  static void quit_edit(GtkWidget *widget, Edit *e);

  void write_source(Edit *e, bool overwrite, bool got_source);
  static gboolean autosave(gpointer data); //Edit *e);

  // Edit Menu
  //static void undo_redo(GtkWidget *widget, Edit *e, bool undo);
  static void callback_undo(GtkWidget *widget, Edit *e);
  static void callback_redo(GtkWidget *widget, Edit *e);
  static void callback_copy(GtkWidget *widget, Edit *e);
  static void callback_cut(GtkWidget *widget, Edit *e);
  static void callback_delete(GtkWidget *widget, Edit *e);
  static void callback_paste(GtkWidget *widget, Edit *e);
  static void callback_select_all(GtkWidget *widget, Edit *e);

  // Search Menu
  static void callback_find(GtkWidget *widget, Edit *e);
  static void callback_replace(GtkWidget *widget, Edit *e);
  static void callback_fr_next(GtkWidget *widget, Edit *e);

  void create_display(void);

  string pathname;
  string temp_pathname;
  string sourcebuffer;

  GtkTextIter iter;
  void create_toolbar(GtkWidget *vbox);

  GtkAccelGroup *accel_group;

  int num_changes;      // number of changes when autosave occurs if = max from gman
  gint old_line_number;
  guint timer_id;

  static const guint8 image_pixbuf[];

  GtkClipboard *clipboard;
  GdkPixbuf *pixbuf;
  GtkWidget *view_menuitem;

  GtkWidget *undo_menuitem;
  GtkWidget *redo_menuitem;
    
  bool tbuf_changed;
  bool terminating;
  bool undoing;
  bool redoing;


protected:

};


#endif // __EDIT_H__
