/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       editprefs.cpp                                             *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       29-Mar-2013 12.59.06 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Get editor autosave preferences                                           *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include "editprefs.h"
#include "misc.h"

using namespace misc;

EditPrefs::EditPrefs(GMan *g)
{	
cout << "EditPrefs:" << endl;
gman = g;
seconds_check_button = changes_check_button = NULL;
create_display();
}


EditPrefs::~EditPrefs(void)
{
cout << "~EditPrefs:" << endl;
if (pixbuf) g_object_unref(G_OBJECT(pixbuf));
}


void EditPrefs::callback_close_dialog(GtkWidget *widget, EditPrefs *ep)
{
ep->gman->ep = NULL;
delete(ep);
}

void EditPrefs::callback_cancel_editprefs(GtkWidget *widget, EditPrefs *ep)
{
gtk_widget_destroy(ep->window);
}

void EditPrefs::get_user_data(EditPrefs *ep)
{
if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ep->seconds_check_button)))
  {
  ep->gman->prefs.autosave_seconds = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ep->seconds_spinner));
  }
else ep->gman->prefs.autosave_seconds = 0;
  
if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ep->changes_check_button)))
  {
  ep->gman->prefs.autosave_changes = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ep->changes_spinner));
  }
else ep->gman->prefs.autosave_changes = 0;
}


// -----------------------------------------------------------------------------
// callback_do_save & callback_do_use
// These need to go through the g edits list and fix each current edit
// Not sure what to do about new edits started after this with callback_do_use
// -----------------------------------------------------------------------------

void EditPrefs::restart_autosave_timers(EditPrefs *ep)
{
GList *gl = ep->gman->edits;;
GList *nextnode;
Edit *e;

while (gl)
	{
	nextnode = gl->next;
	e = (Edit *)(gl->data);
		
	e->seconds = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ep->seconds_spinner));
	e->restart_timed_autosaves(e);
	gl = nextnode;
	}


}

void EditPrefs::callback_do_save(GtkWidget *widget, EditPrefs *ep)
{
cout << "EditPrefs::callback_do_save" << endl;
ep->get_user_data(ep);
ep->gman->prefs.save();
ep->restart_autosave_timers(ep);
callback_cancel_editprefs(widget, ep);
}

void EditPrefs::callback_do_use(GtkWidget *widget, EditPrefs *ep)
{
cout << "EditPrefs::callback_do_use" << endl;
ep->get_user_data(ep);
ep->restart_autosave_timers(ep);
callback_cancel_editprefs(widget, ep);
}

void EditPrefs::seconds_check_button_clicked(GtkWidget *widget, EditPrefs *ep)
{
if (ep->seconds_check_button)
  {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ep->seconds_check_button)))
    {
    gtk_widget_set_sensitive(GTK_WIDGET(ep->seconds_spinner), TRUE);
    }
  else
    {
    gtk_widget_set_sensitive(GTK_WIDGET(ep->seconds_spinner), FALSE);
    }
  }
}

void EditPrefs::changes_check_button_clicked(GtkWidget *widget, EditPrefs *ep)
{
if (ep->changes_check_button)
  {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ep->changes_check_button)))
    {
    gtk_widget_set_sensitive(GTK_WIDGET(ep->changes_spinner), TRUE);
    }
  else
    {
    gtk_widget_set_sensitive(GTK_WIDGET(ep->changes_spinner), FALSE);
    }
  }
}


void EditPrefs::create_display(void)
{
GtkWidget *table;
GtkWidget *widget;
GtkWidget *vbox;
GtkWidget *hbox;

window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
gtk_window_set_title (GTK_WINDOW (window), "Editor Autosave Preferences");
gtk_container_set_border_width(GTK_CONTAINER(window), 10);
gtk_grab_add(window);
g_signal_connect (G_OBJECT (window), "destroy",
		    G_CALLBACK (callback_close_dialog),
		    this);
gtk_widget_set_size_request (window,
			       gman->prefs.editprefs_width,
			       gman->prefs.editprefs_height);

pixbuf = gtk_widget_render_icon_pixbuf(GTK_WIDGET(window),
				  GTK_STOCK_PREFERENCES,
				  GTK_ICON_SIZE_DIALOG);

if (pixbuf) gtk_window_set_icon( GTK_WINDOW(window), pixbuf);

vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
gtk_container_add(GTK_CONTAINER(window), vbox);
	
	
hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, FALSE, 0);


  
seconds_check_button = gtk_check_button_new_with_label("Seconds");
g_signal_connect (G_OBJECT (seconds_check_button), "clicked",
            G_CALLBACK(seconds_check_button_clicked), this);
gtk_box_pack_start(GTK_BOX(hbox), seconds_check_button, TRUE, FALSE, 0);
  
seconds_spinner = create_spinner(gman->prefs.autosave_seconds, 0, 10000);
gtk_box_pack_start(GTK_BOX(hbox), seconds_spinner, TRUE, FALSE, 0);

gtk_widget_set_tooltip_text(GTK_WIDGET(seconds_spinner),
			  "Seconds\nSeconds to elapse between timed autosaves.\n0 to disable timed autosaves.");

if (gman->prefs.autosave_seconds)
  {
  gtk_widget_set_sensitive(GTK_WIDGET(seconds_spinner), TRUE);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(seconds_check_button), TRUE);
  }
else
  {
  gtk_widget_set_sensitive(GTK_WIDGET(seconds_spinner), FALSE);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(seconds_check_button), FALSE);
  }

hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 20);
gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, FALSE, 0);


  
changes_check_button = gtk_check_button_new_with_label("Changes");
g_signal_connect (G_OBJECT (changes_check_button), "clicked",
            G_CALLBACK(changes_check_button_clicked), this);
gtk_box_pack_start(GTK_BOX(hbox), changes_check_button, TRUE, FALSE, 0);
  
changes_spinner = create_spinner(gman->prefs.autosave_changes, 0, 10000);
gtk_box_pack_start(GTK_BOX(hbox), changes_spinner, TRUE, FALSE, 0);

gtk_widget_set_tooltip_text(GTK_WIDGET(changes_spinner),
			  "Seconds\nNumber of edit changes to elapse between autosaves.\n0 to disable change autosaves.");
  
if (gman->prefs.autosave_changes)
  {
  gtk_widget_set_sensitive(GTK_WIDGET(changes_spinner), TRUE);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(changes_check_button), TRUE);
  }
else
  {
  gtk_widget_set_sensitive(GTK_WIDGET(changes_spinner), FALSE);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(changes_check_button), FALSE);
  }

hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, FALSE, 10);



widget = gtk_button_new_with_label ("SAVE");
g_signal_connect(G_OBJECT (widget), "clicked",
		    G_CALLBACK(callback_do_save),  this);
                                                         
gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 0);


widget = gtk_button_new_with_label ("USE (no save)");
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK(callback_do_use),  this);
gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 0);

                                                         
  
widget = gtk_button_new_with_label ("CANCEL");
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK ( callback_cancel_editprefs ),  this);
gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 0);


gtk_widget_show_all(window);
}