/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       gman.h                                                    *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       29-Mar-2013 11.38.02 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Header                                                                    *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#ifndef __GMAN_H__
#define __GMAN_H__

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <list>

#include "about.h"
#include "apropos.h"
#include "complain.h"
#include "edit.h"
#include "editprefs.h"
#include "gman.h"
#include "mandir.h"
#include "misc.h"
#include "prefs.h"
#include "printprefs.h"
#include "quit.h"
#include "windowsizes.h"

class GMan
    {
    public:
      GMan(int argc, char *argv[], Quit *quit);
      ~GMan(void);

      void delete_mandir(class ManDir *mandir);
      void delete_manpage(class ManPage *manpage);
      void delete_apropos(class Apropos *ap);
      void delete_about(class About *about);
      void delete_windowsizes(class Windowsizes *ws);
      void delete_printprefs(class Printprefs *printprefs);
      gboolean process_manpath(void);
      void get_main_menu (GtkWidget *window, GtkWidget *menubar);

      static bool do_manpage(GMan *g, char *manpage);
      static bool do_infopage(GMan *g, char *infopage);
      static void chose_section(GtkWidget *widget, class Section* man_section);
      static void find_ip(GMan *g);

      static void find_manpage(GtkWidget *widge, GMan *g);
      static void find_apropos(GtkWidget *widget, GMan *g);
      static void callback_quit(GtkWidget *widget, Quit *q);
      static void callback_help(GtkWidget *widget, GMan *g);
      static void callback_about(GtkWidget *widget, GMan *g);
      static void callback_windowsizes(GtkWidget *widget, GMan *g);
      static void callback_printprefs(GtkWidget *widget, GMan *g);

      static void callback_new_edit(GtkWidget *widget, GMan *g);
      static void callback_load_for_edit(GtkWidget *widget, GMan *g);

			static void key_event(GtkWidget *widget, GdkEventKey *event, GMan *g);


				
      static void window_exposed(GtkWidget *widget, GMan *g);

      Quit *q;
      static GList *manpath;
      GList *mandirs; 
      static GList *manpages;
      static GList *edits;
      static GList *apropos;
      static list<string> section_num_list;
      
      About *about;
      Windowsizes *windowsizes;

      static Printprefs *printprefs;
      class EditPrefs *ep;

      static Prefs prefs;

      // static because they aren't fixed before "expose_event"
      static GdkPixbuf *icon_pixbuf;
      static guint     expose_signal;
      static char *start_manpage;
      list<Section *> section_list;

    private:
      static void callback_open(GtkWidget *widget, GMan *g);
      static void load_selected(GtkWidget *widget, string pathname, GMan *g);
      static void callback_edit_prefs(GtkWidget *widget, GMan *g);

      GtkWidget *window;
      GtkWidget *entry;
      GtkAccelGroup *accel_group;

      static const guint8 icon_pixbuf_source[];


    protected:


    };

class Section
{
 public:
  string name;
  GMan *g;
};


#endif // _GMAN_H
