/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       mandir.cpp                                                *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       16-Mar-2013 14.31.30 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Select a manpage to view from a section of manual pages                   *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <errno.h>
#include "mandir.h"
#include "manpage.h"
#include "misc.h"

using namespace std;

ManDir::ManDir(GMan *gman)
{
cout << "ManDir:\n";

window = NULL;
pathnames = NULL;
display_names = NULL;
manpages = NULL;
mandirs = NULL;
g = gman;
  
}


ManDir::~ManDir(void)
{
  using namespace misc;

  GList *gl;
  GList *nextnode;

  cout << "~ManDir:\n";

  // free the GLists
  free_list(pathnames);
  pathnames = NULL;
  free_list(display_names);
  display_names = NULL;
  if (window)
    {
    gtk_widget_destroy ( GTK_WIDGET(window) );
    }

  if (pixbuf) g_object_unref(G_OBJECT(pixbuf));
}


void ManDir::CloseDialog (GtkWidget *widget, ManDir *md)
{
  md->g->delete_mandir(md);
}


void ManDir::callback_page_selected(GtkWidget *widget, ManDir *md)
{
GtkTreeModel* model;	
GtkTreeIter iter;
GList *glist = NULL, *node;
  
gchar *name;
gchar *pathname; 

list< GtkWidget * >::iterator i;

//cout << "ENTERED page_selected: " << endl;

for (i=md->view_list.begin(); i!=md->view_list.end(); ++i)
  {

  glist = gtk_tree_selection_get_selected_rows(
											gtk_tree_view_get_selection(GTK_TREE_VIEW( *i ) ),
											&model);
  if (glist)
    {
    for (node=glist; node; node=node->next)
	    {
	    if (gtk_tree_model_get_iter(model, &iter, (GtkTreePath *)node->data))
		    {
		    gtk_tree_model_get(model, &iter,
                          0, &name,
                          1, &pathname,
                          -1);

//        cout <<  "ManDir::callback_page_selected - GOT: " << name << " : " << pathname << endl;

            
        ManPage *manpage = new ManPage(md->g);

        md->g->manpages = g_list_append(md->g->manpages, manpage);
        manpage->load_manpage((char *)"", pathname, name); 
            
            
            
        g_free(name);  
        g_free(pathname);
		    }

      gtk_tree_selection_unselect_all( GTK_TREE_SELECTION( gtk_tree_view_get_selection( GTK_TREE_VIEW(*i) ) ) );
      break;
      }
	  }
  }

}



void ManDir::choose_manpage(char *man_section)
{
GtkWidget *wid;
GtkWidget *vbox, *box;
string title = "SECTION ";
  
if (strlen(man_section)>3) title += &man_section[3];
  
// --- Create a dialog window
window = gtk_window_new (GTK_WINDOW_TOPLEVEL); //gtk_dialog_new ();

// --- Make the window visible
gtk_widget_show (window);  // prevents choosing premeturely

// --- Trap the window close signal
g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (CloseDialog),
		      this);

// --- Add a title to the window
gtk_window_set_title (GTK_WINDOW (window), title.c_str());
	
//gtk_widget_set_size_request(GTK_WIDGET(window),
//                                 g->prefs.get_mandir_width(),
//                                 g->prefs.get_mandir_height());

// Could use this instead...
gtk_window_set_default_size(GTK_WINDOW (window),
                                g->prefs.get_mandir_width(),
                                g->prefs.get_mandir_height());
	
// --- Create a small border
gtk_container_set_border_width(GTK_CONTAINER(window),5);
pixbuf = gtk_widget_render_icon_pixbuf(GTK_WIDGET(window),
				    GTK_STOCK_INDEX,
				    GTK_ICON_SIZE_DIALOG);
	
if (pixbuf) gtk_window_set_icon( GTK_WINDOW(window), pixbuf);


vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
gtk_box_set_homogeneous(GTK_BOX(vbox), FALSE);  // Doesn't work as expected: last two are homogeneous!


gtk_container_add (GTK_CONTAINER (window), vbox);

// --- Create the message in a label
wid = gtk_label_new("CHOOSE MANPAGE");
gtk_box_pack_start (GTK_BOX (vbox), wid, FALSE, FALSE, 0);
  
  

// A scrolled window
box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2); //box = gtk_vbox_new(FALSE, 2);
gtk_box_set_homogeneous(GTK_BOX(box), TRUE);

//gtk_container_add(GTK_CONTAINER(vbox), box);
  
gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, TRUE, 0);
create_mandir_box(box, man_section);
  
 
box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2); //box = gtk_hbox_new(FALSE, 2);
gtk_box_set_homogeneous(GTK_BOX(box), FALSE);
//gtk_widget_show (box);

gtk_container_add(GTK_CONTAINER(vbox), box);


// --- Create the "Close" button
wid = gtk_button_new_with_label ("Close");

// --- Need to close the window if they press "Close"
g_signal_connect (G_OBJECT (wid), "clicked",
		    G_CALLBACK (CloseDialog), this);


// --- Allow it to be the default button
gtk_widget_set_can_default(GTK_WIDGET(wid), TRUE); //GTK_WIDGET_SET_FLAGS (wid, GTK_CAN_DEFAULT);

// --- Add the button to the dialog
gtk_box_pack_start (GTK_BOX(box), wid, TRUE, FALSE, 0);

// --- Make the button the default button
gtk_widget_grab_default (wid);

// --- Make everything visible
gtk_widget_show_all (window);

}


void ManDir::create_mandir_box(GtkWidget *box, char *man_section)
{
using namespace misc;

GtkWidget *scrolled_window;
GtkWidget *vbox;
GtkWidget *hbox;
GtkWidget *wid, *view;
GList *mp = g->manpath;
string full_mname;

// cout << "ManDir::create_mandir_box - ENTERED" << endl;
  
scrolled_window = gtk_scrolled_window_new(NULL, NULL);
    // the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
    // GTK_POLICY_AUTOMATIC will automatically decide whether you need
    // scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
    // there.  The first one is the horizontal scrollbar, the second, 
    // the vertical.
gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
                                GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS );

gtk_widget_show (scrolled_window);
gtk_box_pack_start(GTK_BOX(box), scrolled_window, TRUE, TRUE, 0);

vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);//vbox = gtk_vbox_new(FALSE, 0);
gtk_box_set_homogeneous (GTK_BOX(vbox), FALSE);
gtk_widget_show (vbox);

gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window),
					vbox);

while (mp)
  {
  full_mname = (char *)mp->data;
  full_mname += "/";
  full_mname += man_section;

  view = create_view_and_model();
  gtk_widget_show (view);

  if (add_to_view(view, full_mname, man_section))
    {
    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0); //hbox = gtk_hbox_new(FALSE, 0);
    gtk_widget_show (hbox);

    // --- Create the  label
    wid = gtk_label_new (full_mname.c_str());

    gtk_misc_set_alignment (GTK_MISC (wid), .5, 0);
    gtk_box_pack_start(GTK_BOX(hbox), 
			              wid, FALSE, FALSE, 0);
    // --- Make the label visible
    gtk_widget_show (wid);
		decorate_widget(GTK_WIDGET(wid), &red, NULL,  NULL, NULL);

//    gtk_widget_set_style(GTK_WIDGET(wid), red_style); 
			
    gtk_container_add(GTK_CONTAINER(vbox), hbox);

    gtk_widget_show (view);
    gtk_container_add(GTK_CONTAINER(vbox), view);
    view_list.push_back(view);
    }
  else
    {
    gtk_widget_destroy( GTK_WIDGET(view) );
    }

  mp = mp->next;
  }
}


int ManDir::add_to_view(GtkWidget *view, string full_mname, char *man_section)
{
using namespace misc;

DIR    *dp;                   // Directory pointer
struct dirent *direntry;      // This directory entry
string pathname;
string display_name;
string fname;
char *c;
  
gchar *str;
GtkTreeModel *model;
GtkTreeIter iter;  
GtkTreeIter sibling;
gboolean valid;
int n = 0;  // number of entries 
  
//cout << "ManDir::add_to_view - ENTERED: full_mname= " << full_mname << endl;
  
pathname = full_mname;
pathname += "/";

dp = opendir(full_mname.c_str());
  
// Not all paths have all sections...
if (!dp)
  {
//  cerr << "ManDir::add_to_view - NOT GOT dp for " << "\"" << full_mname << "\"" << endl;
//  cerr << strerror(errno) << endl;   // No such file or directory !!!
  return n;
  }

  
//cout << "ManDir::add_to_view - GOT dp" << endl;
  
while ( (direntry = readdir(dp)) )
  {
  if (STREQ(direntry->d_name, (char *)".") || STREQ(direntry->d_name, (char *)"..")) continue;

  fname = pathname + direntry->d_name;

  display_name = direntry->d_name;

  // usually compressed files nowadays
  if (SUFEQ((char *)display_name.c_str(), (char *)".gz"))
	  {
    display_name = display_name.substr(0, display_name.length()-3);
	  }
  else if (SUFEQ((char *)display_name.c_str(), (char *)".bz2"))
	  {
    display_name = display_name.substr(0, display_name.length()-4);
	  }

int p = display_name.length() - 1;
while (p > 0)
    {
    if (display_name[p] == '.') break;
    p--;
    }
if ((display_name[p] == '.') && (p>0))
    {
    display_name = display_name.substr(0, p);
    display_name += " (";
    display_name += &man_section[3];
    display_name += ")";
    }
    
  model = gtk_tree_view_get_model( GTK_TREE_VIEW(view) );

  // They should be arranged alphabetically somehow
  valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);
    
  while (valid)
    {
    gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 
                          0, // the column #,
                          &str,
                          -1);

    if (display_name != str)
      {
      if (strcasecmp(display_name.c_str(), str) < 0) // ignore case to find where to put it
//      if (strcmp(display_name.c_str(), str) < 0)       // case senstive to find where to put it
        {
        // INSERT IT HERE          
        gtk_list_store_insert_before(GTK_LIST_STORE(model),
                                &sibling,
                                &iter); 
        gtk_list_store_set(GTK_LIST_STORE(model), &sibling, 0,
                           (gchar *)display_name.c_str(), 1, (gchar *)fname.c_str(), -1);
        n++;
        break;
        }
      }      
    valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);  
    } // END OF while (valid)

  if (!valid)
    {
    gtk_list_store_append(GTK_LIST_STORE(model), &iter);
        gtk_list_store_set (GTK_LIST_STORE(model), &iter,
                      0, (gchar *)display_name.c_str(),
                      1, (gchar *)fname.c_str(),
                      -1);
    n++;
    }
// cout << "ManDir::add_to_view - display_name= " << display_name << endl;
// cout << "ManDir::add_to_view -     pathname= " << pathname << endl; 

  } // END OF:  while ( (direntry = readdir(dp)) )

closedir(dp);
 
// Make sure it can be selected!
g_signal_connect(G_OBJECT ( gtk_tree_view_get_selection(GTK_TREE_VIEW(view)) ),
								 "changed",
								 G_CALLBACK(callback_page_selected), this);

// Attempt to prevent automatic selection at startup fails - PAIN
gtk_tree_selection_unselect_all( GTK_TREE_SELECTION( gtk_tree_view_get_selection( GTK_TREE_VIEW(view) ) ) );

return n;
}



GtkWidget *ManDir::create_view_and_model(void)
{
GtkTreeViewColumn   *col;
GtkCellRenderer     *renderer;
GtkWidget           *view;
GtkTreeModel        *model;

model = GTK_TREE_MODEL(gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING));

view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(model));

//g_signal_connect(G_OBJECT(view), "row-activated", G_CALLBACK(callback_page_selected), this);

// Make headers invisible
gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);

gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(view), TRUE);
gtk_tree_view_set_search_column(GTK_TREE_VIEW(view), 0);

g_object_unref(model);

// Column #1
renderer = gtk_cell_renderer_text_new();
col = gtk_tree_view_column_new_with_attributes("Manpage",
                 renderer,
                 "text",
                 0,
                 NULL);  
gtk_tree_view_column_set_resizable(col, TRUE);

// pack tree view column into tree view
gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
  


// Column #2
renderer = gtk_cell_renderer_text_new();
col = gtk_tree_view_column_new_with_attributes("Pathname",
                 renderer,
                 "text",
                 0,
                 NULL);  
gtk_tree_view_column_set_resizable(col, TRUE);

// pack tree view column into tree view
gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
gtk_tree_view_column_set_visible(col, FALSE);
  

// Prevent highlighting of a row
//gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(view)),
 //                             GTK_SELECTION_NONE);
  
// Multiple select
//gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(view)),
//                              GTK_SELECTION_MULTIPLE);
  
gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(view)),
                              GTK_SELECTION_SINGLE);

// Show the whole of the model
gtk_tree_view_expand_all(GTK_TREE_VIEW(view));

return view;
}

