/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       manpage.cpp                                               *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       10-Mar-2013 19.52.08 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Viewer for a manual page                                                  *
* Italics are italics (not underlined) and coloured                         *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <iostream>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <signal.h>

#include "manpage.h"
#include "misc.h"
#include "printprefs.h"
#include "prefs.h"
#include "printfile.h"
#include "complain.h"
#include "gman.h"

using namespace std;
using namespace misc;

gint ManPage::timer = 0;
int ManPage::thread_pid = 0;

void ManPage::CloseDialog (GtkWidget *widget, ManPage *mp)
{
if (mp->edit)
  {
  mp->edit->delete_manpage(mp);
  mp->edit = NULL;
  }
else mp->gman->delete_manpage(mp);
}

void ManPage::fix_comment_flag (GtkWidget *widget, ManPage *mp)
{
mp->comment_flag =
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widget));
    
cout << "ManPage::comment_flag: " 
       << (char *)((mp->comment_flag) ? "active" : "not active")
       << endl;
       
// The buffer of the TextView widget needs to be re-initialised
// This inserts at start of buffer
  
mp->insert_text();

}


void ManPage::reload(GtkWidget *widget, ManPage *mp, bool same_line_number)
{
static gint old_line_number = -1;

GtkTextIter insert_iter;
GtkTextIter target_iter;  
gint line_number;
gint buffer_x, buffer_y;
  
// This is the problem, mp->filename gets corrupted
cout << "ManPage::reload - mp->filename= " << mp->filename << endl;

if (same_line_number)
  {
  line_number = old_line_number;
  }
else
  {
  gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(mp->text),
                                                    GTK_TEXT_WINDOW_TEXT,
                                                    0,          //gint window_x,
                                                    0,          //gint window_y,
                                                    &buffer_x,
                                                    &buffer_y);  
  gtk_text_view_get_line_at_y(GTK_TEXT_VIEW(mp->text),
                            &target_iter,
                            buffer_y,        // gint y,
                            NULL);

  // What line is this?
  line_number = gtk_text_iter_get_line(&target_iter);
  }
old_line_number = line_number;

cout << "ManPage::reload - line_number at top= " << line_number << endl;
// This gives us the line number of the top line
// Now when the page is loaded, we need to put this line number at window 0,0

// First clear the text buffer
gtk_text_buffer_set_text(GTK_TEXT_BUFFER(mp->tbuf ),
			   "",
			   -1);

// Then re-load the page from the source file

mp->do_load_manpage(mp->filename, mp->mpagename);


// AND make it display the same portion of text
mp->insert_text();

// Get iter at line_number
gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(mp->tbuf ),
                                 &insert_iter, 
                                 line_number);

gtk_text_buffer_move_mark_by_name(GTK_TEXT_BUFFER(mp->tbuf),
                          "insert",
                          &insert_iter);
gtk_text_buffer_move_mark_by_name(GTK_TEXT_BUFFER(mp->tbuf),
                                  "selection_bound",
                                  &insert_iter);

// Now scroll so that this position is at the top

gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(mp->text),
                             gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(mp->tbuf )), //GtkTextMark *mark,
                             0,              //gdouble within_margin,
                             TRUE,           //gboolean use_align,
                             0,              //gdouble xalign,
                             0               //gdouble yalign);
                             );

}


void ManPage::callback_reload(GtkWidget *widget, ManPage *mp)
{
GtkTextIter insert_iter;
GtkTextIter target_iter;  
gint line_number;
gint buffer_x, buffer_y;

  
// This is the problem, mp->filename gets corrupted
cout << "ManPage::callback_reload - mp->filename= " << mp->filename << endl;


gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(mp->text),
                                                    GTK_TEXT_WINDOW_TEXT,
                                                    0,          //gint window_x,
                                                    0,          //gint window_y,
                                                    &buffer_x,
                                                    &buffer_y);  
gtk_text_view_get_line_at_y(GTK_TEXT_VIEW(mp->text),
                            &target_iter,
                            buffer_y,        // gint y,
                            NULL);

// What line is this?
line_number = gtk_text_iter_get_line(&target_iter);


cout << "ManPage::callback_reload - line_number at top= " << line_number << endl;
// This gives us the line number of the top line
// Now when the page is loaded, we need to put this line number at window 0,0

// First clear the text buffer
gtk_text_buffer_set_text(GTK_TEXT_BUFFER(mp->tbuf ),
			   "",
			   -1);

// Then re-load the page from the source file

mp->do_load_manpage(mp->filename, mp->mpagename);


// AND make it display the same portion of text
// NOTE: this doesn't work if a single multiple action is performed
// such as marking a section and replacing it so that there is deletion
// and insertion in one operation. In that case, the second part
// gives line number 0 every time.

mp->insert_text();

// Get iter at line_number
gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(mp->tbuf ),
                                 &insert_iter, 
                                 line_number);

gtk_text_buffer_move_mark_by_name(GTK_TEXT_BUFFER(mp->tbuf),
                          "insert",
                          &insert_iter);
gtk_text_buffer_move_mark_by_name(GTK_TEXT_BUFFER(mp->tbuf),
                                  "selection_bound",
                                  &insert_iter);

// Now scroll so that this position is at the top

gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(mp->text),
                             gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(mp->tbuf )), //GtkTextMark *mark,
                             0,              //gdouble within_margin,
                             TRUE,           //gboolean use_align,
                             0,              //gdouble xalign,
                             0               //gdouble yalign);
                             );

}


void ManPage::callback_search(GtkWidget *widget, ManPage *mp)
{
cout << "ManPage::callback_search\n";

mp->search_initialised = FALSE;
if (!mp->search)
  {
  mp->search = new Search(mp);
  }
else cout << "Edit::callback_find - search already in use\n";
}

void ManPage::callback_edit(GtkWidget *widget, ManPage *mp)
{
  cout << "ManPage::callback_edit\n";

  Edit *edit = new Edit(mp->gman, mp->filename, (char *)mp->sourcebuffer.c_str() );

  //  if (mp->edit == NULL)
  //    {
  //    mp->edit = new Edit(mp->gman, mp);
  //    }
  //  else
  //    {
  //      Complain *c = new Complain;
  //      string buffer = mp->filename;
  //      buffer += "\n\nYou tried to edit the same manpage more than once.\n";
  //      buffer += "Only one copy at a time can be edited.";      
  //
  //      // INFORM THE USER! - NOTE: Complain deletes itself
  //      // ================================================
  //      c->complain(c->CN_OK, "Edit", (char *)buffer.c_str(), NULL, NULL, mp);
  //    }

  mp->gman->edits = g_list_append(mp->gman->edits, edit); 

}

void ManPage::man_print(GtkWidget *widget, ManPage *mp)
{
  cout << "ManPage::man_print\n";

  Printfile pf(mp);    // This should print the file


}

void ManPage::callback_printprefs(GtkWidget *widget, ManPage *mp)
{
cout << "ManPage::callback_printprefs\n";

if (mp->gman->printprefs == NULL)
  {
  mp->gman->printprefs = new Printprefs(mp->gman);
  }
else cout << "ManPage::callback_printprefs : mp->gman->printprefs not NULL!" << endl;
}




void ManPage::stop_timer(void)
{
  if (timer)
    {
      g_source_remove(timer);
      timer = 0;
    }
}


void ManPage::start_timer(ManPage *mp)
{
  if (!timer)
    timer = g_timeout_add(200, check_preview, mp);
}


gint ManPage::check_preview(gpointer mp) // This has to return TRUE I think.
{
  if (((ManPage *)mp)->done_preview)
    {
      string command;

      // NOT FOUND IT so delete the file and quit the timer

      //printf("ManPage::check_preview - REMOVING %s : pid %d\n",
      //	     ((ManPage *)mp)->tname.c_str(), ((ManPage *)mp)->thread_pid );

      command = "rm " + ((ManPage *)mp)->tname;
      system(command.c_str());
      stop_timer();

    }

  //  else
  //    {
  //      static int n = 0;
  //      if (n == 0)
  //	{
  //	printf("ManPage::check_preview - still using preview : thread_pid %d\n",
  //	       ((ManPage *)mp)->thread_pid);
  //	n++;
  //	}
  //    }

  return TRUE;
}


int ManPage::do_preview(void *arg)
{
  ManPage *mp = (ManPage *)arg;
  system((char *)mp->command.c_str());

  //printf("do_preview: LEAVING\n");

  mp->done_preview = TRUE;

  //  exit(0);  // This seems to be a problem with Font and Paper Size
  return 0;     // Not sure about this... seems OK
}



// Our signal trap
void ManPage::Trap(int signum)
{
  switch (signum)
    {
    case SIGCHLD:
      wait(&thread_pid);
      thread_pid = 0;
      break;

    default:
      break;
    }
}



void ManPage::callback_printpreview(GtkWidget *widget, ManPage *mp)
{
  cout << "ManPage::callback_printpreview\n";

  if ((!timer) && (thread_pid == 0))
    {
      Printfile pf(mp, mp->tname);
      mp->done_preview = FALSE;
      mp->command = *(mp->gman->prefs.get_preview_command());
      mp->command += " ";
      mp->command += mp->tname;

      thread_pid =  clone(do_preview, mp->stack + (50 * 1024), CLONE_VM|SIGCHLD, (void *)mp);

      //printf("thread_pid= %d\n", thread_pid);

      start_timer(mp);

      //cout << "STARTED TIMER\n";
    }
  else
    {
      // Only one preview at a time...

      Complain *c = new Complain;
      string buffer = "You tried to preview more than one manpage.\n";
      buffer += "Only one at a time can be previewed for printing.";      

      // INFORM THE USER! - NOTE: Complain deletes itself
      // ================================================
      c->complain(c->CN_OK,
		  (char *)"Print Preview",
		  (char *)buffer.c_str(),
		  NULL, NULL, mp);
    }
}




void ManPage::create_tags(GtkTextBuffer *buffer)
{

  /* Create a bunch of tags. Note that it's also possible to
   * create tags with gtk_text_tag_new() then add them to the
   * tag table for the buffer, gtk_text_buffer_create_tag() is
   * just a convenience function. Also note that you don't have
   * to give tags a name; pass NULL for the name to create an
   * anonymous tag.
   *
   * In any real app, another useful optimization would be to create
   * a GtkTextTagTable in advance, and reuse the same tag table for
   * all the buffers with the same tag set, instead of creating
   * new copies of the same tags for every buffer.
   *
   * Tags are assigned default priorities in order of addition to the
   * tag table.     That is, tags created later that affect the same text
   * property affected by an earlier tag will override the earlier
   * tag.  You can modify tag priorities with
   * gtk_text_tag_set_priority().
   */

  gtk_text_buffer_create_tag (buffer, "heading",
                "weight", PANGO_WEIGHT_BOLD,
               "size", 15 * PANGO_SCALE,
                  NULL);


  // Should probably use a monospace font so that tables line up properly


  // Try inserting "red"   EXCELLENT!
  // ================================
  gtk_text_buffer_create_tag (buffer,
			      "italic",
			      "style",
			      PANGO_STYLE_ITALIC,
			      "foreground",
			      "red",
			      NULL);

  // Same as italic as far as I can see
  gtk_text_buffer_create_tag (buffer,
			      "oblique",
			      "style",
			      PANGO_STYLE_OBLIQUE,
			      "foreground",
			      "blue",
			      NULL);

  gtk_text_buffer_create_tag (buffer,
			      "comment_text",
			      "style",
			      PANGO_STYLE_NORMAL,
			      "foreground",
			      "green",
			      "family",
			      "monospace",
			      NULL);

  gtk_text_buffer_create_tag (buffer,
			      "normal_text",
			      "style",
			      PANGO_STYLE_NORMAL,
			      "foreground",
			      "black",
			      "family",
			      "monospace",
			      NULL);

// I can't find where these colours come from.
// Where are they documented?
// They come from /usr/share/X11/rgb.txt
// It appears that this is understood by GDK and GTK...
  gtk_text_buffer_create_tag (buffer, "green_foreground",   // not quite dark enough
                  "foreground", "green", NULL);
                  
  gtk_text_buffer_create_tag (buffer, "green_background",   // not quite dark enough
                  "background", "green", NULL);  

  gtk_text_buffer_create_tag (buffer, "violet_foreground",
                  "foreground", "violet", NULL);  

  gtk_text_buffer_create_tag (buffer, "purple_foreground",
                  "foreground", "purple", NULL);  

  gtk_text_buffer_create_tag (buffer, "magenta_foreground",
                  "foreground", "magenta", NULL);  

 gtk_text_buffer_create_tag (buffer, "red_foreground",
                  "foreground", "red", NULL);  

 gtk_text_buffer_create_tag (buffer, "black_foreground",
                  "foreground", "black", NULL);  


// Although these tags are created they all appear either normal or bold
gtk_text_buffer_create_tag (buffer, "ultralight",
                 "weight", PANGO_WEIGHT_ULTRALIGHT, NULL);  

gtk_text_buffer_create_tag (buffer, "light",
                 "weight", PANGO_WEIGHT_LIGHT, NULL);  

gtk_text_buffer_create_tag (buffer, "normal",
                 "weight", PANGO_WEIGHT_NORMAL, NULL);  

gtk_text_buffer_create_tag (buffer, "bold",
                 "weight", PANGO_WEIGHT_BOLD, NULL);  
  
gtk_text_buffer_create_tag (buffer, "ultrabold",
                 "weight", PANGO_WEIGHT_ULTRABOLD, NULL);  
  
gtk_text_buffer_create_tag (buffer, "heavy",
                 "weight", PANGO_WEIGHT_HEAVY, NULL);  
  
  
  
  
  
  
  
  gtk_text_buffer_create_tag (buffer, "big",
                  /* points times the PANGO_SCALE factor */
                  "size", 20 * PANGO_SCALE, NULL);

  gtk_text_buffer_create_tag (buffer, "xx-small",
                "scale", PANGO_SCALE_XX_SMALL, NULL);

  gtk_text_buffer_create_tag (buffer, "x-large",
                "scale", PANGO_SCALE_X_LARGE, NULL);
  
  gtk_text_buffer_create_tag (buffer, "monospace",
                 "family", "monospace", NULL);
  
  gtk_text_buffer_create_tag (buffer, "blue_foreground",
                  "foreground", "blue", NULL);  

  gtk_text_buffer_create_tag (buffer, "red_background",
                "background", "red", NULL);

  gtk_text_buffer_create_tag (buffer, "big_gap_before_line",
               "pixels_above_lines", 30, NULL);

  gtk_text_buffer_create_tag (buffer, "big_gap_after_line",
                  "pixels_below_lines", 30, NULL);

  gtk_text_buffer_create_tag (buffer, "double_spaced_line",
                  "pixels_inside_wrap", 10, NULL);

  gtk_text_buffer_create_tag (buffer, "not_editable",
                "editable", FALSE, NULL);
  
  gtk_text_buffer_create_tag (buffer, "word_wrap",
                "wrap_mode", GTK_WRAP_WORD, NULL);

  gtk_text_buffer_create_tag (buffer, "char_wrap",
                 "wrap_mode", GTK_WRAP_CHAR, NULL);

  gtk_text_buffer_create_tag (buffer, "no_wrap",
               "wrap_mode", GTK_WRAP_NONE, NULL);
  
  gtk_text_buffer_create_tag (buffer, "center",
                  "justification", GTK_JUSTIFY_CENTER, NULL);

  gtk_text_buffer_create_tag (buffer, "right_justify",
                "justification", GTK_JUSTIFY_RIGHT, NULL);

  gtk_text_buffer_create_tag (buffer, "wide_margins",
                  "left_margin", 50, "right_margin", 50,
                 NULL);
  
  gtk_text_buffer_create_tag (buffer, "strikethrough",
               "strikethrough", TRUE, NULL);
  
  gtk_text_buffer_create_tag (buffer, "underline",
                "underline", PANGO_UNDERLINE_SINGLE, NULL);

  gtk_text_buffer_create_tag (buffer, "double_underline",
                 "underline", PANGO_UNDERLINE_DOUBLE, NULL);

  gtk_text_buffer_create_tag (buffer, "superscript",
                  "rise", 10 * PANGO_SCALE,   /* 10 pixels */
                "size", 8 * PANGO_SCALE,    /* 8 points */
                 NULL);
  
  gtk_text_buffer_create_tag (buffer,
			      "subscript",
			      "rise", -10 * PANGO_SCALE,   /* 10 pixels */
			      "size", 8 * PANGO_SCALE,     /* 8 points */
			      NULL);

  gtk_text_buffer_create_tag (buffer,
			      "rtl_quote",
			      "wrap_mode", GTK_WRAP_WORD,
			      "direction", GTK_TEXT_DIR_RTL,
			      "indent", 30,
			      "left_margin", 20,
			      "right_margin", 20,
			      NULL);
}


ManPage::ManPage(GMan *g)
{
  cout << "ManPage:\n";

  struct sigaction act;

  act.sa_handler = Trap;
  sigemptyset(&act.sa_mask);
  act.sa_flags = 0;
  sigaction(SIGCHLD, &act, 0);

  gman = g;
  progname = NULL;
  section_num = NULL;
  filebuffer.clear();
  sourcebuffer.clear();
  filename = NULL;
  tbuf = NULL;
  edit = NULL;
  search = NULL;
  fblen = 0;
  man_window = NULL;
  comment_flag = FALSE;
  stack = (char *)malloc(50 * 1024);  // allow 50 k

}

ManPage::~ManPage(void)
{
  cout << "~ManPage:\n";
  if (progname)
    {
      free(progname);
      progname = NULL;
    }
  if (section_num) 
    {
      free(section_num);
      section_num = NULL;
    }
  if (filename) 
    {
      free(filename);
      filename = NULL;
    }
  filebuffer.clear();
  sourcebuffer.clear();

  if (man_window)
    {
      gtk_widget_destroy ( GTK_WIDGET(man_window) );
      man_window = NULL;
    }

  if (stack) free(stack);
}

void ManPage::do_load_manpage(char *fname, char *mpage)
{
using namespace misc;
  
string fb;
GError *error = NULL;
string command;
string format_command;
string name = fname;     // fname may be a pointer to filename so a copy is required

cout << "ManPage::do_load_manpage: ENTERED, fname= " << fname << " path= " << path << endl;

  
// The actual command used by "man man" at the command line is:-
//
// cd "/usr/share/man/en" && (echo ".pl 11i"; /usr/bin/gunzip -c '/usr/share/man/en/man1/man.1.gz') | /usr/bin/gtbl | /usr/bin/nroff -c -mandoc 2>/dev/null | /usr/bin/less -is 
//
// I'm wondering if we can use:-   
//            /usr/bin/gunzip -c '/usr/share/man/en/man1/man.1.gz') | /usr/bin/gtbl | /usr/bin/nroff -c -mandoc 2>/dev/null
// i.e. miss out:-
//            cd "/usr/share/man/en" && (echo ".pl 11i";
// Give it a try...
// What we need is the manpath 

if ( SUFEQ(fname, (char *)".gz" ) )
  {
  format_command = "gunzip -c '"; 
  }
else if ( SUFEQ(fname, (char *)".bz2" ) )
  {
   format_command = "bunzip2 -c '";
  }
else
  {
  format_command = "cat '";
  }

format_command += fname;
format_command +=  "'"; 
 
command = "cd  \"";
command += path;
command += "\" && (echo \".pl 11i\"; ";                  
command += format_command;                  
command +=") | /usr/bin/gtbl | /usr/bin/nroff -c -mandoc 2>/dev/null";
      
mpagename = mpage;


//cout << "ManPage::do_load_manpage 1a" << endl;


if (filename)
  {
  g_free(filename);
  filename = NULL;
  }

//cout << "ManPage::do_load_manpage 1b" << endl;


//if (filebuffer)
//  {
//  g_free(filebuffer);
//  filebuffer = NULL;
//  }


//cout << "ManPage::do_load_manpage 1c" << endl;


if (!sourcebuffer.empty())
  {
//  cout << "ManPage::do_load_manpage 1d" <<  endl;
  sourcebuffer.clear();
  } 

//cout << "ManPage::do_load_manpage 2" << endl;

filename = make_string((char *)name.c_str());

// THE UNFORTUNATE problem is that there is no way of
// knowing  exactly what the original encoding was.
// How can it be discerned?
// Maybe assume it is the current locale
// This seems to fix it...
// ===================================================
fb = get_file(command);
  
// cout << "ManPage::load_manpage: fb= " << fb << endl;
 

//cout << "ManPage::do_load_manpage 3" << endl;


if (!fb.empty())
  {
  gsize *bytes_written;
//  filebuffer = g_convert(fb, strlen(fb), "UTF-8", "ISO-8859-1", 
//                            NULL, NULL, &error);
  filebuffer = g_locale_to_utf8(fb.c_str(), -1, NULL, NULL, NULL);

  if (filebuffer.empty())
    {
    filebuffer = fb;
    cerr << "Couldn't convert output of command \"" << command << "\" to UTF-8";
    cerr << " - using fb as is." << endl;
    }
//  else free(fb);
  }
else
  {
  cerr << "Couldn't obtain the manpage!" << endl;
  return;
  }

fblen = filebuffer.length();

//cout << "ManPage::do_load_manpage 4" << endl;


sourcebuffer = get_file(format_command);

//cout << "ManPage::do_load_manpage 5" << endl;


}

// NOTE: this is maybe still dodgy if the full pathname is actually a compressed link.
// ALSO: fname is the full pathname of the manpage file, from e.g. mandir? apropos?.
// Probably a good idea to load the source here also.
void ManPage::load_manpage(char *path, char *fname, char *mpage)
{
this->path = path;
do_load_manpage(fname, mpage);

    
    
  // JUST FOR NOW
  /**************************************************
  {
    char *c = filebuffer;
    char *p;


    while (*c)
      {
	p = c;
	for (int i=0; i<42; i++,p++)
	  {
	    if (*p)
	      {
		if (isprint(*p)) printf("%c", *p);
		else printf(".");
	      }
	    else break;
	  }
	printf(" :");

	p = c;
	for (int i=0; i<42; i++,p++)
	  {
	    if (*p)
	      {
		printf(" %02x", (unsigned char)(*p));
	      }
	    else break;
	  }

	printf("\n");

	if (*p) c = p;
	else break;
      }
  }
  **************************************************/



create_display(mpage);

insert_text();
 
}

void ManPage::load_manpage(char *path, char *fname, char *mpage, Edit *e)
{
edit = e;
load_manpage(path, fname, mpage);
}

GtkTextBuffer *ManPage::create_display(char *mpage)
{
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *button;
  GtkWidget *sw;
  string buffer;

// cout << "ManPage::create_display - ENTERED" << endl;

  man_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_show (man_window);
  g_signal_connect (G_OBJECT (man_window), "destroy",
         G_CALLBACK (CloseDialog),
          this);
  buffer = "MANPAGE: ";
  buffer += mpage;
  gtk_window_set_title(GTK_WINDOW(man_window), buffer.c_str());
  
  gtk_container_set_border_width(GTK_CONTAINER(man_window), 0);
  
  gtk_widget_set_size_request(man_window,
			       gman->prefs.get_manpage_width(),
			       gman->prefs.get_manpage_height() );

 
  if (gman->icon_pixbuf)
    {
      gtk_window_set_icon( GTK_WINDOW(man_window), gman->icon_pixbuf);
    }
  
  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);// vbox = gtk_vbox_new(FALSE, 10);      // Not homogeneous, 10 pixel spacing
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (man_window), vbox);
 
  text = gtk_text_view_new();
  tbuf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
  gtk_widget_show (text);
  gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE); // This can't be edited

  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (sw),
        GTK_POLICY_AUTOMATIC,
        GTK_POLICY_AUTOMATIC);
  gtk_widget_show (sw);

  gtk_container_add (GTK_CONTAINER (sw), text);
  gtk_box_pack_start (GTK_BOX (vbox), sw, TRUE, TRUE, 0);
 
 
  create_tags (tbuf); 
 
  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2); // hbox = gtk_hbox_new(FALSE, 2);        // Not homogeneous, 2 pixel spacing
  gtk_widget_show (hbox);
 
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);

  button = gtk_check_button_new_with_label("Comments");
  gtk_widget_show (button);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
                                                
  g_signal_connect (G_OBJECT (button), "toggled",
		    G_CALLBACK (fix_comment_flag), this);
  gtk_box_pack_start (GTK_BOX(hbox), 
		      button, TRUE, TRUE, 0);

  button = gtk_button_new_with_label ("Reload");
  gtk_box_pack_start (GTK_BOX(hbox), 
		      button, TRUE, TRUE, 0);  
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_reload), this);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("Search");
  gtk_box_pack_start (GTK_BOX(hbox), 
		      button, TRUE, TRUE, 0);  
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_search), this);
  gtk_widget_show (button);
  
  button = gtk_button_new_with_label ("Edit");
   gtk_box_pack_start (GTK_BOX(hbox), 
		      button, TRUE, TRUE, 0);  
  gtk_widget_show (button);
 
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_edit), this);

  // --- Create the "Print" button
  button = gtk_button_new_with_label ("Print");
  
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (man_print), this);
  gtk_box_pack_start (GTK_BOX(hbox), 
		      button, TRUE, TRUE, 0);  
  gtk_widget_show (button);
  
  // --- Create the "Print Preview" button
  button = gtk_button_new_with_label ("Print Preview");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_printpreview), this);
  gtk_box_pack_start (GTK_BOX(hbox), 
		      button, TRUE, TRUE, 0);  
  gtk_widget_show (button);

  // --- Create the "Print Preferences" button
  button = gtk_button_new_with_label ("Print Preferences");  
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_printprefs), this);
  gtk_box_pack_start (GTK_BOX(hbox), 
		      button, TRUE, TRUE, 0);  
  gtk_widget_show (button);


  // --- Create the "Close" button
  button = gtk_button_new_with_label ("Close");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (CloseDialog), this);
  // --- Allow it to be the default button
  gtk_widget_set_can_default(GTK_WIDGET(button), TRUE); //GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX(hbox), 
		      button, TRUE, TRUE, 0);
  
  // --- Make the button the default button
  gtk_widget_grab_default (button);
  
  // --- Make the button visible
  gtk_widget_show (button);

  return tbuf;
}


// We have to add comments if required after the 1st line.
// Then it is best to start at the beginning again
int ManPage::process_comments(void)
{
using namespace misc;

int n, num_comments = 0;

cout << "ManPage::process_comments:" << endl;

if (!sourcebuffer.empty())
    {
    char *p = (char *)sourcebuffer.c_str();
    char *s;

    while (*p)
        {
        if ( (STREQN(p, (char *)".\\\"", 3)) ||
                (STREQN(p, (char *)"'\\\"", 3)) )       // COMMENT
            {
            n = 0;
            s = p + 3;
            while (*s == ' ') s++;
            p = s;

            while ( (*p) && (*p != '\n') )
                {
                n++;
                p++;
                }

            if (*p == '\n')
                {
                n++;
                p++;
                }

            if (n)
                {
                // This green isn't dark enough        
                gtk_text_buffer_insert_with_tags_by_name (tbuf, &iter,
                        s, n,
                        "comment_text", NULL);

                num_comments++;
                }
            }
        else p++;
        }
    }
return(num_comments);
}






void ManPage::insert_text(void)
{
GtkTextIter start, end;
  


// our variables
int n;                          // number of characters to be inserted
char *p = (char *)filebuffer.c_str();           // current position in filebuffer
char *e = (char *)filebuffer.c_str() + fblen;   // address of terminal zero byte
int linenum = 0;
int prev_type = 0;              // 0=normal, 1=bold, 2=italic
char current_char[MB_CUR_MAX];  // the character pointed to before p
int cc_len;                     // the length of current_char

gtk_text_buffer_get_bounds(tbuf, &start, &end);

gtk_text_buffer_delete(tbuf, &start, &end);

// cout <<  "ManPage::insert_text - ENTERED" << endl; 
  
/* 
 * get start of buffer; each insertion will revalidate the
 * iterator to point to just after the inserted text.
*/
gtk_text_buffer_get_iter_at_offset (tbuf, &iter, 0);


while (*p && p < e)
    {
    if (*p == '\n')
        {
        if (comment_flag && (!linenum)) // if we have processed the 1st line
            {
            gtk_text_buffer_insert_with_tags_by_name (tbuf,
						      &iter,
						      "\n\n",
						      -1,
						      NULL, NULL);
 

            if ( (linenum += process_comments()) ) // If there were comments
                {
                p = (char *)filebuffer.c_str();                         // start again at beginning
                prev_type = 0;
                }
            else linenum++; // no comments
            continue;
            }
        linenum++;
        }
 
    // To get here it should already be UTF-8 encoded
    // so we should be able to simply get the next UTF-8 character
    
    cc_len = mbrlen(p, MB_CUR_MAX, NULL);
    if (cc_len > 0)
        {
        for (n=0; n<cc_len; n++)
            current_char[n] = *p++;
        }
    else
        {
        // DODGY
        // =====
        
        // -1 or -2 are errors
        // There should be no errors because it was created internally...
        // BUT you never know!
        // 0 is NULL which should be EOF 

	  if (cc_len)
	    {
	      printf("INVALID CHARACTER\n");
	      p++;
	    }
          else
	    {
	      // ? ERROR - I think it means EOF
	      printf("STRANGE ERROR - EOF?\n");
	    }
	}



    // BOLD
    // Do this check first then the possibility of Italic Underscore is gone
    // NOTE: EXTRA_BOLD is XbsXbsX
    // It seems it can be repeated several times
    // Not sure how to represent this
    if ( *p=='\b' )
        {
        if (STREQN(current_char, p+1, cc_len))
            {
            // BOLD
            // If we use monospace it is a different width from normal anyway
            // This is a pain
            // It should really be the same width
            // We must make it so if possible
            gtk_text_buffer_insert_with_tags_by_name (tbuf, &iter,
                    current_char,  cc_len,
                    "monospace", "bold", "black_foreground", NULL);
 
            p += cc_len + 1;
            while ( *p=='\b' )  // For now we skip over this (EXTRA_BOLD instead)
                {
                  if (STREQN(current_char, p+1, cc_len))
                    {
                    p += cc_len + 1;
                    }
                }
            continue;
            }

        // SUPERIMPOSED CHARACTERS
        // not sure if this is possible...
        
        //      printf("insert_text: char was superimposed\n");

        //display_colour = (valid_char == 1) ? &man_blue : &text->style->black;
        //put_a_char(fontnum, display_colour, display_char, normal);

        //display_colour = (valid_char2 == 1) ? &man_blue : &text->style->black;
        //put_a_char(f, display_colour, d2, normal);
        if ( current_char[0] != '_' )  // _ then backspace is Italic if not bold
            {
            gtk_text_buffer_insert_with_tags_by_name (tbuf, &iter,
                    current_char, cc_len,
                    "black_foreground", "monospace", NULL);
                    
            p++; // Pass over the backspace
           
            cc_len = mbrlen(p, MB_CUR_MAX, NULL);
            if (cc_len > 0)
                {
                for (n=0; n<cc_len; n++)
                    current_char[n] = *p++; // Get the character, advance pointer
                }

            gtk_text_buffer_insert_with_tags_by_name (tbuf, &iter,
                    current_char, cc_len,
                    "blue_foreground", "monospace", NULL);
  
            continue;
            }
        }
        
        
        
    // UNDERSCORE and BACKSPACE (Italic)
    if ( (current_char[0] == '_') && *p=='\b' && *(p+1) )
        {
        p++;   // Pass over the backspace to the italic character
        cc_len = mbrlen(p, MB_CUR_MAX, NULL);
        if (cc_len > 0)
            {
            for (n=0; n<cc_len; n++)
                current_char[n] = *p++;  // Get the character, advance pointer
            }
      
        // Display the ITALIC character
        gtk_text_buffer_insert_with_tags_by_name (tbuf, &iter,
            current_char, cc_len,
            "italic", "monospace", NULL);
 
        continue;
        }



    // NORMAL, includes newlines
    gtk_text_buffer_insert_with_tags_by_name (tbuf, &iter,
            current_char, cc_len,
            "monospace", NULL);

    }




// Apply word_wrap tag to whole buffer
gtk_text_buffer_get_bounds (tbuf, &start, &end);
gtk_text_buffer_apply_tag_by_name (tbuf, "word_wrap", &start, &end);
 
}
    



























