/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       manpage.h                                                 *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       19-Jul-2011 21.19.04 -36000 0                             *
*                                                                           *
*File Summary:                                                              *
* Header                                                                    *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#ifndef __MANPAGE_H__
#define __MANPAGE_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "gman.h"
#include "mandir.h"
#include "printprefs.h"
#include "edit.h"

using namespace std;

class ManPage
{
 public:
  ManPage(GMan *g);
  ~ManPage(void);

  void reload(GtkWidget *widget, ManPage *mp, bool same_line_number);
  void do_load_manpage(char *fname, char *mpage);
  void load_manpage(char *path, char *fname, char *mpage);
  void load_manpage(char *path, char *fname, char *mpage, class Edit *e);
  int process_comments(void);
  void insert_text(void);
  void create_tags(GtkTextBuffer *buffer);

  static void stop_timer(void);
  static void start_timer(ManPage *mp);
  static gint check_preview(gpointer mp);
  static gint timer;

  GtkWidget *man_window;
  GtkWidget *text;   // TextView

  Edit *edit;

  GMan *gman;
    
  char *mpagename;
  string tname;        // temporary filename for preview
  string command;      // complete command to obtain preview
  char *filename;      // full pathname of this manpage
  char *path;          // path portion of this filename
  string filebuffer;   // the processed file
  string sourcebuffer; // the source (uncompressed)
  size_t fblen;
  class Search *search;
  GtkTextBuffer *tbuf;
  bool search_initialised;

 private:
  static void callback_reload(GtkWidget *widget, ManPage *mp);
  static void fix_comment_flag (GtkWidget *widget, ManPage *mp);
  static void callback_search(GtkWidget *widget, ManPage *mp);
  static void callback_edit(GtkWidget *widget, ManPage *mp);
  static void callback_printprefs(GtkWidget *widget, ManPage *mp);
  static void callback_printpreview(GtkWidget *widget, ManPage *mp);
  static void CloseDialog (GtkWidget *widget, ManPage *mp);
  static void man_print(GtkWidget *widget, ManPage *mp);
  GtkTextBuffer *create_display(char *mpage);
  static int do_preview(void *arg);

  static void Trap(int signum);
  static pid_t thread_pid;
  char *stack;
  bool done_preview;

  char *progname;
  char *section_num;
  
  GtkTextIter iter;

  gboolean comment_flag;

 protected:


};







#endif // _MANPAGE_H
