/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       misc.cpp                                                  *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       15-Mar-2013 11.31.01 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Miscellaneous functions                                                   *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include "misc.h"

namespace misc
{
  
  /*********************************************
   * Return TRUE if pathname is a regular file *
   *********************************************/

  gboolean isfile(char *pathname)
  {
    struct stat statbuf;

    if (! (stat(pathname, &statbuf)) )
      {
	if (S_ISREG(statbuf.st_mode))
	  {
	    return(TRUE);
	  }
      }
    return(FALSE); // Not a regular file or ERROR
  }

  /**********************************************
   * Return TRUE if pathname name exists at all *
   **********************************************/

  gboolean isname(char *pathname)
  {
    struct stat statbuf;

    if (! (stat(pathname, &statbuf)) )
	if ( S_ISREG(statbuf.st_mode) ||
	     S_ISDIR(statbuf.st_mode) ||
	     S_ISCHR(statbuf.st_mode) ||
	     S_ISBLK(statbuf.st_mode) ||
	     S_ISFIFO(statbuf.st_mode) ||
	     S_ISLNK(statbuf.st_mode) ||
	     S_ISSOCK(statbuf.st_mode))
	  {
	    return(TRUE);
	  }

    return(FALSE); // Not a name in the file system or ERROR
  }
 


  /********************************
   * Return length of file fname  *
   * in bytes.			  *
   * Return FALSE if it fails.	  *
   ********************************/
  gboolean get_fsize(char *fname, size_t *size)
  {
    struct stat statbuf;

    if (stat(fname, &statbuf) == 0)
      {
	*size = statbuf.st_size;
	return(TRUE);
      }
    return(FALSE);
  }
/*****************************************************************************

char *get_file(char *command, char *fname)
{
size_t size = 0;
char *buffer = NULL;
size_t len = 0;

if (fname) get_fsize(fname, &size);
if (!size) size = 1024;

if ( (buffer = (char *)calloc(size + 1, 1)) ) // zero byte
  {
	FILE *fp;
	char pipebuffer[PIPE_BUF];
	size_t pipelen;


	if ( (fp = popen(command, "r")) )
	  {
	  while ( fgets(pipebuffer, PIPE_BUF, fp) )
      {
		  pipelen = strlen(pipebuffer);
		
		  if ( (pipelen + len) >= size )
		    {
		  
		    // We need to enlarge the buffer!
		    size_t newsize = size +(4 * PIPE_BUF);
		    char *newbuffer = (char *)calloc(newsize + 1, 1);
		  
		    if (newbuffer)
          {
			    memcpy(newbuffer, buffer, size);
			    free(buffer);
			    buffer = newbuffer;
			    size = newsize;
		      }
		    else
		      {
			    // ERROR: Can't allocate memory
		      }		     
		    }

		  strcat(buffer, pipebuffer);
		  len += pipelen;
      }
    pclose(fp);
	  }
	else
	  {
	    // ERROR: can't open pipe
	  }
  }
else
  {
	// ERROR: Can't allocate memory	
  }

// This assumes no zero in the file, i.e. a text file of some kind
if (buffer)
  {
	char *newbuffer;
	if ( (newbuffer = make_string(buffer)) ) // remove unwanted memory
	  {
    free(buffer);
    buffer = newbuffer;
	  }
  }
return(buffer);
}
*****************************************************************************/

// Pipe output of a command to a string buffer
string get_file(string command)
{
string buffer;
FILE *fp;
char pipebuffer[PIPE_BUF];

if ( (fp = popen(command.c_str(), "r")) )
  {
  while ( fgets(pipebuffer, PIPE_BUF, fp) )
    {
    buffer += pipebuffer;
    }

  if (ferror(fp))
    {
    // ERROR CONDITION
    buffer.clear();
    }
  }
else
  {
	// ERROR: can't open pipe
  // Empty string is returned
  }
pclose(fp);
return(buffer);  // returns a zero-terminated string
}

/*****************************************************************************/

  /*******************************************
   * GList *get_tail_node(GLIst *list)       *
   * Return the tail node of a list or NULL. *
   *******************************************/
  GList *get_tail_node(GList *list)
  {
    GList *g = list;
    
    while(g && g->next)
      {
	g = g->next;
      }
    return(g);
  }

  // Return the index of a list entry or -1 if not present
  int compare_list(GList *list, char *string)
  {
    GList *gl = list;
    
    if (string)
      {
	for (int n=0; gl; gl=gl->next, ++n)
	  {
	    if (STREQ(string, (char *)gl->data)) return(n); 
	  }
      }
    return(-1);
  }


// Return index if string in list or -1 if not in list
gint glist_index(GList *list, char *string)
{
  GList *node;
  gint index;

  for(node=list, index=0; node; node=node->next, index++)
    {
      if (STREQ( (char *)node->data, string)) return(index);
    }
  return(-1);
}

  // Free the data in a list then free the list itself
void free_list(GList *list)
  {
      using namespace std;
      
    GList *gl = list;

//  cout << "free_list\n";
  
    while (gl)
      {
//            cout << "Freeing " << (char *)gl->data << endl;
            
        free(gl->data);
        gl = gl->next;
      }
    g_list_free(list);
    list = NULL;

//   cout << "free_list: LEAVING\n";
  }

// Create a style
/*
GtkStyle *create_style(GdkColor fg, GdkColor text, GdkColor bg)
{
GtkStyle *defaultstyle;
GtkStyle *newstyle;
int i;

defaultstyle = gtk_widget_get_default_style();
newstyle = gtk_style_copy(defaultstyle);

// Copy for each state
for (i=0; i<5; i++)
  {
  newstyle->fg[i] = fg;
  newstyle->text[i] = text;
  newstyle->bg[i] = bg;
  }

return(newstyle);
}
*/

/***************************************
 * Colourize a button                  *
 * fg bg prelight stock_id             *
 * Do nothing for NULL colour pointers *
 ***************************************/
void decorate_widget(GtkWidget *widget, GdkRGBA *fg_normal, GdkRGBA *bg_normal,  GdkRGBA *bg_prelight, const gchar *stock_id)
{
if (fg_normal) gtk_widget_override_color(GTK_WIDGET(widget), GTK_STATE_FLAG_NORMAL, fg_normal); 
if (bg_normal) gtk_widget_override_background_color(GTK_WIDGET(widget), GTK_STATE_FLAG_NORMAL, bg_normal);  
if (bg_prelight) gtk_widget_override_background_color(GTK_WIDGET(widget), GTK_STATE_FLAG_PRELIGHT, bg_prelight);

if (GTK_IS_BUTTON(widget) && (stock_id) && *(stock_id))
  {
  gtk_button_set_image(GTK_BUTTON(widget), gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_BUTTON));
  // Make sure the image is displayed!
  gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(widget)))); 
  }
}


// ====================================================
// CreateCombobox
//
// Create a drop down combobox with a few items in it.
//
// ====================================================

void SetComboTextList(GtkWidget *combo, GList *list)
{
  GList *text;

  for (text=list; text; text=text->next)
    {
    gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo), (gchar *)text->data);
    }

  gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);  // Make visible

}


GtkWidget *CreateCombobox (GList *list)
{
  GtkWidget *combo;

  // --- Make a combo box.

  // USE: GtkWidget*  gtk_combo_box_new_text(void);
  // A straight combo is deprecated...
  // This will mean that a lot of the code will have to be re-written
  // Use the following to manipulate the combobox
  // gtk_combo_box_append_text()
  // gtk_combo_box_insert_text()
  // gtk_combo_box_prepend_text()
  // gtk_combo_box_remove_text()
  // It accepts the "changed" signal same as combo


//  combo = gtk_combo_box_new_text();
  combo = gtk_combo_box_text_new();



  SetComboTextList(combo, list);

  // --- Make it visible
  gtk_widget_show(combo);
  
  return (combo);
}


// Create a spinner (integer values)
GtkWidget *create_spinner(int s_start, int s_min, int s_max)
{
GtkAdjustment *spinner_adj;

spinner_adj = (GtkAdjustment *) gtk_adjustment_new (s_start, s_min, s_max, 1.0, 0.0, 0.0);

// creates the spinner, with no decimal places
return( gtk_spin_button_new (spinner_adj, 1.0, 0) );
}

/*****************************************************************************
 * Create an integer hscale and spinbutton combination that work together    *
 * Returns an hbox containing them and fills in hspinner with the spinbutton *
 *****************************************************************************/
GtkWidget *create_hspinner(GtkWidget **hspinner, double s_start, double s_min, double s_max)
{
//GtkWidget *hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0); //gtk_hbox_new(FALSE, 0); // Not homogeneous, 0 pixel spacing
//gtk_box_set_homogeneous(GTK_BOX(hbox), FALSE);

//typedef enum {
//  GTK_BUTTONBOX_SPREAD = 1,
//  GTK_BUTTONBOX_EDGE,
//  GTK_BUTTONBOX_START,
//  GTK_BUTTONBOX_END,
//  GTK_BUTTONBOX_CENTER
//} GtkButtonBoxStyle;
//void gtk_button_box_set_child_non_homogeneous(GtkButtonBox *widget, GtkWidget *child, gboolean non_homogeneous);
	
GtkWidget *hbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL); //gtk_hbox_new(FALSE, 0); // Not homogeneous, 0 pixel spacing
gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_START);


// NOTE: The last 2 MUST BE  0.0, 0.0); otherwise the hscale misbehaves
GtkAdjustment *adj = (GtkAdjustment *) gtk_adjustment_new (s_start, s_min, s_max, 0.001, 0.0, 0.0);
//	                                                       gdouble value,
//                                                         gdouble lower,
//                                                         gdouble upper,
//                                                         gdouble step_increment,
//                                                         gdouble page_increment,
//                                                         gdouble page_size);


*hspinner = gtk_spin_button_new (adj, .001, 3); // gdouble climb_rate, number od digits to display

GtkWidget *hscale = gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, GTK_ADJUSTMENT(adj));
//gtk_range_set_update_policy(GTK_RANGE (hscale), GTK_UPDATE_CONTINUOUS); // I thnk this is the default anyway
gtk_scale_set_digits(GTK_SCALE(hscale), 0);
gtk_scale_set_value_pos(GTK_SCALE(hscale), GTK_POS_RIGHT);
gtk_scale_set_draw_value(GTK_SCALE(hscale),FALSE);

gtk_widget_show(hbox);
gtk_widget_show(*hspinner);
gtk_widget_show(hscale);

//                                           expand   fill   pad
gtk_box_pack_start(GTK_BOX(hbox), hscale,     TRUE,   TRUE,   0); 
gtk_widget_set_size_request(hscale, 295, -1);

gtk_box_pack_start(GTK_BOX(hbox), *hspinner,  TRUE,   FALSE,   0);

gtk_button_box_set_child_non_homogeneous(GTK_BUTTON_BOX(hbox), hscale, TRUE);
//gtk_button_box_set_child_non_homogeneous(GTK_BUTTON_BOX(hbox), *hspinner, FALSE);

return hbox;
}





GtkWidget *create_bar_submenu (GtkWidget *menubar, char *name)
{
    GtkWidget *menuitem;
    GtkWidget *submenu;
 
    menuitem = gtk_menu_item_new_with_label (name);

    gtk_menu_shell_append(GTK_MENU_SHELL(menubar), menuitem);
    gtk_widget_show (menuitem);

    submenu = gtk_menu_new ();
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), submenu);

    return (submenu);
}


GtkWidget *create_menuitem (GtkWidget *window,
                            GtkWidget *menu, 
				                    char *name,
                            GtkAccelGroup *accel_group,
				                    char *accel,
                            //GtkTooltips *tooltips,
				                    char *tip, 
				                    void *func,
				                    gpointer data)
{
GtkWidget *menuitem;

if (name && strlen (name))
  {
  menuitem = gtk_menu_item_new_with_label (name);
  g_signal_connect (G_OBJECT (menuitem), "activate",
                    G_CALLBACK(func), data);
  }
else
  {
  menuitem = gtk_menu_item_new ();
  }

gtk_menu_shell_append(GTK_MENU_SHELL (menu), menuitem);
gtk_widget_show (menuitem);

if (accel_group == NULL) 
  {
  accel_group = gtk_accel_group_new ();
	gtk_window_add_accel_group( GTK_WINDOW(window),  GTK_ACCEL_GROUP(accel_group));
  }

if (accel && accel[0] == '^')
  {
  gtk_widget_add_accelerator(menuitem, 
                                    "activate", 
                                    accel_group,
                                    accel[1], 
                                    GDK_CONTROL_MASK,
                                    GTK_ACCEL_VISIBLE);
  }

else if (accel && accel[0]=='!')
  {
  gtk_widget_add_accelerator (menuitem, 
                                    "activate", 
                                    accel_group,
                                    accel[1], 
                                    (GdkModifierType)(GDK_SHIFT_MASK | GDK_CONTROL_MASK),
                                    GTK_ACCEL_VISIBLE);

  }



if (tip && strlen (tip))
  {
//  gtk_tooltips_set_tip (tooltips, menuitem, tip, NULL);
	gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), tip);
  }

return (menuitem);
}



  /*******************************************
   * int suficmp(char * string, char *suffix *
   * Compare the suffix "suffix" with the    *
   * terminal part of "string" and return as *
   * a normal case insensitive compare.      *
   *******************************************/
  
  int suficmp(char *string, char *suffix)
  {
    int rc;
    
    int len_suffix = strlen(suffix);
    int len_string = strlen(string);
    
    if ( (rc = len_string - len_suffix) >= 0 )
      {
	rc = strcasecmp(&string[rc], suffix);
      }
    return(rc);
  }

  /*******************************************
   * int sufcmp(char * string, char *suffix  *
   * Compare the suffix "suffix" with the    *
   * terminal part of "string" and return as *
   * a normal case-sensitive compare.        *
   *******************************************/
  
  int sufcmp(char *string, char *suffix)
  {
    int rc;

    int len_suffix = strlen(suffix);
    int len_string = strlen(string);

    if ( (rc = len_string - len_suffix) >= 0 )
      {
	rc = strcmp(&string[rc], suffix);
      }
    return(rc);
  }


// Create a string in memory, needs to be freed later
char *make_string(char *string)
{
if (string && strlen(string))
  {
	int len = strlen(string) + 1;
	char *s = (char *)malloc(len);
	
	if (s)
	  {
	    strcpy(s, string);
	  }
	return(s);
  }

return(NULL);
}

/****************************
 * Convert string to double *
 ****************************/
double string_to_double( const std::string& s )
{
std::istringstream i(s);
double x;
if (!(i >> x))  return 0;
return x;
} 


// I don't seem to be able to make the following functions "inline"
int STREQ(char *x, char *y)
  {
    return(strcmp(x,y)==0);
  }

int STRIEQ(char *x, char *y)
  {
    return(strcasecmp(x,y)==0);
  }
int STREQN(char *x, char *y, int n)
  {
    return(strncmp(x,y,n)==0);
  }
int STRIEQN(char *x, char *y, int n)
  {
    return(strncasecmp(x,y,n)==0);
  }
int SUFEQ(char *x, char *y)
  {
    return(sufcmp(x,y)==0);
  }
int SUFIEQ(char *x, char *y)
  {
    return(suficmp(x,y)==0);
  }

}
