/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       prefs.h                                                   *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       13-Mar-2013 14.26.38 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Header                                                                    *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#ifndef __PREFS_H__
#define __PREFS_H__

#include <iostream>
#include <gtk/gtk.h>

using namespace std;

struct Media
{
  string size;
  int width;
  int height;
};

class Prefs

{
  friend class Printprefs;   // Allow printprefs to access everything
  friend class Windowsizes;  // Allow Windowsizes to access everything
  friend class Printfile;    // Allow printfile to access everything
  friend class EditPrefs;    // Allow EditPrefs to access everything

 public:
  Prefs(void);
  ~Prefs(void);
  void load(void);
  void save(void);

  gint get_main_window_height(void);
  gint get_main_window_width(void);
  gint get_about_height(void);
  gint get_about_width(void);
  gint get_apropos_height(void);
  gint get_apropos_width(void);  
  gint get_manpage_height(void);
  gint get_manpage_width(void);
  gint get_mandir_height(void);
  gint get_mandir_width(void);
  gint get_edit_width(void);
  gint get_edit_height(void);

  guint get_autosave_seconds(void);
  int get_autosave_changes(void);

  string *get_preview_command(void);

  static struct Media media[30];
  static string fonts[3];

    
 private:
  bool IsWhiteSpace(char x); // except for newline
  bool IsSeparator(char x);
  void getright(string &s, string &v);
  bool getline(fstream &ifp, string &line);
  int get_int(string &v);
  void i2hex(int n, string &v, bool uppercase);

  string pathname;          // pathname of saved preferences file

  string printername;
  struct Media papersize;
  int l_margin;
  int r_margin;
  int t_margin;
  int b_margin;
  string fontname;
  int fontsize;
  bool fixed;
  string preview_command;

	

		
  gint main_window_width;
  gint main_window_height;
  gint manpage_width;
  gint manpage_height;
  gint mandir_width;
  gint mandir_height;
  gint edit_width;
  gint edit_height;
  gint apropos_width;
  gint apropos_height;
  gint printprefs_width;
  gint printprefs_height;
  gint displayprefs_width;
  gint displayprefs_height;
  gint editprefs_width;
  gint editprefs_height;
  gint about_width;
  gint about_height;

  guint autosave_seconds;
  int autosave_changes;

  string d_printername;     // Default values
  struct Media d_papersize;
  int d_l_margin;
  int d_r_margin;
  int d_t_margin;
  int d_b_margin;
  string d_fontname;
  int d_fontsize;
  bool d_fixed;
  string d_preview_command;



		
  double n_p_red;
  double n_p_green;
  double n_p_blue;

  double i_p_red;
  double i_p_green;
  double i_p_blue;
		
  double b_p_red;
  double b_p_green;
  double b_p_blue;

  double eb_p_red;
  double eb_p_green;
  double eb_p_blue;
	
	double er_p_red;
  double er_p_green;
  double er_p_blue;

		
static  double d_n_p_red;
static  double d_n_p_green;
static  double d_n_p_blue;

static  double d_i_p_red;
static  double d_i_p_green;
static  double d_i_p_blue;
		
static  double d_b_p_red;
static  double d_b_p_green;
static  double d_b_p_blue;

static  double d_eb_p_red;
static  double d_eb_p_green;
static  double d_eb_p_blue;

static	double d_er_p_red;
static  double d_er_p_green;
static  double d_er_p_blue;
	






		
  int d_main_window_width;
  int d_main_window_height;
  int d_manpage_width;
  int d_manpage_height;
  int d_mandir_width;
  int d_mandir_height;
  int d_edit_width;
  int d_edit_height;
  int d_apropos_width;
  int d_apropos_height;
  int d_printprefs_width;
  int d_printprefs_height;
  int d_displayprefs_width;
  int d_displayprefs_height;
  int d_editprefs_width;
  int d_editprefs_height;
  int d_about_width;
  int d_about_height;

  guint d_autosave_seconds;
  int d_autosave_changes;

 protected:

};




#endif // _PREFS_H
