/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       printprefs.cpp                                            *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       15-Mar-2013 11.44.45 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Printer preferences module                                                *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include "printprefs.h"
#include "prefs.h"
#include "gman.h"
#include "misc.h"

using namespace std;
using namespace misc;

GMan *Printprefs::gman = NULL;


Printprefs::~Printprefs(void)
{
  cout << "~Printprefs:\n";

  if (window)
    {
      gtk_widget_destroy ( GTK_WIDGET(window) );
      window = NULL;
    }

  if (printer_list) free_list(printer_list);
  printer_list = NULL;

  if (font_list)  g_list_free(font_list);
  font_list = NULL;

  if (paper_size_list) g_list_free(paper_size_list);
  paper_size_list = NULL;

  if (timer)
    {
//    gtk_timeout_remove(timer);
		g_source_remove (timer);
    }

  if (pixbuf) g_object_unref(G_OBJECT(pixbuf));
}

// if it receives the "destroy" signal, i.e. window close gadget
void Printprefs::close_dialog(GtkWidget *widget, Printprefs *pp)
{
  //  cout <<  "Printprefs::close_dialogs\n";
  pp->gman->delete_printprefs(pp);
}


void Printprefs::quit_printprefs(GtkWidget *widget, Printprefs *pp)
{
  GtkWidget *w = pp->window;

  pp->window = NULL;

  //  cout <<  "Printprefs::quit_printprefs\n";

  // This sends the "destroy" signal which invokes close_dialog

  gtk_widget_destroy ( GTK_WIDGET(w) );  
	
}

Printprefs::Printprefs(GMan *g)
{	
  cout << "Printprefs:\n";

  gman = g;
  printer_list = NULL;
  paper_size_list = NULL;
  font_list = NULL;
  window = NULL;
  timer = 0;


  // This list causes a crash when it is freed
  for (int n=0; n<30; ++n)
    {
      // Try without using make_string, then only
      // the list itself has to be freed.
      paper_size_list =
	g_list_append(
		      paper_size_list,
		      (char *)( Prefs::media[n].size).c_str() );
    }


  for (int n=0; n<3; n++)
    {
      font_list = g_list_append(font_list,
				(char *)(Prefs::fonts[n]).c_str() );
    }

  da_normal = da_italic = da_bold = da_ebold = da_err = NULL;

  create_display((char *)"Printer Preferences");
}




void Printprefs::paint_da(GtkWidget *da, double red, double green, double blue, Printprefs *pp)
{
static GdkColor *color = NULL;
int width, height;
	
cairo_t *cr = gdk_cairo_create (gtk_widget_get_window( da )); //->window);
cairo_set_source_rgb(cr, red, green, blue);
cairo_paint (cr);
cairo_destroy (cr);
}






gint Printprefs::repaint(gpointer data)
{
  Printprefs *pp = (Printprefs *)data;


  pp->paint_da(pp->da_normal, pp->nr, pp->ng, pp->nb, pp );
  pp->paint_da(pp->da_italic, pp->ir, pp->ig, pp->ib, pp );
  pp->paint_da(pp->da_bold, pp->br, pp->bg, pp->bb, pp );
  pp->paint_da(pp->da_ebold, pp->ebr,pp-> ebg, pp->ebb, pp );
  pp->paint_da(pp->da_err, pp->err, pp->erg, pp->erb, pp );


  return TRUE;
}


// Make only the selected button active

void Printprefs::callback_toggle_mutex(GtkWidget *widget, Printprefs *pp)
{
static GtkWidget *current_widget = pp->normal_tog;
		
//cout << "Printprefs::callback_toggle_mutex\n";

if (widget != current_widget)
	{
	decorate_widget(GTK_WIDGET(widget), pp->tog_fg, pp->tog_bg, NULL, 0);
	current_widget = 	widget;

	if (widget == pp->normal_tog)
		{
		pp->da_current = pp->da_normal;
		pp->r = &(pp->nr);
		pp->g = &(pp->ng);
		pp->b = &(pp->nb);
		}
	else
		{
  	decorate_widget(GTK_WIDGET(pp->normal_tog), &black, &white, 0, NULL);
		}

	if (widget == pp->italic_tog)
		{
		pp->da_current = pp->da_italic;
		pp->r = &(pp->ir);
		pp->g = &(pp->ig);
		pp->b = &(pp->ib);
		}
	else
		{
		decorate_widget(GTK_WIDGET(pp->italic_tog), &black, &white, 0, NULL);
		}

	if (widget == pp->bold_tog)
		{
		pp->da_current = pp->da_bold;
		pp->r = &(pp->br);
		pp->g = &(pp->bg);
		pp->b = &(pp->bb);
		}
	else
		{
		decorate_widget(GTK_WIDGET(pp->bold_tog), &black, &white, 0, NULL);
		}

	if (widget == pp->ebold_tog)
		{
		pp->da_current = pp->da_ebold;
		pp->r = &(pp->ebr);
		pp->g = &(pp->ebg);
		pp->b = &(pp->ebb);
		}
	else
		{
		decorate_widget(GTK_WIDGET(pp->ebold_tog), &black, &white, 0, NULL);
		}

	if (widget == pp->err_tog)
		{
		pp->da_current = pp->da_err;
		pp->r = &(pp->err);
		pp->g = &(pp->erg);
		pp->b = &(pp->erb);
		}
	else
		{
		decorate_widget(GTK_WIDGET(pp->err_tog), &black, &white, 0, NULL);
		}
	}
	
gtk_spin_button_set_value(GTK_SPIN_BUTTON(pp->red_hspinner),   *(pp->r) );
gtk_spin_button_set_value(GTK_SPIN_BUTTON(pp->green_hspinner), *(pp->g) );
gtk_spin_button_set_value(GTK_SPIN_BUTTON(pp->blue_hspinner),  *(pp->b) );                            
}

void Printprefs::callback_general_defaults(GtkWidget *widget, Printprefs *pp)
{
  gint index;
  cout << "Printprefs::callback_general_defaults\n";

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(pp->left_margin_spinner),  gman->prefs.d_l_margin);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(pp->right_margin_spinner), gman->prefs.d_r_margin);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(pp->top_margin_spinner),   gman->prefs.d_t_margin);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(pp->bottom_margin_spinner),gman->prefs.d_b_margin);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(pp->fontsize_spinner),     gman->prefs.d_fontsize);

  index = glist_index(pp->font_list, (char *)(gman->prefs.d_fontname).c_str() );
  if (index >= 0) gtk_combo_box_set_active(GTK_COMBO_BOX(pp->font_combo), index);
  index = glist_index(pp->paper_size_list, (char *)(gman->prefs.d_papersize.size).c_str() );
  if ((index >= 0) && (index < 30))
    {
      gtk_combo_box_set_active(GTK_COMBO_BOX(pp->paper_sizes_combo), index);
    }

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pp->fw_checkbutton), gman->prefs.d_fixed);

  gtk_entry_set_text (GTK_ENTRY(pp->pv_entry), (char *)gman->prefs.d_preview_command.c_str());

  //cout << "  - window width = " << pp->window->allocation.width << endl;
  //cout << "  - window height= " << pp->window->allocation.height << endl;
}

void Printprefs::callback_default_text_colours(GtkWidget *widget, Printprefs *pp)
{
  cout << "Printprefs::callback_default_text_colours\n";

  pp->nr = gman->prefs.d_n_p_red;   //colour >> 16) & 0xFF;
  pp->ng = gman->prefs.d_n_p_green; //colour >> 8) & 0xFF;
  pp->nb = gman->prefs.d_n_p_blue;  //colour & 0XFF;

  pp->ir = gman->prefs.d_i_p_red;   //pcolour >> 16) & 0xFF;
  pp->ig = gman->prefs.d_i_p_green; //pcolour >> 8) & 0xFF;
  pp->ib =  gman->prefs.d_i_p_blue; //colour & 0XFF;

  pp->br = gman->prefs.d_b_p_red;   //colour >> 16) & 0xFF;
  pp->bg = gman->prefs.d_b_p_green; //colour >> 8) & 0xFF;
  pp->bb = gman->prefs.d_b_p_blue;  //pcolour & 0XFF;

  pp->ebr = gman->prefs.d_eb_p_red;   //pcolour >> 16) & 0xFF;
  pp->ebg = gman->prefs.d_eb_p_green; //pcolour >> 8) & 0xFF;
  pp->ebb = gman->prefs.d_eb_p_blue;  //pcolour & 0XFF;

  pp->err = gman->prefs.d_er_p_red;   //pcolour >> 16) & 0xFF;
  pp->erg = gman->prefs.d_er_p_green; //pcolour >> 8) & 0xFF;
  pp->erb = gman->prefs.d_er_p_blue;  //pcolour & 0XFF;

  // Now set the hspinner widgets 
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(pp->red_hspinner),   *(pp->r) );
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(pp->green_hspinner), *(pp->g) );
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(pp->blue_hspinner),  *(pp->b) );                            

}


void Printprefs::do_save(GtkWidget *widget, Printprefs *pp)
{
cout << "Printprefs::do_save\n";

do_use(NULL, pp);
gman->prefs.save();
quit_printprefs(widget, pp);
}

void Printprefs::do_use(GtkWidget *widget, Printprefs *pp)
{
  gint index;

  cout << "Printprefs::do_use\n";

  // Put the values into gman->prefs

  if (pp->printers_combo)
    {
      index = gtk_combo_box_get_active(GTK_COMBO_BOX(pp->printers_combo));
      if (index >= 0) gman->prefs.printername = (char *)(g_list_nth(pp->printer_list, index))->data;
    }

  index = gtk_combo_box_get_active(GTK_COMBO_BOX(pp->paper_sizes_combo));

  gman->prefs.papersize.size = (char *)(g_list_nth(pp->paper_size_list, index))->data;
  gman->prefs.papersize.width = Prefs::media[index].width;
  gman->prefs.papersize.height = Prefs::media[index].height;

  index = gtk_combo_box_get_active(GTK_COMBO_BOX(pp->font_combo));
  gman->prefs.fontname = (char *)(g_list_nth(pp->font_list, index))->data;

  gman->prefs.fontsize = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pp->fontsize_spinner));

  gman->prefs.fixed = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pp->fw_checkbutton));

  gman->prefs.l_margin = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pp->left_margin_spinner));
  gman->prefs.r_margin = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pp->right_margin_spinner));
  gman->prefs.t_margin = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pp->top_margin_spinner));
  gman->prefs.b_margin = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pp->bottom_margin_spinner));

  gman->prefs.preview_command = (char *)gtk_editable_get_chars(GTK_EDITABLE(pp->pv_entry), 0, -1);

  // Printing colours
  gman->prefs.n_p_red = pp->nr;
  gman->prefs.n_p_green = pp->ng;
  gman->prefs.n_p_blue = pp->nb;

  gman->prefs.i_p_red = pp->ir;
  gman->prefs.i_p_green = pp->ig;
  gman->prefs.i_p_blue = pp->ib;

  gman->prefs.b_p_red = pp->br;
  gman->prefs.b_p_green = pp->bg;
  gman->prefs.b_p_blue = pp->bb;


  gman->prefs.eb_p_red = pp->ebr;
  gman->prefs.eb_p_green = pp->ebg;
  gman->prefs.eb_p_blue = pp->ebb;



  gman->prefs.er_p_red = pp->err;
  gman->prefs.er_p_green = pp->erg;
  gman->prefs.er_p_blue = pp->erb;




  if (widget) quit_printprefs(widget, pp);
	
}



GtkWidget *Printprefs::get_printers(void)
{
FILE *fp;
char buffer[1024];
char *p, *t;
GtkWidget *widget;


// Inspect the printcap file
// This assumes CUPS or LPRNG
// We need to add the LPRng method of print cap
// The original idea was that a printer should have 3 names:
//     1: A short name of at most 4 chars
//     2: A longer name that indicates the printer's owner
//     3: A descriptive name that identifies the printer to all users on the system
// If printing with lpr and the PRINTER environment variable is undefined
//     the first printer in printcap is used.

fp = fopen("/etc/printcap", "r");

if (fp)
  {
    while (fgets(buffer, 1024, fp) == buffer)
      {
        p = buffer;
        if (*p == '#') continue;
        
        while ( (*p == ' ') || (*p == '\t') ) p++;   // white space

        if (*p == ':') continue;   // It was a continued line in LPRng
        
        t = strstr(p, "|");
        if (!t) t = strstr(p, ":");
        
        if (t)
          {
            *t = '\0';
            printer_list = g_list_append(printer_list, make_string(p));
          }
      }
    fclose(fp);
  }
else
   {
   perror("Can't open etc/printcap\n");
   }

if (gman->prefs.printername.length())
  {
  gint index = glist_index( printer_list, (char *)(gman->prefs.printername).c_str() );
  if (index < 0) // if we have a printer preference and it's not in the list, add it
    {
    printer_list = g_list_append(printer_list, make_string((char *)(gman->prefs.printername).c_str()));
    }
  }

if (printer_list)
  {
  widget = printers_combo = CreateCombobox(printer_list);
  }
else
  {
  printers_combo = NULL;
  widget = gtk_label_new("NO PRINTERS FOUND!");
  gtk_widget_show (widget);
  }
return(widget);
}




void Printprefs::callback_colour_r(GtkWidget *widget, Printprefs *pp)
{  
*(pp->r) = gtk_spin_button_get_value(GTK_SPIN_BUTTON(pp->red_hspinner));
}

void Printprefs::callback_colour_g(GtkWidget *widget, Printprefs *pp)
{
*(pp->g) = gtk_spin_button_get_value(GTK_SPIN_BUTTON(pp->green_hspinner));
}

void Printprefs::callback_colour_b(GtkWidget *widget, Printprefs *pp)
{
*(pp->b) = gtk_spin_button_get_value(GTK_SPIN_BUTTON(pp->blue_hspinner));
}



void Printprefs::create_display(char *title)
{
GtkWidget *grid;
GtkWidget *widget;
GtkWidget *vbox, *hbox, *hbox2;
	
window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
gtk_widget_show (window);
g_signal_connect (G_OBJECT (window), "destroy",
        G_CALLBACK (close_dialog),
        this);

gtk_window_set_title (GTK_WINDOW (window), title);
  	
gtk_window_set_default_size(GTK_WINDOW(window),
 			       gman->prefs.printprefs_width, 
			       gman->prefs.printprefs_height);

gtk_window_set_modal(GTK_WINDOW(window), TRUE);  // Prevent interaction with other windows

gtk_container_set_border_width(GTK_CONTAINER(window), 0);

pixbuf = gtk_widget_render_icon_pixbuf(GTK_WIDGET(window),
				  GTK_STOCK_PREFERENCES,
				  GTK_ICON_SIZE_DIALOG);
	
if (pixbuf) gtk_window_set_icon( GTK_WINDOW(window), pixbuf);

vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
gtk_widget_show (vbox);
gtk_container_add (GTK_CONTAINER (window), vbox);

	
grid = gtk_grid_new();
gtk_grid_set_column_spacing(GTK_GRID(grid), 10);
gtk_grid_set_row_spacing(GTK_GRID(grid), 10);
gtk_grid_set_row_homogeneous(GTK_GRID(grid), TRUE);	
gtk_box_pack_start(GTK_BOX(vbox), grid, TRUE, FALSE, 0);	//expand, fill, padding
gtk_widget_show (grid);

widget = gtk_label_new("Printer: ");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .5);     // Left aligned, vertically central
gtk_widget_show (widget);


gtk_grid_attach(GTK_GRID(grid), widget, 0, 0, 1, 1);


widget =  get_printers();

// If there is a printer already chosen, select that one to show
if ( printers_combo && (gman->prefs.printername).length() )
   {
     gint index = glist_index( printer_list, (char *)(gman->prefs.printername).c_str() );
     if (index >= 0) gtk_combo_box_set_active(GTK_COMBO_BOX(widget), index);
     // if index < 0 then maybe no printer in prefs or printer in prefs isn't available
   }

gtk_grid_attach(GTK_GRID(grid), widget, 1, 0, 1, 1);


hbox2 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);
gtk_widget_show (hbox2);

widget = gtk_label_new("Font size: ");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .4);     // Left aligned, vertically central
gtk_widget_show (widget);
gtk_box_pack_start(GTK_BOX(hbox2), widget, TRUE, FALSE, 0);

fontsize_spinner = create_spinner(gman->prefs.fontsize, 0, 100);

gtk_widget_show (fontsize_spinner);
gtk_box_pack_start(GTK_BOX(hbox2), fontsize_spinner, TRUE, FALSE, 0);

gtk_grid_attach(GTK_GRID(grid), hbox2, 2, 0, 1, 1);
	

widget = gtk_label_new("Paper Size: ");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .5);     // Left aligned, vertically central
gtk_widget_show (widget);
//gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 0);

gtk_grid_attach(GTK_GRID(grid), widget, 0, 1, 1, 1);
	

paper_sizes_combo = CreateCombobox(paper_size_list);
// Fix up from gman->prefs
if ( (gman->prefs.papersize.size).length() )
    {
     gint index = glist_index( paper_size_list, (char *)(gman->prefs.papersize.size).c_str() );
     if ((index >= 0) && (index < 30))
       {
	 gtk_combo_box_set_active(GTK_COMBO_BOX(paper_sizes_combo), index);
       }
    }

gtk_grid_attach(GTK_GRID(grid), paper_sizes_combo, 1, 1, 1, 1);



fw_checkbutton = gtk_check_button_new_with_label("Force fixed width");
gtk_widget_show (fw_checkbutton);
gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fw_checkbutton), gman->prefs.fixed);
//gtk_box_pack_start(GTK_BOX(hbox), fw_checkbutton, TRUE, FALSE, 0);

gtk_grid_attach(GTK_GRID(grid), fw_checkbutton, 2,1, 1, 1);
	


widget = gtk_label_new("Font: ");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .3);     // Left aligned
gtk_widget_show (widget);

gtk_grid_attach(GTK_GRID(grid), widget, 0, 2, 1, 1);

	
font_combo = CreateCombobox(font_list);
if ( (gman->prefs.fontname).length() )
    {
     gint index = glist_index( font_list, (char *)(gman->prefs.fontname).c_str() );
     if (index >= 0) gtk_combo_box_set_active(GTK_COMBO_BOX(font_combo), index);
    }

gtk_grid_attach(GTK_GRID(grid), font_combo, 1, 2, 1, 1);
	


// ===========================================
// RGB SLIDERS for the different kinds of text
// ===========================================


hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
gtk_box_set_homogeneous (GTK_BOX(hbox), FALSE);
gtk_widget_show (hbox);
gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);	//expand, fill, padding


grid = gtk_grid_new();
gtk_grid_set_row_homogeneous(GTK_GRID(grid),TRUE);
gtk_grid_set_column_homogeneous(GTK_GRID(grid), TRUE);
gtk_box_pack_start(GTK_BOX(hbox), grid, TRUE, TRUE, 0);	//expand, fill, padding
gtk_widget_show (grid);

	
widget = gtk_label_new("Red:");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .3);     // Left aligned
gtk_widget_show (widget);

gtk_grid_attach(GTK_GRID(grid), widget, 0, 0, 1, 1);


widget = create_hspinner(&red_hspinner, 0.0, 0.0, 1.0); // red_hscale
gtk_widget_show (widget);

gtk_grid_attach(GTK_GRID(grid), widget, 1, 0, 8, 1);



	
widget = gtk_label_new("Green:");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .3);     // Left aligned
gtk_widget_show (widget);

gtk_grid_attach(GTK_GRID(grid), widget, 0, 1, 1, 1);
	
	
widget = create_hspinner(&green_hspinner, 0.0, 0.0, 1.0);
gtk_widget_show (widget);

gtk_grid_attach(GTK_GRID(grid), widget, 1, 1, 8, 1);

	
widget = gtk_label_new("Blue:");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .3);     // Left aligned
gtk_widget_show (widget);

gtk_grid_attach(GTK_GRID(grid), widget, 0, 2, 1, 1);
	


widget = create_hspinner(&blue_hspinner, 0.0, 0.0, 1.0);
gtk_widget_show (widget);
	

gtk_grid_attach(GTK_GRID(grid), widget, 1, 2, 8, 1);





	
//===================================
//               MARGINS
//===================================


grid = gtk_grid_new();
//gtk_grid_set_column_homogeneous(GTK_GRID(grid), TRUE);
gtk_grid_set_column_spacing(GTK_GRID(grid), 10);
gtk_box_pack_start(GTK_BOX(vbox), grid, TRUE, TRUE, 0);	//expand, fill, padding
gtk_widget_show (grid);


	
widget = gtk_label_new("MARGINS (point=1/72 inch)");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .5);     // left
gtk_widget_show (widget);
	
gtk_grid_attach(GTK_GRID(grid), widget, 0, 0, 3, 1);

	
widget = gtk_label_new("Left Margin: ");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .4);     // Left aligned
gtk_widget_show (widget);
	
	
gtk_grid_attach(GTK_GRID(grid), widget, 0, 1, 1, 1);


left_margin_spinner = create_spinner(gman->prefs.l_margin, 0,  5000);
gtk_widget_show(left_margin_spinner);
	
gtk_grid_attach(GTK_GRID(grid), left_margin_spinner, 1, 1, 2, 1);

	
widget = gtk_label_new("Right Margin: ");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .4);     // Left aligned
gtk_widget_show (widget);
	
gtk_grid_attach(GTK_GRID(grid), widget, 0, 2, 1, 1);

  
right_margin_spinner = create_spinner(gman->prefs.r_margin, 0,  5000);
gtk_widget_show(right_margin_spinner);
	
gtk_grid_attach(GTK_GRID(grid), right_margin_spinner, 1, 2, 2, 1);

	
widget = gtk_label_new("Top Margin: ");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .4);     // Left aligned
gtk_widget_show (widget);
	
gtk_grid_attach(GTK_GRID(grid), widget, 0, 3, 1, 1);

  
top_margin_spinner = create_spinner(gman->prefs.t_margin, 0,  5000); 
gtk_widget_show(top_margin_spinner);
	
gtk_grid_attach(GTK_GRID(grid), top_margin_spinner, 1, 3, 2, 1);

	
widget = gtk_label_new("Bottom Margin: ");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .4);     // Left aligned
gtk_widget_show (widget);
	
	
gtk_grid_attach(GTK_GRID(grid), widget, 0, 4, 1, 1);


bottom_margin_spinner = create_spinner(gman->prefs.b_margin, 0,  5000);
gtk_widget_show (bottom_margin_spinner);
//gtk_box_pack_start(GTK_BOX(hbox), bottom_margin_spinner, TRUE, FALSE, 0);	//expand, fill, padding	
	
	
gtk_grid_attach(GTK_GRID(grid), bottom_margin_spinner, 1, 4, 2, 1);

	
widget = gtk_label_new("Preview: ");
gtk_widget_show (widget);
gtk_misc_set_alignment(GTK_MISC (widget), 0, .5);     // Left aligned, vertically central
//gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 0);	//expand, fill, padding	
	
gtk_grid_attach(GTK_GRID(grid), widget, 0, 5, 1, 1);


	
pv_entry = gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(pv_entry), 100);
gtk_entry_set_text (GTK_ENTRY(pv_entry), (char *)gman->prefs.preview_command.c_str());
gtk_widget_show(pv_entry);
//gtk_box_pack_start(GTK_BOX(hbox), pv_entry, TRUE, FALSE, 0);	//expand, fill, padding	
	
gtk_grid_attach(GTK_GRID(grid), pv_entry, 1, 5, 2, 1);

	
widget = gtk_button_new_with_label ("GENERAL DEFAULTS");
gtk_widget_show (widget);
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK (callback_general_defaults),  this);
	
gtk_grid_attach(GTK_GRID(grid), widget, 0, 6, 3, 1);


//=====================
// Text colours
//=====================

widget = gtk_label_new("Colours ");
gtk_widget_show (widget);

	
gtk_grid_attach(GTK_GRID(grid), widget, 3, 0, 4, 1);


normal_tog = gtk_button_new_with_label("Normal Text");
gtk_widget_show (normal_tog);
g_signal_connect (G_OBJECT (normal_tog), "clicked",
  		    G_CALLBACK (callback_toggle_mutex), (gpointer)this);
	
gtk_grid_attach(GTK_GRID(grid), normal_tog, 3, 1, 3, 1);

	
da_normal = gtk_drawing_area_new();
gtk_widget_set_size_request(GTK_WIDGET(da_normal), 50, 50); //default size                                                         
gtk_widget_show(da_normal);
	
gtk_grid_attach(GTK_GRID(grid), da_normal, 6, 1, 1, 1);




italic_tog = gtk_button_new_with_label("Italic Text");
gtk_widget_show (italic_tog);
g_signal_connect (G_OBJECT (italic_tog), "clicked",
		    G_CALLBACK (callback_toggle_mutex), (gpointer)this);
	
gtk_grid_attach(GTK_GRID(grid), italic_tog, 3, 2, 3, 1);

	
da_italic = gtk_drawing_area_new();
gtk_widget_set_size_request(GTK_WIDGET(da_italic), 50, 50); //default size
gtk_widget_show(da_italic);
//gtk_box_pack_start(GTK_BOX(hbox), da_italic, TRUE, FALSE, 0);	//expand, fill, padding	
	
gtk_grid_attach(GTK_GRID(grid), da_italic, 6, 2, 1, 1);

	
ir = gman->prefs.i_p_red;    //pcolour >> 16) & 0xFF;
ig = gman->prefs.i_p_green; //pcolour >> 8) & 0xFF;
ib = gman->prefs.i_p_blue;  //pcolour & 0XFF;


bold_tog = gtk_button_new_with_label("Bold Text");
gtk_widget_show (bold_tog);
g_signal_connect (G_OBJECT (bold_tog), "clicked",
		    G_CALLBACK (callback_toggle_mutex), (gpointer)this);
	
gtk_grid_attach(GTK_GRID(grid), bold_tog, 3, 3, 3, 1);

	
da_bold = gtk_drawing_area_new();
gtk_widget_set_size_request(GTK_WIDGET(da_bold), 50, 50); //default size
gtk_widget_show(da_bold);	
	
gtk_grid_attach(GTK_GRID(grid), da_bold, 6, 3, 1, 1);

	
br = gman->prefs.b_p_red;   //pcolour >> 16) & 0xFF;
bg = gman->prefs.b_p_green; //pcolour >> 8) & 0xFF;
bb = gman->prefs.b_p_blue;  //pcolour & 0XFF;


ebold_tog = widget = gtk_button_new_with_label("Extra Bold");
gtk_widget_show (ebold_tog);
g_signal_connect (G_OBJECT (ebold_tog), "clicked",
		    G_CALLBACK (callback_toggle_mutex), (gpointer)this);
	
gtk_grid_attach(GTK_GRID(grid), ebold_tog, 3, 4, 3, 1);

	
da_ebold = gtk_drawing_area_new();
gtk_widget_set_size_request(GTK_WIDGET(da_ebold), 50, 50); //default size
gtk_widget_show(da_ebold);
	
gtk_grid_attach(GTK_GRID(grid), da_ebold, 6, 4, 1, 1);
	
ebr = gman->prefs.eb_p_red;   //pcolour >> 16) & 0xFF;
ebg = gman->prefs.eb_p_green; //pcolour >> 8) & 0xFF;
ebb = gman->prefs.eb_p_blue;  //pcolour & 0XFF;



err_tog = gtk_button_new_with_label("Error Text");
gtk_widget_show (err_tog);
g_signal_connect (G_OBJECT (err_tog), "clicked",
		    G_CALLBACK (callback_toggle_mutex), (gpointer)this);	
	
gtk_grid_attach(GTK_GRID(grid), err_tog, 3, 5, 3, 1);



da_err = gtk_drawing_area_new();
gtk_widget_set_size_request(GTK_WIDGET(da_err), 50, 50); //default size
gtk_widget_show(da_err);
	
gtk_grid_attach(GTK_GRID(grid), da_err, 6, 5, 1, 1);

	
err = gman->prefs.er_p_red;   //colour >> 16) & 0xFF;
erg = gman->prefs.er_p_green; //colour >> 8) & 0xFF;
erb =  gman->prefs.er_p_blue; //pcolour & 0XFF;

da_current = da_normal;
nr = gman->prefs.n_p_red;   //pcolour >> 16) & 0xFF;
ng = gman->prefs.n_p_green; //pcolour >> 8) & 0xFF;
nb = gman->prefs.n_p_blue;  //pcolour & 0XFF;

r = &nr;
g = &ng;
b = &nb;

gtk_spin_button_set_value(GTK_SPIN_BUTTON(red_hspinner),   nr);
gtk_spin_button_set_value(GTK_SPIN_BUTTON(green_hspinner), ng);
gtk_spin_button_set_value(GTK_SPIN_BUTTON(blue_hspinner),  nb);

g_signal_connect (G_OBJECT (red_hspinner), "value-changed",
		    G_CALLBACK (callback_colour_r), (gpointer)this);
g_signal_connect (G_OBJECT (green_hspinner), "value-changed",
  		    G_CALLBACK (callback_colour_g), (gpointer)this);
g_signal_connect (G_OBJECT (blue_hspinner), "value-changed",
  		    G_CALLBACK (callback_colour_b), (gpointer)this);

	
tog_fg =  &red;        //{1.0, 0.0, 0.0, 1.0};           //red;
tog_bg =  &white;      //{1.0, 1.0, 1.0, 1.0};           //white;
	
decorate_widget(GTK_WIDGET(normal_tog), tog_fg, tog_bg, NULL, 0);



widget = gtk_button_new_with_label ("DEFAULT COLOURS");
gtk_widget_show (widget);
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK (callback_default_text_colours),  this);
	
gtk_grid_attach(GTK_GRID(grid), widget, 3, 6, 4, 1);



hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
gtk_widget_show (hbox);
gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, FALSE, 0);	//expand, fill, padding


widget = gtk_button_new_with_label ("SAVE");
gtk_widget_show (widget);
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK (do_save),  this);
gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 0);	//expand, fill, padding	


widget = gtk_button_new_with_label ("USE (no save)");
gtk_widget_show (widget);
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK (do_use),  this);
gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 0);	//expand, fill, padding	


widget = gtk_button_new_with_label ("CANCEL");

// --- Make the button visible
gtk_widget_show (widget);

// --- Need to close the window if they press "Cancel"
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK ( quit_printprefs ),  this);
gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 0);	//expand, fill, padding	



// --- Allow it to be the default button
gtk_widget_set_can_default(GTK_WIDGET(widget), TRUE);

// --- Make the button the default button
gtk_widget_grab_default (widget);

timer = g_timeout_add(100, repaint, (gpointer)this);
gtk_widget_show (window);

}

