/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       search.cpp                                                *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       29-Mar-2013 11.20.36 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Search module for edit                                                    *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include "search.h"

using namespace misc;

Search::~Search(void)
{
cout << "~Search:\n";
if (ttext) g_free(ttext);
if (timer_id) stop_timer(this);
if (pixbuf) g_object_unref(G_OBJECT(pixbuf));
}

Search::Search(ManPage *m)
{
cout << "Search:\n";
manpage = m;
text = manpage->text;
edit = NULL;
display(FALSE); // not replacing, searchng only
}


Search::Search(gboolean replace, Edit *e)
{
manpage = NULL;
edit = e;
text = edit->text;

display(replace);
}

void Search::display(gboolean replace)
{    
GtkWidget *grid;	
GtkWidget *w;
gint extra = (replace) ? 1 : 0;

//cout << "Search::display - ENTERED\n";

ttext = NULL;
p = 0;
timer_id = 0;
searching = FALSE;
	
//cout << "Search::display - 1\n";


tbuf = (edit) ? edit->tbuf : manpage->tbuf;
num_chars = gtk_text_buffer_get_char_count(GTK_TEXT_BUFFER(tbuf));
old_char_index = -1;

//cout << "Search::display - 2\n";

window = gtk_dialog_new();
gtk_window_set_title(GTK_WINDOW(window),
		       (replace) ? "Search & Replace" : "Search");
gtk_widget_show (window);
g_signal_connect (G_OBJECT (window), "destroy",
		    G_CALLBACK (close_search),
		    (gpointer)this);




pixbuf = gtk_widget_render_icon_pixbuf(GTK_WIDGET(window),
				  ( (replace) ?
				    GTK_STOCK_FIND_AND_REPLACE :
				    GTK_STOCK_FIND ),
				  GTK_ICON_SIZE_DIALOG);
	

if (pixbuf) gtk_window_set_icon( GTK_WINDOW(window), pixbuf);

// ===================================================================


	grid = gtk_grid_new();
	gtk_grid_set_column_homogeneous(GTK_GRID(grid), TRUE);
	gtk_grid_set_row_homogeneous(GTK_GRID(grid), TRUE);

  gtk_widget_show(grid);
	
	gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG (window))), grid); 


   // Search string
  w = gtk_label_new("Search string: ");
  gtk_misc_set_padding (GTK_MISC (w), 2, 1);
  gtk_widget_show(w);

	gtk_grid_attach(GTK_GRID(grid), w, 0, 1, 2, 1);      //left, top, width,height
	

  search_entry = gtk_entry_new();
  gtk_widget_show(search_entry);

	gtk_grid_attach(GTK_GRID(grid), search_entry, 2, 1, 3, 1);      //left, top, width,height

	
   // If required, replace string
  if (replace)
    {

      w = gtk_label_new("Replace string: ");
      gtk_misc_set_padding (GTK_MISC (w), 2, 1);
      gtk_widget_show(w);

			gtk_grid_attach(GTK_GRID(grid), w, 0, 2, 2, 1);      //left, top, width,height

			
      replace_entry = gtk_entry_new();
      gtk_widget_show(replace_entry);

			gtk_grid_attach(GTK_GRID(grid), replace_entry, 2, 2, 3, 1);      //left, top, width,height

		}
  else replace_entry = NULL;

  forwards = gtk_check_button_new_with_label("Forwards");
  gtk_widget_show(forwards);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(forwards), TRUE);
  g_signal_connect (G_OBJECT (forwards), "clicked",
		    G_CALLBACK (callback_forwards_reverse),
		    (gpointer)this);
	

	gtk_grid_attach(GTK_GRID(grid), forwards, 0, 2+extra, 2, 1);      //left, top, width, height
 
  reverse = gtk_check_button_new_with_label("Reverse");
  gtk_widget_show(reverse);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(reverse), FALSE);
  g_signal_connect (G_OBJECT (reverse), "clicked",
		    G_CALLBACK (callback_forwards_reverse),
		    (gpointer)this);
	

 
	gtk_grid_attach(GTK_GRID(grid), reverse, 0, 3+extra, 2, 1);      //left, top, width, height
 
	status = gtk_label_new("");
  gtk_widget_show(status);
	
 
	gtk_grid_attach(GTK_GRID(grid), status, 2, 3+extra, 3, 1);      //left, top, width, height

	working_label = gtk_label_new("");
	gtk_grid_attach(GTK_GRID(grid), working_label, 2, 4+extra, 3, 1);      //left, top, width, height
  gtk_widget_show(working_label);


  match_case = gtk_check_button_new_with_label("Case Sensitive");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(match_case), FALSE);
  gtk_widget_show(match_case);

 
	gtk_grid_attach(GTK_GRID(grid), match_case, 0, 4+extra, 2, 1);      //left, top, width, height



  //=================================================
  // User input here - replace? yes no, but NOT SHOWN
  //=================================================
  replace_label = gtk_label_new("Replace?");
	
 
	gtk_grid_attach(GTK_GRID(grid), replace_label, 0, 5+extra, 2, 1);      //left, top, width, height



  yes_button = gtk_button_new_with_label("YES");
	

 
	gtk_grid_attach(GTK_GRID(grid), yes_button, 2, 5+extra, 1, 1);      //left, top, width, height

	
  g_signal_connect (G_OBJECT (yes_button), "clicked",
		    G_CALLBACK (callback_replace),
		    (gpointer)this);

	
  w = gtk_button_new_with_label("Next");
  gtk_widget_show(w);
  g_signal_connect (G_OBJECT (w), "clicked",
		    G_CALLBACK (callback_next),
		    (gpointer)this);


 
	gtk_grid_attach(GTK_GRID(grid), w, 3, 5+extra, 1, 1);      //left, top, width, height



  w = gtk_button_new_with_label("Cancel");
  gtk_widget_show(w);
	

	gtk_box_pack_start(GTK_BOX (gtk_dialog_get_action_area (GTK_DIALOG(window))), 
			    w, TRUE, TRUE, 0);

	
  g_signal_connect (G_OBJECT (w), "clicked",
		    G_CALLBACK (close_search),
		    (gpointer)this);


//cout << "Search::display - LEAVING\n";
}

void Search::close_search(GtkWidget *widget, Search *s)
{
if (s->searching)
	{
	s->searching = FALSE;
	}

else if (s->window)
  {
  GtkWidget *w = s->window;
  s->window = NULL;
  gtk_widget_destroy(GTK_WIDGET(w));

  if (s->edit) s->edit->search = NULL;
  else if (s->manpage) s->manpage->search = NULL;
  delete s;
  }
}

// ==============================================
// callback_replace needs to add to the undo list
// - assume this is editing!
// 1 - delete the new entry
// 2 - insert the old value
// Edit if edit is not NULL
// BUT undo and redo lists are private at present
// ==============================================

void Search::callback_replace(GtkWidget *widget, Search *s)
{
GtkTextIter start, end;
GtkTextMark *insert;
GtkTextMark *selection;
Undo u;

gtk_widget_hide(s->replace_label);
gtk_widget_hide(s->yes_button);
	
cout << "Search::callback_replace - ENTERED\n";


insert = gtk_text_buffer_get_insert(s->tbuf);
selection = gtk_text_buffer_get_selection_bound(s->tbuf);

gtk_text_buffer_get_iter_at_mark(s->tbuf, &start, insert);
gtk_text_buffer_get_iter_at_mark(s->tbuf, &end, selection);

	// If start and end are the same, what then?
	// It reports "NO SEARCH STRING" in "Next" before it gets here
//	gint gtk_text_iter_get_offset(const GtkTextIter *iter);


	
// Delete start-end - DON'T ADD replace here - should come back as Insert 1
gtk_text_buffer_delete(s->tbuf,
			 &start,
			 &end);




cout << "Search::callback_replace - DELETED start-end\n";


	
s->edit->replacing = TRUE; // Inform edit that this is a replace

  // Insert replace_string at start - ADD replace here - should come back as DELETE R 6
  gtk_text_buffer_insert_at_cursor(s->tbuf,
				   (char *)s->replace_string.c_str(), -1);

cout << "Search::callback_replace - INSERTED replace_string\n";

s->edit->inspect_undo_list();


	
  gtk_text_buffer_get_iter_at_mark(s->tbuf, &start, insert);

  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(s->forwards)))
    {
      // If forwards, cursor all start + length of replace_string
      gtk_text_iter_forward_visible_cursor_positions(&start,
						     s->replace_string.length());
    }

  // N.B. If reverse, cursor all at existing start
  gtk_text_buffer_select_range(s->tbuf,
			       &start,
			       &start);

cout << "\nSearch::callback_replace - LEAVING\n";

s->edit->inspect_undo_list();
}

/* Possible useful functions -------------------------------------
 
gint gtk_text_buffer_get_char_count(GtkTextBuffer *buffer);

gint gtk_text_buffer_get_line_count(GtkTextBuffer *buffer);

void gtk_text_buffer_get_iter_at_line_index(GtkTextBuffer *buffer,
                                            GtkTextIter *iter,
                                            gint line_number,
                                            gint byte_index);

void gtk_text_buffer_get_iter_at_line(GtkTextBuffer *buffer,
                                      GtkTextIter *iter,
                                      gint line_number);

void gtk_text_buffer_get_iter_at_offset(GtkTextBuffer *buffer,
                                        GtkTextIter *iter,
                                        gint char_offset);

-----------------------------------------------------------------*/



gboolean Search::callback_searching(gpointer data)  // Search *s
{
Search *s = (Search *)data;

//f (!s) cout << "Search::callback_searching - NO s" << endl;
//else cout << "Search::callback_searching - GOT s" << endl;

static int i = 0;
static int j = 0;
static string str;

decorate_widget(s->status, &darkblue, NULL,  NULL, NULL);
gtk_label_set_text(GTK_LABEL(s->status), "SEARCHING");

if (i < 10)
	{
	++i;

	str += "*";
	}
else
	{
	i = 0;
	++j;
	if (j > 9) j = 0;
	str.clear();
	}



	
switch (j)
	{
	case 1:
	decorate_widget(s->working_label, &darkred, NULL,  NULL, NULL);
	break;
			
	case 2:
	decorate_widget(s->working_label, &black, NULL,  NULL, NULL);
	break;
			
	case 3:
	decorate_widget(s->working_label, &lightgreen, NULL,  NULL, NULL);
	break;
			
	case 4:
	decorate_widget(s->working_label, &lightyellow, NULL,  NULL, NULL);
	break;
			
	case 5:
	decorate_widget(s->working_label, &pink, NULL,  NULL, NULL);
	break;
			
	case 6:
	decorate_widget(s->working_label, &lightblue, NULL,  NULL, NULL);
	break;
			
	case 7:
	decorate_widget(s->working_label, &red, NULL,  NULL, NULL);
	break;
			
	case 8:
	decorate_widget(s->working_label, &darkgreen, NULL,  NULL, NULL);
	break;
			
	case 9:
	decorate_widget(s->working_label, &brown, NULL,  NULL, NULL);
	break;
			
	case 0:
	default:
	decorate_widget(s->working_label, &darkblue, NULL,  NULL, NULL);
	break;

			
	}
	
gtk_label_set_text(GTK_LABEL(s->working_label), str.c_str());


return TRUE;
}

void Search::start_timer(Search *s)
{
cout << "Search::start_timer - ENTERED" << endl;
if (!s->timer_id)
	{

	s->timer_id = g_timeout_add(200, callback_searching, s); // 1/1000 second i.e 10 times per secnd
	cout << "Search::start_timer - starting: id= " << s->timer_id << endl;
	}
}

void Search::stop_timer(Search *s)
{
cout << "Search::stop_timer - ENTERED" << endl;

if (s->timer_id)
	{
	cout << "Search::stop_timer - stopping: id= " << s->timer_id << endl;

	g_source_remove(s->timer_id);
	s->timer_id = 0;	
	}
}


void Search::callback_next(GtkWidget *widget, Search *s)
{
GtkTextIter insert_iter, start, end;
GtkTextIter char_iter;
GtkTextMark *insert = gtk_text_buffer_get_insert(s->tbuf);
bool found = FALSE;
	
bool case_sensitive = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(s->match_case));    
	
cout << "Search::callback_next - ENTERED" << endl;



s->num_chars = gtk_text_buffer_get_char_count(GTK_TEXT_BUFFER(s->tbuf)); // May have changed...

if ( s->manpage && (!s->manpage->search_initialised) )
  {
  s->char_index = ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(s->forwards)) ) ? 0 : s->num_chars-1;
  s->manpage->search_initialised = TRUE;
  }
else if ( s->edit && (!s->edit->search_initialised) )
  {
  s->char_index = ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(s->forwards)) ) ? 0 : s->num_chars-1;
  s->edit->search_initialised = TRUE;
  }

else
  {
  if (s->search_string != gtk_entry_get_text(GTK_ENTRY(s->search_entry)))
    {
    // Re-initialise char_index
    s->char_index = ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(s->forwards)) ) ? 0 : s->num_chars-1;    
    }
  else
    {

    if ( s->char_inc != ((gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(s->forwards)) ) ? 1 : -1) )
      {
 
      // Only do this if there is a selection i.e. previously found
      if (gtk_text_buffer_get_has_selection(GTK_TEXT_BUFFER(s->tbuf)))
        {
        if (s->char_inc == -1) s->char_index += 2;       // if old is reverse, go forwards 1
        else if  (s->char_inc == 1) s->char_index -= 2; // if old is forwards, go back 1
        if (s->char_index < 0) s->char_index = 0;                                  // Not before start
        else if (s->char_index > (s->num_chars-1)) s->char_index = s->num_chars-1; // Not after end
        }  
      }
    }
  }

//------------------------------------------------------------------

s->char_inc = ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(s->forwards)) ) ? 1 : -1;

s->search_string = gtk_entry_get_text(GTK_ENTRY(s->search_entry));
s->replace_string = (char *)((s->replace_entry) ?
			       gtk_entry_get_text(GTK_ENTRY(s->replace_entry)) : "");
size_t len = (size_t)s->search_string.length();


if ((s->num_chars <= 0) || (len==0))
  {
  // NOT FOUND
  gtk_label_set_text(GTK_LABEL(s->status), (len==0) ? "NO SEARCH STRING!" : "NOT FOUND");
	gtk_label_set_text(GTK_LABEL(s->working_label), "");

	decorate_widget(s->status, &red, NULL,  NULL, NULL);
	
  return;
  }
        
//cout << "Search::callback_next - ENTERED : char_index= " << s->char_index << endl;


gtk_label_set_text(GTK_LABEL(s->status), "");
gtk_widget_hide(s->replace_label);
gtk_widget_hide(s->yes_button);
gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(s->tbuf), &start, &end);

//gboolean  gtk_text_iter_is_end(const GtkTextIter *iter);

//cout << "Search::callback_next - num_chars= " << s->num_chars << endl;
//sleep(1);
s->start_timer(s);
s->searching = TRUE;
	
while (s->searching && (s->char_index>=0) && (s->char_index < s->num_chars) )
  {
  // Get char_iter at this position
		
//	cout << "Search::callback_next - char_index/num_chars= " << s->char_index << "/" << s->num_chars << endl;
			
	while (gtk_events_pending ()) gtk_main_iteration ();
		
  gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(s->tbuf),
                                   &char_iter,
                                   s->char_index);

	if (gtk_text_iter_is_end(&char_iter)) break;
                        
  // Get text at this iter
  s->ttext = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(s->tbuf), &char_iter, &end, FALSE);

  if ( case_sensitive )
    {
    // CASE SENSITIVE SEARCH
    //substring = strstr( c, search_string );
    if (strncmp(s->ttext, s->search_string.c_str(), len) == 0)
      {
      found = TRUE;
      }
    }
  else
    {
    // CASE INSENSITIVE SEARCH
    if (strncasecmp(s->ttext, s->search_string.c_str(), len) == 0)
      {
      found = TRUE;
      }
    }

  if (found)
    {
    // FOUND - Mark the place and make it visible
//    GtkTextIter start, end;
    string str;


    // FOUND, got char_iter for s->ttext

    for (int n=0; n<len; ++n)
	    {
	    str += s->ttext[n];
	    }

 //   cout << "Search::callback_next -FOUND, Got str= " << str << endl;

    if (gtk_text_iter_forward_search( &char_iter,
					     (char *)str.c_str(),
					     GTK_TEXT_SEARCH_TEXT_ONLY,
					     &start,
					     &end,
					     NULL))
      {
			s->stop_timer(s);

      found = TRUE;
      gtk_label_set_text(GTK_LABEL(s->status), "FOUND");
			decorate_widget(s->status, &darkgreen, NULL,  NULL, NULL);
   		gtk_label_set_text(GTK_LABEL(s->working_label), "");

	    gtk_text_buffer_select_range(s->tbuf,
				       &start,
				       &end);

 //	    cout << "Search::callback_next - done gtk_text_buffer_select_range" << endl;
          
      gtk_text_buffer_move_mark(GTK_TEXT_BUFFER(s->tbuf),
                               insert,
                               &char_iter);

          
//	    cout << "Search::callback_next - done gtk_text_buffer_move_mark" << endl;


	    // Make sure it is visible
	    // =======================

      gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(s->text),
				       insert,
				       0.0,
				       FALSE,
				       0.0,
				       0.0);
     
	    if (s->replace_entry)
	      {

        // Check if user wants to replace this string

	      gtk_widget_show(s->replace_label);
	      gtk_widget_show(s->yes_button);

	      }

      s->old_char_index = s->char_index;
      s->char_index += s->char_inc;  // next position

      if (s->char_index < 0) s->char_index = 0;                                  // Not before start
      else if (s->char_index > (s->num_chars-1)) s->char_index = s->num_chars-1; // Not after end

      return;
      }
    }
	g_free(	s->ttext );
	s->ttext = NULL;
  s->char_index += s->char_inc;
  } // END of while

s->stop_timer(s);

if (s->searching == FALSE) // if "Cancel" was clicked
	{
	s->close_search(NULL, s);
	}
else
	{
	s->searching = FALSE;

	gtk_label_set_text(GTK_LABEL(s->status), "NOT FOUND");
	decorate_widget(s->status, &red, NULL,  NULL, NULL);
	gtk_label_set_text(GTK_LABEL(s->working_label), "");

	if (s->char_inc == -1) s->char_index = s->old_char_index = 0;
	else s->char_index = s->old_char_index = s->num_chars-1;



	// Clear selection
	gtk_text_buffer_select_range(s->tbuf, &start, &start);
	}

return;
}


void Search::callback_forwards_reverse(GtkWidget *widget, Search *s)
{
if (widget == s->reverse)
	{
	if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(s->reverse)) )
		{
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(s->forwards), FALSE);
		}
	else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(s->forwards), TRUE);
	}
else
	{
	if (  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(s->forwards)) )
		{
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(s->reverse), FALSE);
		}
	else
		{
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(s->reverse), TRUE);
		}
	}
}
