/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       tfile.cpp                                                 *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       21-Mar-2009 12.05.05 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Mainly functions for temporary files                                      *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <sstream>
#include <iomanip>
#include <sys/stat.h>

#include "tfile.h" 

using namespace std;

TFile::TFile(void)
{
cout << "TFile:\n";
}
        
TFile::~TFile(void)
{
cout << "~TFile:\n";
}

 // Raise base to the n of base      
int TFile::power(int n, int base)
{
  int p;

  for (p=1; n>0; --n)
    {
    p *= base;
    }

  return(p);
}


string TFile::convert_i2str(int i, int pad)
{
ostringstream ost;
int p = (pad <=0 ) ? 1 : pad;
ost <<  setw(p) << setfill('0') << dec << i;
return ost.str();
}

string TFile::convert_i2str(int i)
{
return convert_i2str(i, 1);
}


// Return a filename only or NULL if failure
string *TFile::tname(string *pathname, int n, string *suffix)
{
static string fname;
struct stat statbuf;

int max = power(10, n);
  
// i represents the suffix put into a_val
for (int i=0; i<max; i++)
    {
    fname = *pathname + convert_i2str(i, n) + (*suffix);

    if (stat(fname.c_str(), &statbuf)) // if it doesn't exist
      return &fname;
    }
return NULL;
}

string *TFile::tname(string *pathname, int n)
{
  string suffix = "";
  return(tname(pathname, n, &suffix));
}

// Return the filename (string, not char *) of filestream ofp or NULL if failure
string *TFile::tfile(string *pathname, fstream *ofp, int n, string *suffix)
{
static string fname;
struct stat statbuf;

int max = power(10, n);
  
// i represents the suffix put into a_val
for (int i=0; i<max; i++)
    {
    fname = *pathname + convert_i2str(i, n) + (*suffix);


    cout << "TFile::tfile: checking fname: " << fname << endl;


    if (stat(fname.c_str(), &statbuf)) // if it doesn't exist
        {
        ofp->open(fname.c_str(), ios::out | ios::binary | ios::trunc );
        if ( ofp->is_open() ) return(&fname);
        }
    }
return(NULL);
}



string *TFile::tfile(string *pathname, fstream *ofp, int n)
{
string suffix = "";
return( tfile(pathname, ofp, n, &suffix) );
}



string *TFile::tfile(string *pathname, fstream *ofp)
{
return( tfile(pathname, ofp, 3) );
}

int TFile::do_copy(fstream *ofp, fstream *ifp)
{
if (ifp && ofp && ifp->is_open() && ofp->is_open() )
    {
    char c;
      
    while (ofp && ifp->get(c))
        ofp->put(c);
    return(1);  
    }
return(0);
}


int TFile::filecopy(string *dname, string *sname, int overwrite)
{
fstream ofile;
int rc;

fstream ifile( sname->c_str(), ios::in  | ios::binary );
if (overwrite)
    {
    ofile.open(dname->c_str(), ios::out | ios::binary | ios::trunc );
    }
else ofile.open( dname->c_str(), ios::out | ios::binary );
rc = (do_copy(&ofile, &ifile));
ofile.close();
ifile.close();
return(rc);
}

int TFile::overwritefile(string *dname, string *sname)
{
return(filecopy(dname, sname, 1));
}

int TFile::filecopy(string *dname, string *sname)
{
return(filecopy(dname, sname, 0));
}

string *TFile::copy(string *pathname, int n)
{
static string fname;
struct stat statbuf;

int max = power(n, 10);

// i represents the suffix put into a_val
for (int i=0; i<max; i++)
    {
    fname = *pathname + convert_i2str(i, n);

    if (stat(fname.c_str(), &statbuf)) // if it doesn't exist
        {
         if (filecopy(&fname, pathname, 0))
            {
            return(&fname);
            }
        // otherwise try the next possible
        }
    }
return(NULL); 
}

string *TFile::copy(string *pathname)
{
return(copy(pathname, 3));
}
