/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gmanpage                                                  *
*Project Version: 1.2.3                                                     *
*Copyright Owner: David M. Balean <linprogs@exemail.com.au>                 *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* iunder the terms of the GNU General Public License as published by        *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Saturday 27th April 2013                                  *
*                                                                           *
*Start DateStamp: 21-Mar-2009 12.12.52 -36000 3600                          *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
*                                                                           *
*Project Summary:                                                           *
* View manual pages and optionally create/edit them                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       windowsizes.cpp                                           *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       11-Mar-2013 13.09.11 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Window sizes module (user preferences)                                    *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <gdk/gdkkeysyms.h>

#include "windowsizes.h"
#include "misc.h"
#include "gman.h"

using namespace misc;


Windowsizes::~Windowsizes(void)
{
  cout << "~Windowsizes:\n";

  if (pixbuf) g_object_unref(G_OBJECT(pixbuf));
}

Windowsizes::Windowsizes(GMan *g)
{	
  cout << "Windowsizes:\n";
  gman = g;

  create_display();
}

void Windowsizes::close_dialog (GtkWidget *widget, Windowsizes *ws)
{
 ws->gman->delete_windowsizes(ws);
}

void Windowsizes::quit_displayprefs(GtkWidget *widget, Windowsizes *ws)
{
  GtkWidget *w = ws->window;
  ws->window = NULL;	
  gtk_widget_destroy ( GTK_WIDGET(w) );
}


void Windowsizes::get_user_data(Windowsizes *ws)
{
ws->gman->prefs.main_window_width   = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->main_window_width_spinner));
ws->gman->prefs.main_window_height  = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->main_window_height_spinner));

ws->gman->prefs.manpage_width       = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->manpage_width_spinner));
ws->gman->prefs.manpage_height      = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->manpage_height_spinner));

ws->gman->prefs.edit_width          = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->edit_width_spinner));
ws->gman->prefs.edit_height         = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->edit_height_spinner));
  
ws->gman->prefs.apropos_width       = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->apropos_width_spinner));
ws->gman->prefs.apropos_height      = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->apropos_height_spinner));

ws->gman->prefs.printprefs_width    = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->printprefs_width_spinner));
ws->gman->prefs.printprefs_height   = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->printprefs_height_spinner));

ws->gman->prefs.displayprefs_width  = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->displayprefs_width_spinner));
ws->gman->prefs.displayprefs_height = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->displayprefs_height_spinner));

ws->gman->prefs.editprefs_width     = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->editprefs_width_spinner));
ws->gman->prefs.editprefs_height    = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->editprefs_height_spinner));

ws->gman->prefs.about_width         = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->about_width_spinner));
ws->gman->prefs.about_height        = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ws->about_height_spinner));
}



void Windowsizes::do_use(GtkWidget *widget, Windowsizes *ws)
{
ws->get_user_data(ws);
if (widget) quit_displayprefs(widget, ws);
}

void Windowsizes::do_save(GtkWidget *widget, Windowsizes *ws)
{
cout << "Windowsizes::do_save\n";
ws->get_user_data(ws);
do_use(NULL, ws);
ws->gman->prefs.save();
quit_displayprefs(widget, ws);
}



void Windowsizes::default_main_window_size(GtkWidget *widget, Windowsizes *ws)
{
  cout << "Windowsizes::default_main_window_size\n";
  
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->main_window_width_spinner), ws->gman->prefs.d_main_window_width);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->main_window_height_spinner), ws->gman->prefs.d_main_window_height);
 
}


void Windowsizes::default_manpage_size(GtkWidget *widget, Windowsizes *ws)
{
  cout << "Windowsizes::default_manpage_size\n";

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->manpage_width_spinner), ws->gman->prefs.d_manpage_width);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->manpage_height_spinner), ws->gman->prefs.d_manpage_height);
}

void Windowsizes::default_mandir_size(GtkWidget *widget, Windowsizes *ws)
{
  cout << "Windowsizes::default_mandir_size\n";

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->mandir_width_spinner), ws->gman->prefs.d_mandir_width);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->mandir_height_spinner), ws->gman->prefs.d_mandir_height);
}

void Windowsizes::default_edit_size(GtkWidget *widget, Windowsizes *ws)
{
  cout << "Windowsizes::default_edit_size\n";

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->edit_width_spinner), ws->gman->prefs.d_edit_width);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->edit_height_spinner), ws->gman->prefs.d_edit_height);
}

void Windowsizes::default_apropos_size(GtkWidget *widget, Windowsizes *ws)
{
  cout << "Windowsizes::default_apropos_size\n";

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->apropos_width_spinner), ws->gman->prefs.d_apropos_width);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->apropos_height_spinner), ws->gman->prefs.d_apropos_height);
}

void Windowsizes::default_printprefs_size(GtkWidget *widget, Windowsizes *ws)
{
  cout << "Windowsizes::default_printprefs_size\n";

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->printprefs_width_spinner), ws->gman->prefs.d_printprefs_width);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->printprefs_height_spinner), ws->gman->prefs.d_printprefs_height);
}

void Windowsizes::default_displayprefs_size(GtkWidget *widget, Windowsizes *ws)
{
  cout << "Windowsizes::default_displayprefs_size\n";

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->displayprefs_width_spinner), ws->gman->prefs.d_displayprefs_width);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->displayprefs_height_spinner), ws->gman->prefs.d_displayprefs_height);
}

void Windowsizes::default_ediprefs_size(GtkWidget *widget, Windowsizes *ws)
{
cout << "Windowsizes::default_editprefs_size\n";

gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->editprefs_width_spinner), ws->gman->prefs.d_editprefs_width);
gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->editprefs_height_spinner), ws->gman->prefs.d_editprefs_height);
}

void Windowsizes::default_about_size(GtkWidget *widget, Windowsizes *ws)
{
  cout << "Windowsizes::default_about_size\n";

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->about_width_spinner), ws->gman->prefs.d_about_width);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ws->about_height_spinner), ws->gman->prefs.d_about_height);
}




void Windowsizes::create_display(void)
{

//GtkWidget *table;
GtkWidget *widget;
GtkWidget *vbox, *hbox;
GtkWidget *column[4];

window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
gtk_widget_show (window);
g_signal_connect (G_OBJECT (window), "destroy",
		    G_CALLBACK (close_dialog),
		    this);

  gtk_window_set_title (GTK_WINDOW (window), "Window Sizes");

//  gtk_widget_set_size_request (window,
//			       gman->prefs.displayprefs_width,
//			       gman->prefs.displayprefs_height);
	
gtk_window_set_default_size(GTK_WINDOW (window),
             gman->prefs.displayprefs_width,
			       gman->prefs.displayprefs_height);

		
gtk_container_set_border_width(GTK_CONTAINER(window), 0);

// Prevent interaction with other windows
gtk_window_set_modal(GTK_WINDOW(window), TRUE);


pixbuf = gtk_widget_render_icon_pixbuf(GTK_WIDGET(window),
				  GTK_STOCK_PREFERENCES,
				  GTK_ICON_SIZE_DIALOG);
	
if (pixbuf) gtk_window_set_icon( GTK_WINDOW(window), pixbuf);


vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0); //vbox = gtk_vbox_new(FALSE, 0);
gtk_box_set_homogeneous(GTK_BOX(vbox), FALSE);
gtk_container_add (GTK_CONTAINER (window), vbox);
	
//gtk_widget_show (vbox);

hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
gtk_box_set_homogeneous(GTK_BOX(hbox), FALSE);
gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0); // expand, fill, padding

//gtk_container_add (GTK_CONTAINER (vbox), hbox);


// 10 rows, 4 columns

for (int i=0; i<4; ++i)
	{
	column[i] = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
	gtk_box_set_homogeneous(GTK_BOX(column[i]), TRUE); //(n<7) ? TRUE: FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), column[i], TRUE, TRUE, 0); // expand, fill, padding
	}
	
//  table = gtk_table_new(100, 90, TRUE);  // 100 rows, 100 columns, homogeneous

//  gtk_container_add(GTK_CONTAINER(vbox), table);
//  gtk_widget_show (table);

	
//----------
// HEADINGS
//----------
widget = gtk_label_new(" ");
gtk_box_pack_start(GTK_BOX(column[0]), widget, FALSE, FALSE, 0); // expand, fill, padding

widget = gtk_label_new("WIDTH");
gtk_misc_set_alignment(GTK_MISC (widget), .5, .5);
gtk_box_pack_start(GTK_BOX(column[1]), widget, FALSE, FALSE, 0); // expand, fill, padding
	
widget = gtk_label_new("HEIGHT");
gtk_misc_set_alignment(GTK_MISC (widget), .5, .5);
gtk_box_pack_start(GTK_BOX(column[2]), widget, FALSE, FALSE, 0); // expand, fill, padding
	
widget = gtk_label_new(" ");
gtk_box_pack_start(GTK_BOX(column[3]), widget, FALSE, FALSE, 0); // expand, fill, padding
	
//  widget = gtk_label_new("WIDTH");
//  gtk_misc_set_alignment(GTK_MISC (widget), .5, .5);
//  gtk_widget_show (widget);
//  gtk_table_attach(GTK_TABLE(table), widget,
//		   21, 35, 2, 10,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                                         // xpadding, ypadding
//
//  widget = gtk_label_new("HEIGHT");
//  gtk_misc_set_alignment(GTK_MISC (widget), .5, .5);
//  gtk_widget_show (widget);
//  gtk_table_attach(GTK_TABLE(table), widget,
//		   40, 55, 2, 10,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding


// CHANGE ALL THESE ENTRIES TO SPINNERS

//============
// MAIN WINDOW
//============
widget = gtk_label_new("Main");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .5);
gtk_box_pack_start(GTK_BOX(column[0]), widget, FALSE, FALSE, 0); // expand, fill, padding


	
//  gtk_widget_show (widget);
//  gtk_table_attach(GTK_TABLE(table), widget,
//		   2, 21, 10, 18,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

main_window_width_spinner = create_spinner(gman->prefs.main_window_width, 0, 1000);
gtk_box_pack_start(GTK_BOX(column[1]), main_window_width_spinner, FALSE, FALSE, 0); // expand, fill, padding


//  gtk_widget_show (main_window_width_spinner);
//  gtk_table_attach(GTK_TABLE(table),
//		   main_window_width_spinner,
//		   21, 35, 10, 18,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

main_window_height_spinner =  create_spinner(gman->prefs.main_window_height, 0, 1000);
gtk_box_pack_start(GTK_BOX(column[2]), main_window_height_spinner, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_widget_show (main_window_height_spinner);
//  gtk_table_attach(GTK_TABLE(table),
//		   main_window_height_spinner,
//		   40, 55, 10, 18,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

widget = gtk_button_new_with_label ("DEFAULT");
gtk_widget_show (widget);
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK ( default_main_window_size ),  this);
gtk_box_pack_start(GTK_BOX(column[3]), widget, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_table_attach(GTK_TABLE(table), widget,
//		   60, 88, 10, 18,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding
  
//========
// MANPAGE
//========
widget = gtk_label_new("Manpage");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .5);
gtk_box_pack_start(GTK_BOX(column[0]), widget, FALSE, FALSE, 0); // expand, fill, padding


//  gtk_widget_show (widget);
//  gtk_table_attach(GTK_TABLE(table), widget,
//		   2, 21, 18, 26,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

                                                                        
manpage_width_spinner = create_spinner(gman->prefs.manpage_width, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[1]), manpage_width_spinner, FALSE, FALSE, 0); // expand, fill, padding
 
//  gtk_widget_show (manpage_width_spinner);
//  gtk_table_attach(GTK_TABLE(table), manpage_width_spinner,
//		   21, 35, 18, 26,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);   // xpadding, ypadding

manpage_height_spinner = create_spinner(gman->prefs.manpage_height, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[2]), manpage_height_spinner, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_widget_show (manpage_height_spinner);
//  gtk_table_attach(GTK_TABLE(table), manpage_height_spinner,
//		   40, 55, 18, 26,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

widget = gtk_button_new_with_label ("DEFAULT");
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK ( default_manpage_size ),  this);
gtk_box_pack_start(GTK_BOX(column[3]), widget, FALSE, FALSE, 0); // expand, fill, padding


//  gtk_table_attach(GTK_TABLE(table), widget,
//		   60, 88, 18, 26,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding
//  gtk_widget_show (widget);
  
  
  

  
//==================
// MANDIR (Section)
//==================
widget = gtk_label_new("Section");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .5);
gtk_box_pack_start(GTK_BOX(column[0]), widget, FALSE, FALSE, 0); // expand, fill, padding


//  gtk_widget_show (widget);
//  gtk_table_attach(GTK_TABLE(table), widget,
//		   2, 21, 26, 34,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding
                                                                      
mandir_width_spinner = create_spinner(gman->prefs.manpage_width, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[1]), mandir_width_spinner, FALSE, FALSE, 0); // expand, fill, padding
  
//  gtk_widget_show(mandir_width_spinner);
//  gtk_table_attach(GTK_TABLE(table), mandir_width_spinner,
//		   21, 35, 26, 34,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);   // xpadding, ypadding


mandir_height_spinner = create_spinner(gman->prefs.mandir_height, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[2]), mandir_height_spinner, FALSE, FALSE, 0); // expand, fill, padding
  
//  gtk_widget_show (mandir_height_spinner);
//  gtk_table_attach(GTK_TABLE(table), mandir_height_spinner,
//		   40, 55, 26, 34,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding
  
widget = gtk_button_new_with_label ("DEFAULT");
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK ( default_mandir_size ),  this);
gtk_box_pack_start(GTK_BOX(column[3]), widget, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_table_attach(GTK_TABLE(table), widget,
//		   60, 88, 26, 34,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

  
//=====
// EDIT
//=====
widget = gtk_label_new("Edit");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .5);
gtk_box_pack_start(GTK_BOX(column[0]), widget, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_widget_show (widget);
//  gtk_table_attach(GTK_TABLE(table), widget,
//		   2, 21, 34, 42,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

edit_width_spinner = create_spinner(gman->prefs.edit_width, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[1]), edit_width_spinner, FALSE, FALSE, 0); // expand, fill, padding
  
//  gtk_widget_show (edit_width_spinner);
//  gtk_table_attach(GTK_TABLE(table), edit_width_spinner,
//		   21, 35, 34, 42,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding


edit_height_spinner = create_spinner(gman->prefs.edit_height, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[2]), edit_height_spinner, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_widget_show (edit_height_spinner);
//  gtk_table_attach(GTK_TABLE(table), edit_height_spinner,
//		   40, 55, 34, 42,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

widget = gtk_button_new_with_label ("DEFAULT");
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK ( default_edit_size ),  this);
gtk_box_pack_start(GTK_BOX(column[3]), widget, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_table_attach(GTK_TABLE(table), widget,
//		   60, 88, 34, 42,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

//========
// APROPOS
//========
widget = gtk_label_new("Apropos");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .5);
gtk_box_pack_start(GTK_BOX(column[0]), widget, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_widget_show (widget);
//  gtk_table_attach(GTK_TABLE(table), widget,
//		   2, 21, 42, 50,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

apropos_width_spinner = create_spinner(gman->prefs.apropos_width, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[1]), apropos_width_spinner, FALSE, FALSE, 0); // expand, fill, padding
  
//  gtk_widget_show (apropos_width_spinner);
//  gtk_table_attach(GTK_TABLE(table), apropos_width_spinner,
//		   21, 35, 42, 50,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

apropos_height_spinner = create_spinner(gman->prefs.apropos_height, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[2]), apropos_height_spinner, FALSE, FALSE, 0); // expand, fill, padding

  
//  gtk_widget_show (apropos_height_spinner);
//  gtk_table_attach(GTK_TABLE(table), apropos_height_spinner,
//		   40, 55, 42, 50,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

widget = gtk_button_new_with_label ("DEFAULT");
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK ( default_apropos_size ),  this);
gtk_box_pack_start(GTK_BOX(column[3]), widget, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_table_attach(GTK_TABLE(table), widget,
//		   60, 88, 42, 50,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

//===========
// PRINTPREFS
//===========
widget = gtk_label_new("Printprefs");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .5);
gtk_box_pack_start(GTK_BOX(column[0]), widget, FALSE, FALSE, 0); // expand, fill, padding

//gtk_widget_show (widget);
//  gtk_table_attach(GTK_TABLE(table), widget,
//		   2, 21, 50, 58,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

printprefs_width_spinner =  create_spinner(gman->prefs.printprefs_width, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[1]), printprefs_width_spinner, FALSE, FALSE, 0); // expand, fill, padding
  
//  gtk_widget_show (printprefs_width_spinner);
//  gtk_table_attach(GTK_TABLE(table), printprefs_width_spinner,
//		   21, 35, 50, 58,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

printprefs_height_spinner =  create_spinner(gman->prefs.printprefs_height, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[2]), printprefs_height_spinner, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_widget_show (printprefs_height_spinner);
//  gtk_table_attach(GTK_TABLE(table), printprefs_height_spinner,
//		   40, 55, 50, 58,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

widget = gtk_button_new_with_label ("DEFAULT");
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK ( default_printprefs_size ),  this);
gtk_box_pack_start(GTK_BOX(column[3]), widget, FALSE, FALSE, 0); // expand, fill, padding


//  gtk_table_attach(GTK_TABLE(table), widget,
//		   60, 88, 50, 58,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

//============
// DISLAYPREFS
//============
widget = gtk_label_new("Windowsizes");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .5);
gtk_box_pack_start(GTK_BOX(column[0]), widget, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_widget_show (widget);
//  gtk_table_attach(GTK_TABLE(table), widget,
//		   2, 21, 58, 66,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

displayprefs_width_spinner = create_spinner(gman->prefs.displayprefs_width, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[1]), displayprefs_width_spinner, FALSE, FALSE, 0); // expand, fill, padding
  
//  gtk_widget_show (displayprefs_width_spinner);
//  gtk_table_attach(GTK_TABLE(table), displayprefs_width_spinner,
//		   21, 35, 58, 66,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

displayprefs_height_spinner = create_spinner(gman->prefs.displayprefs_height, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[2]), displayprefs_height_spinner, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_widget_show (displayprefs_height_spinner);
//  gtk_table_attach(GTK_TABLE(table), displayprefs_height_spinner,
//		   40, 55, 58, 66,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

widget = gtk_button_new_with_label ("DEFAULT");
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK ( default_displayprefs_size ),  this);
gtk_box_pack_start(GTK_BOX(column[3]), widget, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_table_attach(GTK_TABLE(table), widget,
//		   60, 88, 58, 66,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding


//============
// EDITPREFS
//============

widget = gtk_label_new("Editorprefs");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .5);
gtk_box_pack_start(GTK_BOX(column[0]), widget, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_widget_show (widget);
//  gtk_table_attach(GTK_TABLE(table), widget,
//		   2, 21, 66, 74,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

editprefs_width_spinner = create_spinner(gman->prefs.editprefs_width, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[1]), editprefs_width_spinner, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_widget_show (editprefs_width_spinner);
//  gtk_table_attach(GTK_TABLE(table), editprefs_width_spinner,
//		   21, 35, 66, 74,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

editprefs_height_spinner = create_spinner(gman->prefs.editprefs_height, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[2]), editprefs_height_spinner, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_widget_show (editprefs_height_spinner);
//  gtk_table_attach(GTK_TABLE(table), editprefs_height_spinner,
//		   40, 55, 66, 74,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

widget = gtk_button_new_with_label ("DEFAULT");
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK ( default_ediprefs_size ),  this);
gtk_box_pack_start(GTK_BOX(column[3]), widget, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_table_attach(GTK_TABLE(table), widget,
//		   60, 88, 66, 74,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding


//======
// ABOUT
//======
widget = gtk_label_new("About");
gtk_misc_set_alignment(GTK_MISC (widget), 0, .5);
gtk_box_pack_start(GTK_BOX(column[0]), widget, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_widget_show (widget);
//  gtk_table_attach(GTK_TABLE(table), widget,
//		   2, 21, 74, 82,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

about_width_spinner = create_spinner(gman->prefs.about_width, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[1]), about_width_spinner, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_widget_show (about_width_spinner);
//  gtk_table_attach(GTK_TABLE(table), about_width_spinner,
//		   21, 35, 74, 82,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

about_height_spinner = create_spinner(gman->prefs.about_height, 0, 10000);
gtk_box_pack_start(GTK_BOX(column[2]), about_height_spinner, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_widget_show (about_height_spinner);
//  gtk_table_attach(GTK_TABLE(table), about_height_spinner,
//		   40, 55, 74, 82,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

widget = gtk_button_new_with_label ("DEFAULT");
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK ( default_about_size ),  this);
gtk_box_pack_start(GTK_BOX(column[3]), widget, FALSE, FALSE, 0); // expand, fill, padding

//  gtk_table_attach(GTK_TABLE(table), widget,
//		   60, 88, 74, 82,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding


hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
gtk_box_set_homogeneous(GTK_BOX(hbox), FALSE);
gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0); // expand, fill, padding

widget = gtk_button_new_with_label ("SAVE");
//gtk_widget_show (widget);
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK (do_save),  this);
gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 0); // expand, fill, padding

//  gtk_table_attach(GTK_TABLE(table), widget,
//		   2, 20, 90, 98,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

widget = gtk_button_new_with_label ("USE (no save)");
//gtk_widget_show (widget);
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK (do_use),  this);
gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 0); // expand, fill, padding

//  gtk_table_attach(GTK_TABLE(table), widget,
//		   25, 55, 90, 98,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding

widget = gtk_button_new_with_label ("CANCEL");
//gtk_widget_show (widget);
g_signal_connect (G_OBJECT (widget), "clicked",
		    G_CALLBACK ( quit_displayprefs ),  this);
gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 0); // expand, fill, padding

//  gtk_table_attach(GTK_TABLE(table), widget,
//		   60, 88, 90, 98,
//		   // left attach, right attach, top attach, bottom attach
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // xoptions
//		   (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL),  // yoptions
//		   0, 0);                               // xpadding, ypadding


//=================================================
// right arrow and they still default!
// so fix this in the snoop function to prevent it
//=================================================

gtk_widget_set_can_default(GTK_WIDGET(widget), TRUE); //GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);


// --- Make the button the default button
gtk_widget_grab_default(widget);

gtk_widget_show_all(window);
}
